/*
 * Created on 31-Oct-2003
 *
 * This class allows OrCompoundDiagrams and AndCompoundDiagrams 
 * to share code for adding, removing components etc.
 */
package rwd.abstractsyntax.spiderdiagram;

import java.util.Vector;

public abstract class NaryCompoundDiagram extends CompoundDiagram {

	public NaryCompoundDiagram(Vector components, String name) {
		super(components, name);
        flatten();
	}

	public boolean equals(Diagram d) {
		//assume subclasses did type-checking - her we'll compare
		// arraylists of components
		CompoundDiagram otherd = (CompoundDiagram) d;
		Vector diags1 = getChildren();
		Vector diags2 = otherd.getChildren();
		if (diags1.size() == diags2.size()) { // compare children
			boolean[] foundMatch = new boolean[diags1.size()];
			for (int i = 0; i < foundMatch.length; i++) {
				foundMatch[i] = false;
			}
			for (int i = 0; i < diags1.size(); i++) {
				Diagram d1 = (Diagram) diags1.get(i);
				d2Search : for (int j = 0; j < diags2.size(); j++) {
					if (!foundMatch[j]) {
						Diagram d2 = (Diagram) diags2.get(j);
						if (d1.equals(d2)) {
							foundMatch[j] = true;
							break d2Search;
						}
					}
				}
			}
			//see whether all foundMatch are true
			for (int i = 0; i < foundMatch.length; i++) {
				if (!foundMatch[i]) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	}
    /**
     * Method flatten.
     * Only allow NaryDiagrams whose children number more than
     * one. 
     * This method is OK for And and Or Compound Diagrams.
     * The idea is that And(P, Or(Q), And(R)) becomes And(P,Q,R).
     */
    public void flatten() { 
        // go through children and check that there aren't
        // singleton ands/ors
        for (int i = 0; i < diagrams.size(); i++) {
            Diagram d = (Diagram) (diagrams.get(i));
            //System.out.println("in flatten, d is "+d);
            if (d instanceof NaryCompoundDiagram) {
                CompoundDiagram cd = (CompoundDiagram) (d);
                if (cd.getChildren().size() == 1) {
                    diagrams.remove(d);
                    diagrams.add(cd.getChildren().get(0));
                }
            }
        }
    }
    
    public boolean containsNegationOf(Diagram part){
        for(int i=0; i<diagrams.size(); i++){
            Diagram child = (Diagram)diagrams.get(i);
            if(child.containsNegationOf(part)){
                return true;
            }
        }
        return false;
    }
} 
