package rwd.concretesyntax.eulerdiagram;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Point2D;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.DiagramItem;
import rwd.gui.editor.Handle;
import rwd.metrics.AreaPolygonAdapter;

/**
 * Note: this class does not seem to have any clients anywhere in the RWD editor.
 * Nevertheless, I have updated it to correspond to the spec of DiagramItem.
 * Maybe it should be removed?
 * 
 * @author RWD,Richard Bosworth
 * 
 * @version 6.7.05   14:25
 */
public class ContourPolygon
	extends DiagramItem 
	implements java.io.Serializable, Polygonal 
{
	public ContourPolygon(Contour contour, Polygon p) {
		super.setLabel(contour.toString());
		this.contour = contour;
		this.poly = p;
		_centerX = median(p.xpoints, p.npoints);
		_centerY = median(p.ypoints, p.npoints);        
        int[] pos = AreaPolygonAdapter.getRightmostPoint(poly);
        setLabelPosition(new Point2D.Float(pos[0], pos[1]));
	}
	
	public void replaceContour(Contour newc) {
		contour = newc;
	}
	
	public Shape getShape(boolean smooth) {
		return getPolygon();
	}
	public Shape getShape() {
	    return getShape(false);
	}
	
	public Polygon getPolygon() {
		return poly;
	}
	public void setPolygon(Polygon p) {
		poly = p;
	}
	
	public Contour getContour() {
		return contour;
	}
	public String getType() {
		return "contour";
	}
	
	public float getX() {
		return _centerX;
	}

	public float getY() {
		return _centerY;
	}

	public void translateTo(float x, float y) {
		poly.invalidate();
		poly.translate((int) (x - _centerX), (int) (y - _centerY));
	}

	public java.util.Set getHandles() {
		java.util.Set res = new java.util.LinkedHashSet();
		for(int i=0; i<poly.npoints; i++) {
			Handle h = new Handle(poly.xpoints[i], poly.ypoints[i], this);
			res.add(h);
		}
		return res;
	}
    
	public void drawLabel(Graphics g) {
		Point2D point = getLabelPosition();
		int msgX = (int)point.getX() + 4;
		int msgY = (int)point.getY();
		String text = getLabel();
		// Draw text
		if (_font == null) _font = g.getFont();
		Font backup = g.getFont();
		// PJR changed to allow fading
		g.setColor(_foreColor);
		g.setFont(_font);
		g.drawString(text, msgX, msgY);
		g.setFont(backup);
	}

	public void autoPlaceLabel() {
		int[] xy = AreaPolygonAdapter.getRightmostPoint(poly);
		setLabelPosition(new Point2D.Float(xy[0], xy[1]));
	}

	/** 
	 * @return string representing this ContourPolygon
	 */
	/*
	 * test: once thru
	 */
	public String toString() { 
		return "" + contour + "_" + stringOf(getPolygon());
	}

    /**
     * Note: unlike clone(), this returns a DiagramItem, and does not throw
     * an exception.
     * @return a deep clone of this diagram item
     */
    /*
     * test: once thru
     */
	public DiagramItem getClone() {
		Contour newC = contour.getClone();
		Polygon newP = new Polygon(poly.xpoints, poly.ypoints, poly.npoints);
		ContourPolygon res = new ContourPolygon(newC, newP);
		// Set attributes
		res.setAttributes(this);
		return res;
	}
	
	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test: once thru
	 */
	public String id() {
		return "contour polygon " + getLabel();
	}
	
	//
	// Local properties
	//
	protected Contour contour;
	protected Polygon poly;
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Test of toString, hashCode, id");
    	Contour a = Contour.getContour("a");
    	pr("contour a = " + a);
    	Polygon p = new Polygon();
    	p.addPoint(1,2);
    	pr("p="+stringOf(p));    	
    	ContourPolygon cp = new ContourPolygon(a,p);
    	pr("cp="+cp);
    	pr("cp.hashCode()="+cp.hashCode());
    	pr("cp.id()="+cp.id());
    	pg();
    	
    	{
    		hd("Test of getClone, equals");
    		ContourPolygon cpc = (ContourPolygon) cp.getClone();
    		pr("cpc="+cpc);
    		pr("cpc.hashCode()="+cpc.hashCode());
       		pr("cpc.equals(cp)="+cpc.equals(cp));
       		pr("cp.equals(cpc)="+cp.equals(cpc));
       		pg();
       		
       		pr("cp.equals(null)="+cp.equals(null));
       		pr("cp.equals(p)="+cp.equals(p));
    		pg();
    	}
    	
    }
    */
}
