package rwd.concretesyntax.eulerdiagram;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Point2D;

import rwd.concretesyntax.DiagramItem;
import rwd.gui.editor.ContourHandle;
import rwd.gui.editor.Handle;
/**
 * 
 * @author RWD,Richard Bosworth
 * 
 * @version 5.8.05   09:07
 */
public class DecorationBox 
	extends DiagramItem 
	implements java.io.Serializable, Polygonal 
{    
	
	// constructor
	// -----------
	
	/**
	 * 
	 * @param label label of this decoration box (must be non-empty)
	 * @param readingTree reading tree of this decoration box
	 * @param p polygon representing the shape of this decoration box.
	 *          The shape must be a quadrilateral, whose corner coordinates are given
	 *          in the order: NW, NE, SE, SW.
	 */
	/*
	 * test: once thru
	 */
    public DecorationBox(String label, String readingTree, Polygon p) {
        _polygon = p;
        _centerX = median(p.xpoints, p.npoints);
		_centerY = median(p.ypoints, p.npoints);
        setLabel(label);
		Point2D pt = new Point2D.Float(_polygon.xpoints[getTopRight()],
				                       _polygon.ypoints[getTopRight()]);
		setReadingForestPosition(pt);
        setReadingForest(readingTree);
    }

    
    // attributes
    // ----------

	/**
	 * @return deep clone of this item
	 */
	/*
	 * test: once thru
	 */
	public DiagramItem getClone() {
		DecorationBox result = new DecorationBox(
			new String(_label),
			new String(_readingForest),
			new Polygon(
				_polygon.xpoints,
				_polygon.ypoints,
				_polygon.npoints));
		result.setAttributes(this);
		return result;                
	}
    
	public java.util.Set getHandles() {
		java.util.Set res = new java.util.LinkedHashSet();
		for(int i=0; i<_polygon.npoints; i++) {
			Handle h = new ContourHandle(_polygon.xpoints[i], _polygon.ypoints[i], i, this);
			res.add(h);
		}
		return res;
	}

    public Polygon getPolygon() {
        return _polygon;
    }
    
	public String getReadingForest() {
		return _readingForest;
	}
	
	/**
	 * 
	 * @return position of NW corner of reading forest string
	 */
	/*
	 * test: once thru
	 */
    public Point2D getReadingForestPosition() {
		return new Point2D.Double(this._readingForestX, 
				                  this._readingForestY);
	}
    
    public Shape getShape(boolean smooth) {
        return getPolygon();
    }

	public Shape getShape() {
		return getShape(false);
	}

	public String getType() {
		return "box";
	}

	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test: once thru
	 */
	public String id() {
		return "decoration box " + getLabel();
	}
	
	/**
	 * @return a string representation of this decoration box
	 */
	/*
	 * tes: once thru
	 */
	public String toString() {
		return _label + "_" + hashCode() + "_" + stringOf(this.getPolygon());    
	}

	
    // operations
    // ----------
    
    /**
     * post: label has been placed in appropriate position for this item
     */
	/*
	 * test: once thru
	 */
	public void autoPlaceLabel() {
		_labelX = (_polygon.xpoints[getTopLeft()] + 
				   _polygon.xpoints[getTopRight()])/2;
		_labelY = _polygon.ypoints[getTopLeft()] + LABEL_Y_OFFSET;
	}

	public void draw(Graphics2D g, boolean smoothed) {
		Polygon polygon = getPolygon();
		g.setColor(getForeColor());
		BasicStroke stroke = new BasicStroke(
				getThickness(),
				BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_ROUND);
		g.setStroke(stroke);
		g.draw(polygon);
		// Draw label
		drawLabel(g);
		// Draw reading tree
		drawReadingTree(g);
	}

//  public void drawLabel(Graphics g) {
//	int msgX = (int)_polygon.xpoints[0];
//	int msgY = (int)_polygon.ypoints[0];
//	String text = getLabel();
//	// Draw text
//	if (_font == null) _font = g.getFont();
//	Font backup = g.getFont();
//	g.setColor(Color.BLACK);
//	g.setFont(_font);
//	g.drawString(text, msgX, msgY);
//	g.setFont(backup);
//}

	/*
	 * test: once thru
	 */
	public void drawReadingTree(Graphics2D g) {
		if (_font == null) _font = g.getFont();
		// Draw text
		Font backup = g.getFont();
		g.setFont(_font);
		String s = DiagramItem.replaceUnicodeSequences(_readingForest);

		// Get measures needed to set the location of the message
		FontMetrics fm = g.getFontMetrics();
		int msgWidth = computeWidth(s, fm);
		int msgHeight = computeHeight(s, fm);
		int lineHeight = computeHeight(fm);
		int ascent = computeAscent(fm);
		int descent = computeDescent(fm);
		
		// Use the string width to find the starting point
		Point2D rfp = this.getReadingForestPosition();
		int msgX = (int)rfp.getX() + 100;
		int msgY = (int)rfp.getY() + ascent;

		// Split text into lines
		String lines[] = s.split("\n");
		// Print each line
		for(int i=0; i<lines.length; i++) {
			g.drawString(lines[i], msgX, msgY);
			msgY += lineHeight; 
		}
		g.setFont(backup);		
	}

	/**
	 * 
	 * @param point id of corner point to be moved
	 * @param _previousX previous x-coord of point
	 * @param _currentX new x-coord of point
	 * @param _previousY previous y-coord of point
	 * @param _currentY new y-coord of point
	 */
	/*
	 * test:
	 *   moving all four corners
	 */
	public void resize(int point, float _previousX, float _currentX, float _previousY, float _currentY) {
		// Set coordinates for point
		float dx = _currentX - _previousX;
		float dy = _currentY - _previousY;
		_polygon.xpoints[point] += dx;
		_polygon.ypoints[point] += dy;
		// Top left
		if (point == getTopLeft()) {
			_polygon.xpoints[getBottomLeft()] = _polygon.xpoints[point];
			_polygon.ypoints[getTopRight()] = _polygon.ypoints[point];
			// reading tree follows top right corner
			translateForestTo(0, dy);
		// Top right
		} else if (point == getTopRight()) {
			_polygon.xpoints[getBottomRight()] = _polygon.xpoints[point];
			_polygon.ypoints[getTopLeft()] = _polygon.ypoints[point];
			// reading tree follows top right corner
			translateForestTo(dx, dy);
		// Bottom right
		} else if (point == getBottomRight()) {
			_polygon.xpoints[getTopRight()] = _polygon.xpoints[point];
			_polygon.ypoints[getBottomLeft()] = _polygon.ypoints[point];
			// reading tree follows top right corner
			translateForestTo(dx, 0);
		// Bottom left
		} else if (point == getBottomLeft()) {
			_polygon.xpoints[getTopLeft()] = _polygon.xpoints[point];
			_polygon.ypoints[getBottomRight()] = _polygon.ypoints[point];
		}
		// label follows middle of top
		autoPlaceLabel();
	}

	public void setReadingForest(String readingTree) {
		this._readingForest = readingTree;
	}

    /**
     * 
     * @param point position of NW corner of reading forest string
     */
    /*
     * test: once thru
     */
	public void setReadingForestPosition(Point2D point) {
		this._readingForestX = point.getX();
		this._readingForestY = point.getY();
	}

	/**
	 * Post: box, its label and its reading tree have been translated as required
	 * @param dx,dy increments to decoration box position
	 */
	/*
	 * test: once thru
	 */
	public void translateTo(float dx, float dy) {
		// Translate polygon
		_polygon.invalidate();
		_polygon.translate((int)dx, (int)dy);
		_centerX += dx;
		_centerY += dy;
		_labelX += dx;
		_labelY += dy;
		_readingForestX += dx;
		_readingForestY += dy;
	}

	
    // local properties
    // ----------------

	private int computeWidth(String text, FontMetrics fm) {
		// Split text into lines
		String lines[] = text.split("\n");
		// Compute maximum width
		int msgWidth = 0;
		for(int i=0; i<lines.length; i++) {
			if (fm != null) {
				int width = fm.stringWidth(lines[i]);
				if (width > msgWidth) {
					msgWidth = width;
				}
			}
		}
		return msgWidth;
	}

	private int computeHeight(FontMetrics fm) {
		int msgHeight = 0;
		if (fm != null)
			msgHeight = fm.getHeight();
		return msgHeight;
	}

	private int computeLeading(FontMetrics fm) {
		return fm.getLeading();
	}

	private int computeHeight(String text, FontMetrics fm) {
		// Split text into lines
		String lines[] = text.split("\n");
		int msgHeight = 0;
		if (fm != null) {
			msgHeight = fm.getHeight()*lines.length;
		}
		return msgHeight;
	}

	private int computeAscent(FontMetrics fm) {
		int ascent = 0;
		if (fm != null)
			ascent = fm.getMaxAscent();
		return ascent;
	}

	private int computeDescent(FontMetrics fm) {
		int descent = 0;
		if (fm != null) {
			descent = fm.getMaxDescent();
		}
		return descent;
	}

	protected void translateForestTo(float dx, float dy) {
		_readingForestX += dx;
		_readingForestY += dy;
	}
	
	
	protected int getTopLeft() {
		return 0;
	}
	protected int getTopRight() {
		return 1;
	}
	protected int getBottomRight() {
		return 2;
	}
	protected int getBottomLeft() {
		return 3;
	}

	private static final int LABEL_Y_OFFSET = -40;

    protected Polygon _polygon;
    protected String _readingForest;
    protected double _readingForestX;
    protected double _readingForestY;
    
    /*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Test of hashCode, toString, id");
    	Polygon p = new Polygon();
    	p.addPoint(10,10);
    	p.addPoint(20,10);
    	p.addPoint(20,0);
    	p.addPoint(10,0);
    	pr("p="+stringOf(p));
    	DecorationBox db = new DecorationBox("d","",p);
    	pr("db="+db);
    	pr("db.id()="+db.id());
    	pg();
    	{
	    	hd("Tests of equals");
	    	pr("db.equals(null)="+db.equals(null));
	    	pr("db.equals(p)="+db.equals(p));
	    	pg();
	    	
	    	DecorationBox dbc = (DecorationBox) db.getClone();
	    	pr("dbc="+dbc);
	    	pr("db.equals(dbc)="+db.equals(dbc));
	    	pg();
	    	
    	}
    }
    */
}
