package rwd.concretesyntax.spiderdiagram;

//import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.*;

//import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
//import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.utilities.AreaRWD;

/**
 * A concrete spider is a derived concept at the concrete level.
 * The feet and legs of the spiders of a concrete diagram 
 * (along with the strands and ties) are held in the ConcreteSpiderFeetAndLegsSet.
 * As the user manipulates these, concrete spiders are formed, modified and 
 * dissolved.
 *   
 * @author  op,rnb
 * @version 12.8.05   11:47
 */
public class ConcreteSpider 
	extends DiagramItem 
	implements java.io.Serializable 
{    
	protected static final Random RAND = new Random();

	// constructor
	// -----------
	
    public ConcreteSpider() {  
    }
    
    // Feet
	protected Set feet = new LinkedHashSet();
    public void addFoot(ConcreteSpiderFoot foot) {
        feet.add(foot);
    }
	public Set getFeet() {
		return feet;
	}
    
	// Legs        
	protected Set legs = new LinkedHashSet();
    public void addLeg(ConcreteSpiderLeg leg) {
        legs.add(leg);
        feet.add(leg.getSource());
        feet.add(leg.getTarget());
    }
    public Set getLegs() {
        return legs;
    }
	public void setLegs(Set legs) {
		this.legs = legs;
		this.feet.clear();
		Iterator i = legs.iterator();
		while (i.hasNext()) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)i.next();
			feet.add(leg.getSource());
			feet.add(leg.getTarget());
		}
    }

	/**
	 * @param cud concrete unitary diagram containing this spider
	 * @return set of concrete zones representing the habitat of this spider
	 */
	public Set getHabitat(ConcreteUnitaryDiagram cud) {
		Set habitat = new LinkedHashSet();
		Iterator i = getFeet().iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			ConcreteZone cz = cud.getConcreteZone(foot);
			habitat.add(cz);
		}
		return habitat;
	}

	public boolean isUniversal() {
		Iterator i = feet.iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			if (foot.isUniversal()) {
				return true;
			}
		}
		return false;
	}

	public boolean isExistential() {
		Iterator i = feet.iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			if (!foot.isExistential()) {
				return false;
			}
		}
		return true;
	}
    
    // Label
	// -----
	
	
	/**
	 * @return number of labels attached to the feet of this spider
	 */
	/*
	 * test: 
	 *   no labels
	 *   two labels
	 */
    public int getNumberOfLabels() {
    	int n = 0;
		Iterator i = feet.iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			String label = foot.getLabel();
			if (label != null && label.length() != 0)
				n++;
		}
		return n;    
    }
    
    public String getLabel() {
		Iterator i = feet.iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			String label = foot.getLabel();
			if (label != null && label.length() != 0)
				return label;
		}
		return "";    
    }
    
    // Abstract spider
    public Spider getAbstractSpider(ConcreteUnitaryDiagram cud) {
    	// Compute habitat
    	Vector habitat = new Vector();
    	// For each feet compute zone
    	Iterator i = feet.iterator();
    	while (i.hasNext()) {
    		ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
    		ConcreteZone cz = cud.getConcreteZone(foot);
    		if (cz != null) {
				Zone az = cz.getAbstractZone();
				if (az != null) {
					habitat.add(az);
				}
    		}
    	}
    	// Get abstract spider
    	Spider sp = Spider.getAbstractSpider(getLabel(), habitat);
    	if (isExistential()) {
    		sp.kind = Spider.EXISTENTIAL;
    	} else if (isUniversal()) {
    		sp.kind = Spider.UNIVERSAL;
    	}
    	return sp;
    }

	public boolean inDomain(ConcreteUnitaryDiagram cud, ConcreteContour c) {
		Set habitat = getHabitat(cud);
		Iterator i = habitat.iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone)i.next();
			if (cz.getContainingPolygonals().contains(c)) {
				return true;
			}
		}
		return false;
	}

	public boolean inDomain(ConcreteUnitaryDiagram cud, ConcreteSpiderFoot foot) {
		ConcreteZone cz = cud.getConcreteZone(foot);
		Set habitat = getHabitat(cud);
		if (habitat.contains(cz)) {
			return true;
		} else {
			return false;
		}
	}

    /**
     * @return string representation of this spider
     */
    public String toString() {
		return "ConcreteSpider{" +
					"label:"+getLabel()+
					", feet:"+getFeet()+
					", legs:"+getLegs()+"}";
    }
    
	/* (non-Javadoc)
	 * @see rwd.concretesyntax.DiagramItem#getShape()
	 */
	public Shape getShape() {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see rwd.concretesyntax.DiagramItem#getShape(boolean)
	 */
	public Shape getShape(boolean smooth) {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see rwd.concretesyntax.DiagramItem#getHandles()
	 */
	public Set getHandles() {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see rwd.concretesyntax.DiagramItem#translateTo(float, float)
	 */
	public void translateTo(float dx, float dy) {
		// TODO Auto-generated method stub
		
	}
	
    /**
     * Note: unlike clone(), this returns a DiagramItem, and does not throw
     * an exception.
     * @return a deep clone of this diagram item
     */
    /*
     * test:
     *   spider with one foot
     *   spider with three feet, two legs
     */
	public DiagramItem getClone() {
		ConcreteSpider cs = new ConcreteSpider();
		cs.setAttributes(this);
		
		// map old feet to new feet
		Map feetToClones = new HashMap();
		Set oldFeet = this.getFeet();
		Iterator fit = oldFeet.iterator();
		while (fit.hasNext()) {
			ConcreteSpiderFoot csf = (ConcreteSpiderFoot) fit.next();
			feetToClones.put(csf,csf.getClone());
		}
		// add cloned feet to new spider
		Collection clones = feetToClones.values();
		Iterator cit = clones.iterator();
		while (cit.hasNext())
			cs.addFoot((ConcreteSpiderFoot)cit.next());
		
		// add new legs to new feet from old legs to old feet
		Set oldLegs = this.getLegs();
		Iterator lit = oldLegs.iterator();
		while (lit.hasNext()) {
			ConcreteSpiderLeg oldLeg = (ConcreteSpiderLeg) lit.next();
			ConcreteSpiderFoot newSource = 
				(ConcreteSpiderFoot) feetToClones.get(oldLeg.getSource());
			ConcreteSpiderFoot newTarget = 
				(ConcreteSpiderFoot) feetToClones.get(oldLeg.getTarget());
			ConcreteSpiderLeg newLeg = 
				new ConcreteSpiderLeg(oldLeg.getLabel(),newSource,newTarget,oldLeg.getType());
			cs.addLeg(newLeg);
		}
		return cs;
	}
	
	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test:
	 *   spider with label
	 */
	public String id() {
		return "spider " + this.getLabel();
	}
	
	// operations
	// ----------
	
	/**
	 * post: foot and any associated leg has been added to concrete spider
	 * @param cud concrete unitary diagram in which this foot is to be added
	 * @param cz concrete zone in which this foot is to be added
	 */
	/*
	 * test:
	 *   spider has no legs
	 *   spider has one or more legs
	 */
	public void addExistentialFoot(ConcreteUnitaryDiagram cud, ConcreteZone cz) {
		
		// create foot
		ConcreteSpiderFoot ncsf  = 
			new ConcreteSpiderFoot("",ConcreteSpiderFoot.NORMAL);
		
		ConcreteSpiderFeetAndLegsSet csfals = cud.getFeetAndLegsSet(); 
		placeFoot(ncsf,cz);
		
		// create leg to nearest foot, if any
        ConcreteSpiderFoot nearest = getNearestFoot(ncsf);
        if (nearest != null) {
        	ConcreteSpiderLeg ncsl = 
        		new ConcreteSpiderLeg("",ncsf,nearest,ConcreteSpiderLeg.NORMAL);
        	
        	// add leg to this spider
            this.addLeg(ncsl);
        	
        	// add leg to feet and legs set
            csfals.addLeg(ncsl);
        	
        }
        
		// add foot to spider
        this.addFoot(ncsf);
		
		// add foot to feet and legs set
        csfals.addFoot(ncsf);

        
	}
	
	/**
	 * post: csf has been placed inside zone cz
	 * @param csf concrete spider foot to be placed
	 * @param cz concrete zone into which csf is to be placed
	 */
	/*
	 * test:
	 *   zone has no segments
	 *     horizontal line does not intersect zone's area more than once
	 *   zone has three or more segments
	 *     horizontal line intersects zone's area more than once
	 * 
	 */
	public void placeFoot(ConcreteSpiderFoot csf, ConcreteZone cz){
        
        AreaRWD a = cz.getArea();
        Rectangle2D rect = a.getBounds2D();
        Collection segments = a.getLineSegments();
        Iterator it = segments.iterator();
        LinkedHashSet xIntersects = new LinkedHashSet();

        double y = (2 * rect.getY() + rect.getHeight()) / 2;
        y += ((rect.getHeight() / 2) * (Math.random() - 0.5));


        Line2D horizontal =
            new Line2D.Double(
                rect.getX(),
                y,
                rect.getX() + rect.getWidth(),
                y);
        it = segments.iterator();
        while(it.hasNext()) {
            Line2D line = (Line2D) it.next();
            if (line.intersectsLine(horizontal)) {
                // this is where the horizontal line enters or leaves the polygon
                // ...let's work out exactly where it intersects
                double dx = line.getX2() - line.getX1();
                double dy = line.getY2() - line.getY1();
                double xIntersect =
                    line.getX1() + dx * (y - line.getY1()) / dy;
                xIntersects.add(new Double(xIntersect));
            }
        }
        Vector xIntersectsList = new Vector(xIntersects);

        Collections.sort(xIntersectsList);

        if (xIntersectsList.size() < 2) {
        	System.out.println("ConcreteSpider.placeFoot: can't place foot");
            return;
        }

        int segmentNumber =
            RAND.nextInt(xIntersectsList.size() / 2) * 2;

        // We can pick an x-coordinate anywhere between these two values.
        double x1 =
            ((Double) xIntersectsList.get(segmentNumber)).doubleValue();
        double x2 =
            ((Double) xIntersectsList.get(segmentNumber + 1))
                .doubleValue();
                
        double x = x1 + (x2 - x1) * (Math.random() / 2 + 0.25);

        csf.setLocation((int) x, (int) y);
        
	}
	
	/*
	 * pre:  csf = concrete spider foot to which nearest foot is to be found
	 * post: returns nearest foot of this spider to csf, or null if none
	 */
	/*
	 * test:
	 *   spider with no feet
	 *   spider with two feet 
	 */
	private ConcreteSpiderFoot getNearestFoot(ConcreteSpiderFoot csf) {
		int x = csf.getX();
		int y = csf.getY();
		int bestDistSq = Integer.MAX_VALUE;
		ConcreteSpiderFoot nearest = null;
		Iterator fit = getFeet().iterator();
		while (fit.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) fit.next();
			int dx = foot.getX() - x;
			int dy = foot.getY() - y;
			int distSq = dx * dx + dy * dy;
			if (distSq < bestDistSq) {
				nearest = foot;
				bestDistSq = distSq;
			}
		}
		return nearest;
	}


	
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	ConcreteSpider cs = new ConcreteSpider();
    	pr("cs="+cs);
    	pg();
    	/*
    	{
	    	hd("tests of getNearestFoot");
	    	ConcreteSpiderFoot csf = 
	    		new ConcreteSpiderFoot("x",ConcreteSpiderFoot.NORMAL);
	    	csf.setLocation(1,2);
	    	pr("csf="+csf);
	    	pr("cs.getNearestFoot(csf)="+cs.getNearestFoot(csf));
	    	pg();
	    	
	    	ConcreteSpiderFoot csf1 = 
	    		new ConcreteSpiderFoot("csf1",ConcreteSpiderFoot.NORMAL);
	    	csf1.setLocation(2,2);
	    	cs.addFoot(csf1);
	    	ConcreteSpiderFoot csf2 = 
	    		new ConcreteSpiderFoot("csf2",ConcreteSpiderFoot.NORMAL);
	    	csf2.setLocation(3,2);
	    	cs.addFoot(csf2);
	    	pr("cs="+cs);
	    	pr("csf="+csf);
	    	pr("cs.getNearestFoot(csf)="+cs.getNearestFoot(csf));
	    	pg();
    	}
	    
    	
    	{
    		hd("Tests of placeFoot");
    		pr("null zone");
    		Polygon pd = new Polygon();
    		DecorationBox d = new DecorationBox("","",pd);
    		// build concrete zones cdz	
    		Vector cvd = new Vector();
    		cvd.add(d);
    		Vector cv = new Vector();
    		ConcreteZone cdz = new ConcreteZone(cvd,cv);
    		pr("cdz="+cdz);
    		ConcreteSpiderFoot csf = 
    			new ConcreteSpiderFoot("x",ConcreteSpiderFoot.NORMAL);
    		pr("csf="+csf);
    		pr("cs.placeFoot(csf,cdz)");
    		cs.placeFoot(csf,cdz);
    		pr("csf="+csf);
    		pg();
    		
    		pr("zone inside box d2");
    		Polygon pd2 = new Polygon();
    		pd2.addPoint(-50,50);
    		pd2.addPoint(50,50);
    		pd2.addPoint(50,-50);
    		pd2.addPoint(-50,-50);
     		DecorationBox d2 = new DecorationBox("","",pd2);
    		// build concrete zones cdz	
    		Vector cvd2 = new Vector();
    		cvd2.add(d2);
    		ConcreteZone cd2z = new ConcreteZone(cvd2,cv);
    		pr("cd2z="+cd2z);
    		csf = 
    			new ConcreteSpiderFoot("x",ConcreteSpiderFoot.NORMAL);
    		pr("csf="+csf);
    		pr("cs.placeFoot(csf,cd2z)");
    		cs.placeFoot(csf,cd2z);
    		pr("csf="+csf);
    		pg();        	

    	}
    	
    	
    	{
    		hd("Tests of addExistentialFoot");
    		//build concrete diagram with contour a
    		Polygon pd = new Polygon();
    		pd.addPoint(-50,50);
    		pd.addPoint(50,50);
    		pd.addPoint(50,-50);
    		pd.addPoint(-50,-50);
    		DecorationBox d = new DecorationBox("","",pd);
    		
    		ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(d);
    		
    		// build contour a 
    		Polygon pa = new Polygon();
    		pa.addPoint(-30,30);
    		pa.addPoint(30,30);
    		pa.addPoint(30,-30);
    		pa.addPoint(-30,-30);
    		Contour a = Contour.getContour("a");
    		ConcreteContour ca = new ConcreteContour(a,pa);
    		cud.addConcreteContour(ca);
    		pr("cud="+cud);
    		pg();
    		    		
    		// build concrete zones cdaz, cdza		
    		Vector cvd = new Vector();
    		cvd.add(d);
    		Vector cva = new Vector();
    		cva.add(ca);
    		ConcreteZone cdza = new ConcreteZone(cvd,cva);
    		pr("cdza="+cdza);
    		
    		Vector cvda = new Vector();
    		cvda.add(d);
    		cvda.add(ca);
    		Vector cv = new Vector();
    		ConcreteZone cdaz = new ConcreteZone(cvda,cv);
    		pr("cdaz="+cdaz);
    		pg();
    		
    		// build concrete spider ("x",[])
    		ConcreteSpider cs0 = new ConcreteSpider();
    		pr("cs0="+cs0);
    		pg();
    		
    		// add existential foot
    		pr("add existential foot inside contour a");
    		cs0.addExistentialFoot(cud,cdaz);
    		pr("cs0="+cs0);
    		pr("cud="+cud);    		
    		pg();
    		
    		// add existential foot
    		pr("add existential foot outside contour a");
    		cs0.addExistentialFoot(cud,cdza);
    		pr("cs0="+cs0);
    		pr("cud="+cud);
    		pg();
    		

    	}
    	
    	
    	{
    		hd("tests of getClone, hashCode, id, equals");
    		pr("cs="+cs);
    		pr("cs.hashCode()="+cs.hashCode());
    		pr("cs.id()="+cs.id());
    		ConcreteSpider csc = (ConcreteSpider) cs.getClone();
    		pr("csc="+csc);
    		pr("csc.hashCode()="+csc.hashCode());
    		pr("csc.id()="+csc.id());
    		pr("cs.equals(csc)="+cs.equals(csc));
    		pr("csc.equals(cs)="+csc.equals(cs));
    		pg();
    		
    		
    	}
    	
    	{
    		hd("Test of getNumberOfLabels");
    		pr("cs="+cs);
    		pr("cs.getNumberOfLabels()="+cs.getNumberOfLabels());
    		pg();
    		
	    	ConcreteSpiderFoot csf1 = 
	    		new ConcreteSpiderFoot("csf1",ConcreteSpiderFoot.NORMAL);
	    	csf1.setLocation(2,2);
	    	cs.addFoot(csf1);
	    	ConcreteSpiderFoot csf2 = 
	    		new ConcreteSpiderFoot("csf2",ConcreteSpiderFoot.NORMAL);
	    	csf2.setLocation(3,2);
	    	cs.addFoot(csf2);
	    	pr("cs="+cs);
    		pr("cs.getNumberOfLabels()="+cs.getNumberOfLabels());
    		pg();
    		
    	}
    	*/
    }
    
}
