package rwd.concretesyntax.spiderdiagram;

import java.awt.*;
import java.awt.geom.*;
import java.util.LinkedHashSet;

import rwd.concretesyntax.DiagramItem;
import rwd.gui.editor.*;

/**
 * @author  op, rnb
 * @version 5.8.05   10:30
 */
public class ConcreteSpiderFoot 
	extends DiagramItem 
	implements java.io.Serializable 
{   
	// constructor
	// -----------
	
    public ConcreteSpiderFoot(String label, int type) {
        setLabel(label);
        setType(type);
        setBackColor(Color.black);
    }
    
    public Shape getShape(boolean smooth) {
        switch (_type) {
            case NORMAL:
                _shape = new Ellipse2D.Float(_centerX - _size, _centerY - _size, _size * 2, _size * 2);
                break;
            case UNIVERSAL:
                GeneralPath petal = new GeneralPath();
                petal.moveTo(_centerX, _centerY);
                petal.lineTo(_centerX - 14, _centerY + 40);
                petal.lineTo(_centerX + 0, _centerY + 65);
                petal.lineTo(_centerX + 14, _centerY + 40);
                petal.lineTo(_centerX, _centerY);
                
                petal.transform(AffineTransform.getRotateInstance(Math.PI / 6, _centerX, _centerY));
                
                GeneralPath star = new GeneralPath();
                for (int i = 0; i < 6; i++) {
                    star.append(petal, true);
                    petal.transform(AffineTransform.getRotateInstance(Math.PI * 2 / 6, _centerX, _centerY));
                }
                _shape = star;
                break;
			case SCHROEDINGER:
				GeneralPath cross = new GeneralPath();
				cross.moveTo(_centerX + 2*_size, _centerY);
				cross.lineTo(_centerX - 2*_size, _centerY);
				cross.moveTo(_centerX, _centerY - 2*_size);
				cross.lineTo(_centerX, _centerY + 2*_size);
				cross.append(new Ellipse2D.Float(_centerX - 2*_size, _centerY - 2*_size, _size * 4, _size * 4), true);
                
				cross.transform(AffineTransform.getRotateInstance(Math.toRadians(-135), _centerX, _centerY));
                
				_shape = cross;
				break;
        }
        return _shape;
    }

	public Shape getShape() {
		return getShape(false);
	}

	public void draw(Graphics2D g, boolean smoothed) {
		Shape shape = getShape(smoothed);
		BasicStroke stroke = new BasicStroke(
				getThickness(),
				BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_ROUND);
		g.setStroke(stroke);
		g.setColor(getBackColor());
		g.fill(shape);
		g.draw(shape);
		// Draw label
		drawLabel(g);
	}

	public Handle getHandle() {
		return new Handle(_centerX, _centerY, this);
	}
 
	public java.util.Set getHandles() {
		java.util.Set res = new java.util.LinkedHashSet();
		res.add(getHandle()); 
		return res;
	}
	
    public int getX() {
        return (int)_centerX;
    }
    
    public int getY() {
        return (int)_centerY;
    }

    public void setZone(String zone) {
    	this._zone = zone;
    }

    public void setType(int type) {
        _type = type;
        if (type == NORMAL)
        	setThickness(DiagramCanvas.DEFAULT.getLineWidth());
        else
        	setThickness(DiagramCanvas.THIN.getLineWidth());        	
    }
    
    public int getType() {
        return _type;
    }
    
    public String getZone() {
    	return _zone;
    }
    
	public boolean isUniversal() {
		return _type == ConcreteSpiderFoot.UNIVERSAL;
	}

	public boolean isExistential() {
		return _type == ConcreteSpiderFoot.NORMAL;
	}
    
    /**
     * Note: unlike clone(), this returns a DiagramItem, and does not throw
     * an exception.
     * @return a deep clone of this diagram item
     */
	public DiagramItem getClone() {
		ConcreteSpiderFoot newFoot = new ConcreteSpiderFoot(_label, _type);
		newFoot.setLocation(getX(), getY());
		newFoot.setAttributes(this);
		newFoot.observers = new LinkedHashSet(observers);
		return newFoot;
	}

    /**
     * @return a string representation of this item
     */
	public String toString() {
    	return getLabel()+"("+getX()+","+getY()+")";
    }
    
	/**
	 * @return a user-friendly description of this item
	 */
	/*
	 * test:
	 *   foot without label
	 *   foot with label
	 */
	public String id() {
		String s = "";
		switch (this.getType()) {
			default:
			case NORMAL: 
				s+= "existential";
				break;
			case UNIVERSAL: 
				s+= "universal";
				break;
			case SCHROEDINGER: 
				s+= "schroedinger";
				break;
		}
		s += " spider foot ";
		if (this.getLabel().length()>0)
			s += this.getLabel() + " ";		
		s += "at (" + getX() + "," + getY() + ")";
		return s;
	}
	

    private static final int LABEL_Y_OFFSET = -80;
    

    /*
     * test: once thru
     */
	public void autoPlaceLabel() {
		setLabelPosition(new Point2D.Float(_centerX,
				                           _centerY + LABEL_Y_OFFSET));
	}

	private int _type;
    private int _size = 30;
    private String _zone = "";
    private transient Shape _shape;
    
	static final long serialVersionUID = -4018921914396685223L;
    
	public static final int NORMAL = 1;
	public static final int UNIVERSAL = 2;
	public static final int SCHROEDINGER = 3;   
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	
    	hd("Test of tostring, hashCode, id");
    	ConcreteSpiderFoot csf = new ConcreteSpiderFoot("x",NORMAL);
    	csf.setLocation(1,2);
    	pr("csf="+csf);
    	pr("csf.hashCode()="+csf.hashCode());
    	pr("csf.id()="+csf.id());
    	pg();
    	
    	ConcreteSpiderFoot csf2 = new ConcreteSpiderFoot("",UNIVERSAL);
    	csf2.setLocation(1,2);
    	pr("csf2="+csf2);
    	pr("csf2.hashCode()="+csf2.hashCode());
    	pr("csf2.id()="+csf2.id());
    	pg();
    	
    	ConcreteSpiderFoot csf3 = new ConcreteSpiderFoot("",SCHROEDINGER);
    	csf3.setLocation(1,2);
    	pr("csf3="+csf3);
    	pr("csf3.hashCode()="+csf3.hashCode());
    	pr("csf3.id()="+csf3.id());
    	pg();
    	
    	{
    		hd("Test of clone, equals, hashCode");
    		ConcreteSpiderFoot csfc = (ConcreteSpiderFoot) csf.getClone();
        	pr("csfc="+csfc);
        	pr("csfc.hashCode()="+csfc.hashCode());
        	pr("csfc.id()="+csfc.id());        	
        	pr("csf.equals(csfc)="+csf.equals(csfc));
        	pg();
        	
    	}
    	
    }
    */
}
