package rwd.deltas;

import java.awt.geom.Line2D;
import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EdgeEdgeCrossingDelta 
	extends Delta
{
	// Constructor
	public EdgeEdgeCrossingDelta(DiagramItem item1, DiagramItem item2, ILog log) {
		this.log = log;
		args.add(item1);
		args.add(item2);
		item1.addObserver(this);
		item2.addObserver(this);
		addEdgeEnds(item1);
		addEdgeEnds(item2);
		computeValue();	
	}
	public EdgeEdgeCrossingDelta(DiagramItem item1, DiagramItem item2) {
		this(item1, item2, new OutputStreamLog(System.out));
	}
	protected void addEdgeEnds(DiagramItem item) {
		if (item instanceof ConcreteSpiderLeg) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)item;
			ConcreteSpiderFoot end1 = leg.getSource();
			ConcreteSpiderFoot end2 = leg.getTarget();
			args.add(end1);
			args.add(end2);
			end1.addObserver(this);
			end2.addObserver(this);
		} else if (item instanceof ConcreteArc) {
			ConcreteArc arc = (ConcreteArc)item;
			ConcreteArcEnd end1 = arc.getStart();
			ConcreteArcEnd end2 = arc.getEnd();
			args.add(end1);
			args.add(end2);
			end1.addObserver(this);
			end2.addObserver(this);
		}		
	}
	protected void computeValue() {
		DiagramItem item1 = (DiagramItem)args.get(0);
		DiagramItem item2 = (DiagramItem)args.get(1);
		// If they cross
		if (cross(item1, item2)) {
			value = 1.0;
		} else {
			value = 0.0;
		}
	}
	protected boolean cross(DiagramItem item1, DiagramItem item2) {
		// Make segments
		List segs1 = makeLines(item1);
		List segs2 = makeLines(item2);
		for(int i=0; i<segs1.size(); i++) {
			Line2D seg1 = (Line2D)segs1.get(i);
			for(int j=0; j<segs2.size(); j++) {
				Line2D seg2 = (Line2D)segs2.get(j);
				if (seg1.intersectsLine(seg2))
					return true;
			}
		}
		return false;
	}
	protected List makeLines(DiagramItem item) {
		if (item instanceof ConcreteSpiderLeg) {
			return makeLines((ConcreteSpiderLeg)item);
		} else if (item instanceof ConcreteArc) {
			return makeLines((ConcreteArc)item);
		} else {
			return new Vector();
		}
	}
	protected List makeLines(ConcreteSpiderLeg leg) {
		List segs = new Vector();
		ConcreteSpiderFoot s = leg.getSource();
		ConcreteSpiderFoot t = leg.getTarget();
		double x1 = s.getX();
		double y1 = s.getY();
		double x2 = t.getX();
		double y2 = t.getY();
		Line2D seg = makeLine(x1, y1, x2, y2);
		segs.add(seg);
		return segs;
	}
	protected List makeLines(ConcreteArc arc) {
		if (arc.getType() == ConcreteArc.LINE) {
			List segs = new Vector();
			ConcreteArcEnd s = arc.getStart();
			ConcreteArcEnd t = arc.getEnd();
			double x1 = s.getX();
			double y1 = s.getY();
			double x2 = t.getX();
			double y2 = t.getY();
			Line2D seg = makeLine(x1, y1, x2, y2);
			segs.add(seg);
			return segs;
		} else {
			List segs = new Vector();
			ConcreteArcEnd s = arc.getStart();
			ConcreteArcEnd t = arc.getEnd();
			double x1 = s.getX();
			double y1 = s.getY();
			double x2 = t.getX();
			double y2 = t.getY();
			Line2D seg = makeLine(x1, y1, x2, y2);
			segs.add(seg);
			return segs;
		}
	}
	protected Line2D makeLine(double coords[]) {
		return makeLine(coords[0], coords[1], coords[4], coords[5]);
	}
	protected Line2D makeLine(double x1, double y1, double x2, double y2) {
		Line2D.Double seg = new Line2D.Double();
		seg.setLine(x1, y1, x2, y2);
		return seg;
	}
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+", "+args.get(1).toString()+")";
		return res;
	}
}
