package rwd.deltas;

import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.PathIterator;
import java.util.List;

import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.Polygonal;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class NodeContourPointSeparationDelta 
	extends Delta
{
	// Constructor
	public NodeContourPointSeparationDelta(ConcreteSpiderFoot foot, ConcreteZone cz, ILog log) {
		this.log = log;
		args.add(foot);
		args.add(cz);
		foot.addObserver(this);
		cz.addObserver(this);
		// Add contours to make observer to update when a point is moved
		addContours(cz.getAllPolygonals()); 
		computeValue();
	}
	public NodeContourPointSeparationDelta(ConcreteSpiderFoot foot, ConcreteZone cz) {
		this(foot, cz, new OutputStreamLog(System.out));
	}
	protected void addContours(List contours) {
		for(int i=0; i<contours.size(); i++) {
			Polygonal p = (Polygonal)contours.get(i);
			if (p instanceof ConcreteContour) {
				ConcreteContour c = (ConcreteContour)p;
				args.add(c);
				c.addObserver(this);
			}
		} 
	}
	protected void computeValue() {
		ConcreteSpiderFoot foot = (ConcreteSpiderFoot)args.get(0);
		ConcreteZone cz = (ConcreteZone)args.get(1);
		value = distance(foot, cz);
		if (value != 0)
			value = 1/value;		
	}
	
	// Sum a distances to points that make the border of zone 
	protected double distance(ConcreteSpiderFoot sf, ConcreteZone cz) {
		double res = Double.POSITIVE_INFINITY;
		if (cz == null || sf == null) return res;
		double x1 = sf.getX();
		double y1 = sf.getY();
		// For each point from shape's border
		Shape shape = cz.getShape();
		PathIterator pi = shape.getPathIterator(null);
		double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
		double x2 = 0;
		double y2 = 0;
		while(!pi.isDone()) {
			int segmentType = pi.currentSegment(pathPoint);
			if(segmentType == PathIterator.SEG_MOVETO) {
//				System.out.println("MOVETO x "+pathPoint[0]+"y "+pathPoint[1]+" ");
				x2 = pathPoint[0];
				y2 = pathPoint[1];
				double d = distance(x1, y1, x2, y2);
				if (d < res)
					res = d;			
			} else if(segmentType == PathIterator.SEG_LINETO) {
//				System.out.println("LINETO x "+pathPoint[0]+" y "+pathPoint[1]);
				x2 = pathPoint[0];
				y2 = pathPoint[1];				
				double d = distance(x1, y1, x2, y2);
				if (d < res)
					res = d;			
			} else if (segmentType == PathIterator.SEG_CLOSE) {
//				System.out.println("CLOSE x "+pathPoint[0]+" y "+pathPoint[1]);
			}
			pi.next();
		}
		return res;
	}	 
	
	// Minimum distance to a point of an including/excluding contour
	protected double distance1(ConcreteSpiderFoot sf, ConcreteZone cz) {
		double res = Double.POSITIVE_INFINITY;
		if (cz == null || sf == null) return res;
		List cp = cz.getAllPolygonals();
		double x1 = sf.getX();
		double y1 = sf.getY();
		for(int i=0; i<cp.size(); i++) {
			Object obj = cp.get(i);
			if (obj instanceof ConcreteContour) {
				ConcreteContour c = (ConcreteContour)obj;
				Polygon poly = c.getPolygon();
				for(int j=0; j<poly.npoints; j++) {
					double x2 = poly.xpoints[j];
					double y2 = poly.ypoints[j];
					double d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
					if (d < res)
						res = d;
				}
			}
		}
		return res;
	}
	protected double distance(double x1, double y1, double x2, double y2) {
		return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	}
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+", "+args.get(1).toString()+")";
		return res;
	}
}
