package rwd.fopl.visitors;

import java.util.*;

import rwd.fopl.*;
import rwd.fopl.ast.*;

import rwd.ocl.constraintdiagram.ReadingTree;
import rwd.ocl.eulerdiagram.EulerDiagram2Uml;

/**
 * @author Octavian Patrascoiu
 *
 */
public class OclVisitor 	
	extends FoplVisitorImpl
	implements FoplVisitor
{
	public OclVisitor() {
		setNo = 0;
	}
	
	/** Visit class 'rwd.fopl.ast.Cuantifier' */
	public Object visit(rwd.fopl.ast.Cuantifier host, Object data) {
		String indent = getIndent(data);
		// Make a let expression
		String res = "\n"+indent+"let \n";
		// Add variable
		String domVar = getNewVar();
		String domValue = getDomainValue(host.getDom(), makeData(""));
		res += indent+tab+domVar+" : Set(OclAny) = "+domValue+"\n";
		res += indent+"in \n";
		// Add body
		String kind = host.getKind();
		if (kind == ReadingTree.EVERY) {
			res += indent+tab+domVar+"->forAll("+host.getVar()+":OclAny|";
		}
		else {
			res += indent+tab+domVar+"->exists("+host.getVar()+":OclAny|";
		}
		// Add condition
		Expression cond = host.getCond();
		if (cond instanceof Cuantifier) {
			String condStr = (String)cond.accept((FoplVisitor)this, makeData(indent+tab+bodyTab));
			res += condStr;
			res += indent + ")";
		} else {
			String condStr = (String)cond.accept((FoplVisitor)this, makeData(""));
			res += condStr;
			res += ")";
		}
		return res;
	}
	
	/** Visit class 'rwd.fopl.ast.BinaryExpression' */
	public Object visit(rwd.fopl.ast.BinaryExpression host, Object data) {
		String indent = getIndent(data);
		String oper = host.getOperator();
		Expression leftHost = host.getLeft();
		Expression rightHost = host.getRight();
		String left = (String)leftHost.accept((FoplVisitor)this, makeData(""));
		String right = (String)rightHost.accept((FoplVisitor)this, makeData(""));
		String res = indent;
		if (oper == ReadingTree.EQ) {
			res += left + " = " + right;
		} else if (oper == ReadingTree.NE) {
			res += left + " <> " + right;
		} else if (oper == ReadingTree.DOT) {
			// A.f -> A.allInstances()->f
			if (leftHost instanceof Type && rightHost instanceof Constant) {
				String newVar = getNewVar();
				res += left+"->collect("+newVar+":OclAny |"+newVar+"."+right+")";
			} else {
				res += left + "." + right;			
			}
		} else if (oper == ReadingTree.AND) {
			res += left + " and " + right;
		} else if (oper == ReadingTree.OR) {
			res += left + " or " + right;
		} else if (oper == ReadingTree.BELONGS) {
			res += right + "->includes(" + left +")";
		} else if (oper == ReadingTree.INCLUDED) {
			res += right + "->includesAll(" + left +")";
		} else if (oper == ReadingTree.INCLUDED_OR_EQUAL) {
			res += right + "->includesAll(" + left +")";
		} else if (oper == ReadingTree.INTERSECTION) {
			res += left + "->intersection(" + right +")";
		} else if (oper == ReadingTree.UNION) {
			res += left + "->union(" + right +")";
		} else if (oper == ReadingTree.MINUS) {
			res += left + " - " + right;
		}
		return res;
	}
	
	/** Visit class 'rwd.fopl.ast.UnaryExpression' */
	public Object visit(rwd.fopl.ast.UnaryExpression host, Object data) {
		String indent = getIndent(data);
		String oper = host.getOperator();
		String left = (String)host.getLeft().accept((FoplVisitor)this, makeData(""));
		if (oper == ReadingTree.NOT) {
			if (left.indexOf('.') == -1) {
				return indent+"(OclAny.allInstances()-"+getUMLType(left)+".allInstances())";
			} else {
				return indent+"(OclAny.allInstances()-"+left+")";
			}
		}
		return indent+oper+left;
	}
	
	/** Visit class 'rwd.fopl.ast.Constant' */
	public Object visit(rwd.fopl.ast.Constant host, Object data) {
		String indent = getIndent(data);
		String value = host.getValue();
		if (value == ReadingTree.EMPTY) {
			return indent+"Set{}"; 
		} else {
			return indent+value;
		}
	}

	/** Visit class 'rwd.fopl.ast.Type' */
	public Object visit(rwd.fopl.ast.Type host, Object data) {
		String indent = getIndent(data);
		String value = getUMLType(host.getValue());
		return indent+value+".allInstances()";
	}
	
	//
	// Compute the domain of a cuantifier
	//
	protected String getDomainValue(Expression domain, Object data) {
		String indent = getIndent(data);
		// Compute in and out contours
		List inCont = new Vector();
		List exCont = new Vector();
		// Compute initial value
		String set = indent+"OclAny.allInstances()";
		// Compute selection predicate
		String selPredStr = getSelection(domain, "x");
		// Add selection predicate
		set += "->select(x : OclAny | "+selPredStr+")";
		return set;
	}
	protected String getSelection(Expression reg, String iter) {
		if (reg instanceof Constant) {
			Constant ct = (Constant)reg;
			String c = ct.getValue();
			String cName = getUMLType(c);
			return getSelection(ct, iter);
		} else if (reg instanceof UnaryExpression) {
			UnaryExpression un = (UnaryExpression)reg;
			String operator = un.getOperator();
			if (operator == ReadingTree.NOT) {
				Expression left = un.getLeft();
				if (left instanceof Constant) {
					return oclOperator(operator)+" "+getSelection((Constant)left, iter);					
				}
			}
		} else if (reg instanceof BinaryExpression) {
			BinaryExpression bin = (BinaryExpression)reg;
			Expression left = bin.getLeft();
			Expression right = bin.getRight();
			return getSelection(left, iter)+" "+oclOperator(bin.getOperator())+" "+getSelection(right, iter);
		};
		return "";
	}
	protected String getSelection(Constant ct, String iter) {
		String c = ct.getValue();
		String cName = getUMLType(c);
		return iter+".oclIsKindOf("+cName+")";
	}
	protected static Map operMap = new LinkedHashMap();
	static {
		operMap.put(ReadingTree.AND, "and");
		operMap.put(ReadingTree.OR, "or");
		operMap.put(ReadingTree.NOT, "not");
		operMap.put(ReadingTree.INTERSECTION, "and");
		operMap.put(ReadingTree.UNION, "or");
	}
	protected String oclOperator(String oper) {
		String oclOper = (String)operMap.get(oper);
		if (oclOper == null) return "";
		else return oclOper;
	}
	protected static String getUMLType(Object c) {
		return EulerDiagram2Uml.modelName+"::"+EulerDiagram2Uml.pkgName+c.toString();
	}
	
	protected static String tab = "  ";
	protected static String bodyTab = "            ";
	protected static String getIndent(Object data) {
		if (data instanceof Map) {
			Map map = (Map)data;
			return (String)map.get("indent");
		}
		return "";
	}

	protected static Map makeData(String indent) {
		Map res = new LinkedHashMap();
		res.put("indent", indent);
		return res;
	}
	
	protected static int setNo = 0;
	protected static String getNewVar() {
		return "_s"+setNo++;
	}
	
}

