package rwd.gui.editor;

import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBox;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.gui.editor.listeners.EditDeleteListener;
import rwd.gui.editor.listeners.EditDiagramItemListener;
import rwd.gui.editor.listeners.FormatBackColorListener;
import rwd.gui.editor.listeners.FormatFontListener;
import rwd.gui.editor.listeners.FormatForeColorListener;
import rwd.gui.editor.listeners.FormatSetLabelListener;
import rwd.gui.editor.listeners.FormatThicknessListener;
import rwd.gui.editor.listeners.FormatTypeListener;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
// Context dependant menu
class EditPopupMenu 
	extends JPopupMenu
{
	/** Constructor */
	public EditPopupMenu(DiagramViewerInterface viewer) {
		super();
		init(viewer);
	}
	
	/** Init menu items */
	protected void init(DiagramViewerInterface viewer) {
		this.viewer = viewer;
		// Add menu items
		type = makeItem("Type", new FormatTypeListener(viewer));
		add(type);
		label = makeItem("Label", new FormatSetLabelListener(viewer));
		add(label);
		foreColor = makeItem("Foreground color", new FormatForeColorListener(viewer));
		add(foreColor);
		backColor = makeItem("Background color", new FormatBackColorListener(viewer));
		add(backColor);
		thickness = makeItem("Thickness", new FormatThicknessListener(viewer));
		add(thickness);
		font = makeItem("Font", new FormatFontListener(viewer));
		add(font);
		edit = makeItem("Edit", new EditDiagramItemListener(viewer));
		add(edit);
		addSeparator();
		
		delete = makeItem("Delete", new EditDeleteListener(viewer));
		add(delete);	
	}
	
	protected JMenuItem makeItem(String name, ActionListener action) {
		JMenuItem item = new JMenuItem(name);
		item.setEnabled(false);
		item.addActionListener(action);
		return item;
	}
	
	protected void validateItems() {
		// Validate the menu items
		if (viewer == null) return;
		Collection selectedItems = viewer.getDiagramCanvas().getSelectedItems();
		if (selectedItems == null || selectedItems.size() != 1) return;
		DiagramItem selectedItem = (DiagramItem)selectedItems.iterator().next();
		// Init menu fields
		// Get the user object and draw it		
		if (selectedItem != null) {
			type.setEnabled(
				selectedItem instanceof ConcreteSpiderFoot ||
				selectedItem instanceof ConcreteSpiderLeg ||	
				selectedItem instanceof ConcreteTextBox ||
				selectedItem instanceof ConcreteArc
			);
			label.setEnabled(
				selectedItem instanceof DecorationBox ||
				selectedItem instanceof ConcreteContour ||
				selectedItem instanceof ConcreteSpiderFoot ||
				selectedItem instanceof ConcreteArc
			);
			foreColor.setEnabled(
					selectedItem instanceof DecorationBox ||
					selectedItem instanceof ConcreteContour ||
					selectedItem instanceof ConcreteSpiderFoot ||
					selectedItem instanceof ConcreteSpiderLeg ||
					selectedItem instanceof ConcreteArc
			);
			backColor.setEnabled(selectedItem instanceof ConcreteZone);
			thickness.setEnabled(
					selectedItem instanceof DecorationBox ||
					selectedItem instanceof ConcreteContour ||
					selectedItem instanceof ConcreteSpiderFoot ||
					selectedItem instanceof ConcreteSpiderLeg ||
					selectedItem instanceof ConcreteArc
			);
			font.setEnabled(
				selectedItem instanceof DecorationBox ||
				selectedItem instanceof ConcreteContour |
				selectedItem instanceof ConcreteSpiderFoot ||
				selectedItem instanceof ConcreteArc ||
				selectedItem instanceof ConcreteTextBox
			);
			edit.setEnabled(
					selectedItem instanceof DecorationBox ||
					selectedItem instanceof ConcreteTextBox
				);
			delete.setEnabled(
					selectedItem instanceof DecorationBox ||
					selectedItem instanceof ConcreteContour |
					selectedItem instanceof ConcreteSpiderFoot ||
					selectedItem instanceof ConcreteSpiderLeg ||
					selectedItem instanceof ConcreteArc ||
					selectedItem instanceof ConcreteTextBox
			);
		}
	}
	
	// Local properties
	protected JMenuItem type;
	protected JMenuItem label;
	protected JMenuItem foreColor;
	protected JMenuItem backColor;
	protected JMenuItem thickness;
	protected JMenuItem font;
	protected JMenuItem edit;
	
	protected JMenuItem delete;
	
	protected DiagramViewerInterface viewer;
}
