package rwd.gui.editor.dialogs;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import rwd.gui.editor.Options;
import rwd.tableaux.semantic.MatrixPanel;

/**
 * @author Octavian Patrascoiu
 *
 */
public class FontDialog
	extends JDialog
{
	public FontDialog(JFrame parent, String title) {
		super(parent, title);
		getContentPane().setLayout(new GridBagLayout());

		//
		// Create and fill general panel
		//
		generalPanel = new MatrixPanel();
		generalPanel.setFieldSize(12);
		generalPanel.setListSize(new Dimension(150, 200));
		
		generalPanel.addLabel("Font", 0, 0);
		generalPanel.addLabel("Size", 0, 1);
		fontNameField = generalPanel.addTextField("Monospaced", 1, 0);
		sizeField = generalPanel.addTextField("200", 1, 1);
		Vector fontNames = getAvailableFonts();
		fontNamesList = generalPanel.addList(fontNames, 2, 0);
		fontNamesList.addListSelectionListener(new ListSelection(this));
//		int pos = fontNames.indexOf("Monospaced");
//		if (pos != -1)
//			fontNamesList.setSelectedIndex(pos);

		//
		// Create and fill CONFIRM panel ---
		//
		MatrixPanel confirmPanel = new MatrixPanel();
		JButton okButton = confirmPanel.addButton("OK", 0, 0);
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okAction(evt);
			}
		});
		//--- Cancel button ---
		JButton cancelButton = confirmPanel.addButton("Cancel", 0, 1);;
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelAction(evt);
			}
		});

		//--- Add TAB panel to MAIN panel ---
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.ipadx = 5;
		c.ipady = 5;
		c.weightx = 0.0;
		c.weighty = 0.0;
		c.gridx = 0;
		c.gridy = 0;
		getContentPane().add(generalPanel, c);

		//--- Add CONFIRM to MAIN panel ---		
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.CENTER;
		getContentPane().add(confirmPanel, c);
    		
		// Put frame in the middle of the screen
		pack();
		Dimension screen = getToolkit().getScreenSize();
		int width  = this.getSize().width+20;
		int height = this.getSize().height+20;
		int left = (screen.width - width)/2;
		int top = (screen.height - height)/2;
		setBounds(left, top, width, height);
	}
	
	// Get available fonts
	protected Vector getAvailableFonts() {
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		String fontNames[] = ge.getAvailableFontFamilyNames();
		Vector result = new Vector();
		for(int i=0; i<fontNames.length; i++)
			result.add(fontNames[i]);
		return result;			
	}

	// Show the dialog
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}

	// OK action
	protected int returnVal = JFileChooser.CANCEL_OPTION;
	protected void okAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.APPROVE_OPTION;
		Font input = getFont();
		if (input != null)  	
			dispose();
	}

	// Cancel action
	protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
		dispose();
	}
	
	// Get the font
	public Font getFont() {
		Font font = null;
		try {
			String fontName = fontNameField.getText();
			int size = Integer.parseInt(sizeField.getText());			
			font = new Font(fontName, Options.DEFAULT_FONT_TYPE, size);
		} catch (Exception e) {
			font = Options.DEFAULT_FONT;
		}
		return font;
	}
	public void setFontName(String fontName) {
		fontNameField.setText(fontName);
	}

	// Local properties	
	protected MatrixPanel generalPanel;
	protected JList fontNamesList;
	protected JTextField fontNameField;
	protected JTextField sizeField;
}

class ListSelection 
	implements ListSelectionListener
{
	public ListSelection(FontDialog dialog) {
		this.dialog = dialog;
	}
	
	public void valueChanged(ListSelectionEvent event) {
		JList list = (JList)event.getSource();
		String fontName = (String)list.getSelectedValue();
		dialog.setFontName(fontName);
	}
	
	protected FontDialog dialog;
}
