package rwd.gui.editor.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JTextField;

import rwd.tableaux.semantic.MatrixPanel;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class RandomCDDialog
	extends JDialog
{
	public RandomCDDialog(JFrame parent, String title, ILog log) {
		super(parent, title);	
		getContentPane().setLayout(new GridBagLayout());
		this.log = log;
		JDialog dialog = new JDialog();

		//
        // Create and fill NUMBERS panel
        //
		numberPanel = new MatrixPanel();
		// Add labels
		int line = 0;
		int column = 0;
		numberPanel.addLabel("Number of contours", line, column);
		numContoursField = numberPanel.addTextField("3", line, column+1);
		line++;
		numberPanel.addLabel("Number of spiders", line, column);
		numSpidersField = numberPanel.addTextField("2", line, column+1);
		line++;
		numberPanel.addLabel("Number of arcs", line, column);
		numArcsField = numberPanel.addTextField("2", line, column+1);
		line++;
		numberPanel.addLabel("Minimum number of points", line, column);
		numPointsField = numberPanel.addTextField("8", line, column+1);
		line++;
		numberPanel.addLabel("Equalise number of points", line, column);
		eqPointsCheck = new JCheckBox();
		eqPointsCheck.setSelected(true);
		numberPanel.add(eqPointsCheck, line, column+1);
		    
		//
        // Create and fill CONFIRM panel 
        //
        MatrixPanel confirmPanel = new MatrixPanel();
        // OK button 
		JButton okButton = confirmPanel.addButton("OK", 0, 0);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
        // Cancel button 
		JButton cancelButton = confirmPanel.addButton("Cancel", 0, 1);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });

	    // Add TAB panel to MAIN panel
		GridBagConstraints c = new GridBagConstraints(); 
	    c.gridx = 0;
	    c.gridy = 0;
		getContentPane().add(numberPanel, c);

		// Add CONFIRM to MAIN panel 		
	    c.gridx = 0;
	    c.gridy = 1;
	    c.anchor = GridBagConstraints.CENTER;
	    getContentPane().add(confirmPanel, c);
    		
        // Put frame in the middle of the screen
        pack();
        Dimension screen = getToolkit().getScreenSize();
        int width  = this.getSize().width+20;
        int height = this.getSize().height+20;
        int left = (screen.width - width)/2;
        int top = (screen.height - height)/2;
        setBounds(left, top, width, height);
	}

	// OK action 
	protected int returnVal = JFileChooser.CANCEL_OPTION;
    protected void okAction(java.awt.event.ActionEvent evt) {
    	returnVal = JFileChooser.APPROVE_OPTION;
    	boolean ok = checkNumbers();
		if (ok)  	
			dispose();
    }

	// Cancel action 
    protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
    	dispose();
    }

	// Get XMI file name -
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}
	
	// Get the numbers -
	public int getNumContours() {
		String text = numContoursField.getText(); 
		return Integer.parseInt(text);
	}
	public int getNumSpiders() {
		String text = numSpidersField.getText(); 
		return Integer.parseInt(text);
	}
	public int getNumArcs() {
		String text = numArcsField.getText(); 
		return Integer.parseInt(text);
	}
	public int getMaxSpiders() {
		int numC = this.getNumContours();
		int numS = this.getNumSpiders();
		int numZ = (int)Math.pow(2, numC);
		int maxS = (int)Math.pow(2, numZ);
		return maxS;
	}
	public int getPointNum() {
		String text = numPointsField.getText(); 
		return Integer.parseInt(text);
	}
	public boolean isEqualised() {
		return eqPointsCheck.isSelected();
	}
	
	// Check the numbers
	boolean checkNumbers() {
		int numC = this.getNumContours();
		int numS = this.getNumSpiders();
		int numA = this.getNumArcs();
		// Check contours
		if (numC < 0 || numC > 4) {
			log.reportError("Invalid number of contours");
			return false;
		}
		// Check spiders
		int numZ = (int)Math.pow(2, numC);
		int maxS = (int)Math.pow(2, numZ);
		if (numS < 0 || numS > maxS) {
			log.reportError("Invalid number of spiders");
			return false;
		}
		// Check arcs
		return true;	
	}

	protected MatrixPanel numberPanel;
	protected JTextField numContoursField;
	protected JTextField numSpidersField;
	protected JTextField numArcsField;
	protected JTextField numPointsField;
	protected JCheckBox eqPointsCheck;

	protected ILog log;
}
