package rwd.gui.editor.listeners;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JOptionPane;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.concretesyntax.ConcreteDiagram;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.eulerdiagram.AEDView;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.spiderdiagram.ConcreteSpider;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.viewers.DiagramViewerComponent;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.views.abstracttoconcrete.converting.DrawableResult;
import rwd.views.abstracttoconcrete.converting.Result;
import rwd.views.abstracttoconcrete.converting.ResultDirector;
import rwd.views.abstracttoconcrete.forcemodel.SpiderPlacer;
import rwd.views.smoother.MetricSmoother;

/**
 * @author Octavian Patrascoiu
 *
 */
public class DrawCS2AS2CSListener implements ActionListener {
	public DrawCS2AS2CSListener(EditorFrame frame) {
		this.frame = frame;
	}
	
	public DrawCS2AS2CSListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		drawCSfromAS();
	}

	public void drawCSfromAS() {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		// Create a dialog to input text
		String label = JOptionPane.showInputDialog("Diagram", "d1");
		if (label == null || label.length() == 0) {
			return;
		}
		// Get concrete diagram
		ConcreteUnitaryDiagram cud = viewer.getCUDCalled(label);
		if (cud == null) return;
		// Make the abstract unitary diagram
		UnitaryDiagram aud = cud.getAbstractUnitaryDiagram();
		// Make the new concrete diagram
		ConcreteUnitaryDiagram newCud = getConcreteDiagram(aud);
		// Add arcs
		addArcs(cud, newCud);
		// Draw the concrete diagram and add viewer - smooth or not
		if (isSmooth) {
			DiagramViewerComponent viewer = new DiagramViewerComponent(newCud, 800, 600, true);
			smooth(newCud, viewer);
			addArcs(cud, newCud);
			// Fit in
			viewer.fitToWidth(400);
			// Add viewer
			frame.addViewer(label, viewer);
		} else {
			DiagramViewerComponent viewer = new DiagramViewerComponent(newCud, 800, 600, false);
			SpiderPlacer sp = new SpiderPlacer(viewer);
			sp.embed();
			addArcs(cud, newCud);
			// Fit in
			viewer.fitToWidth(400);
			// Add viewer
			frame.addViewer(label, viewer);
		}
	}
	
	protected ConcreteUnitaryDiagram getConcreteDiagram(UnitaryDiagram aud) {
		Result result = new ResultDirector().makeResult(aud);
		if (!(result instanceof DrawableResult)) {
			return null;
		}
		// Compute concrete contours
		DrawableResult dr = (DrawableResult) result;
		AEDView aedv = dr.getAEDV();

		// Compute decoration box	
		Set _polygonSet = aedv.getConcreteContours();
		DecorationBox box = getBoxAround(_polygonSet, 1.0/6.0);
		box.setLabel(aud.getName()); 

		// Create a concrete diagram
		ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(box);

		// Add decoration box
		cud.addDecorationBox(box);

		// Add contours and zones
		cud.addConcreteContours(_polygonSet);

		// Shade zones
		java.util.Iterator it = aud.getSZIterator();
		Set czones = cud.getConcreteZones();
		while(it.hasNext()){
			Zone z = (Zone)it.next();
			// to shade "outside zone"
			// need to find out a suitable DecorationBox and
			// include this with the contourPolygons to build
			// the concrete zone (use arraylists of Polygonal objects)
			// ConcreteZone.setZoneColors(_zoneColors);
			java.util.Iterator j = czones.iterator();
			while (j.hasNext()) {
				ConcreteZone cz = (ConcreteZone) j.next();
				if (cz.getAbstractZone() == z) {
					cz.setBackColor(Color.gray);
				}
			}
		}

		// Add spiders and their feet
		Vector abstractSpiders = aud.getSpiders();
		for (int i = 0; i < abstractSpiders.size(); i++) {
			Spider s = (Spider) abstractSpiders.get(i);
			//System.out.println("adding a spider:" + s);
			cud.addAbstractSpider(s);
		}
		
		return cud;
	}

	protected void addArcs(ConcreteUnitaryDiagram cud, ConcreteUnitaryDiagram newCud) {	
		// Get arcs
		Set arcs = cud.getConcreteArcs();
		// For each arc
		Iterator i= arcs.iterator();
		while (i.hasNext()) {
			// Get source and target
			ConcreteArc arc = (ConcreteArc)i.next();
			DiagramItem source = arc.getSource();
			DiagramItem target = arc.getTarget();
			// Compute new source and target
			DiagramItem newSource = getCorrespondingItem(cud, source, newCud);
			DiagramItem newTarget = getCorrespondingItem(cud, target, newCud);
			// Make new arc
			ConcreteArc newArc = new ConcreteArc(newSource, newTarget);
			newCud.addConcreteArc(newArc);
		}
	}
	
	protected DiagramItem getCorrespondingItem(ConcreteUnitaryDiagram cud, DiagramItem item, ConcreteUnitaryDiagram newCud) {
		if (item instanceof ConcreteContour) {
			// Scan for a new concrete contour with an equal abstract contour
			Contour c = ((ConcreteContour)item).getContour();
			Iterator i = newCud.getConcreteContours().iterator();
			while (i.hasNext()) {
				ConcreteContour newC = (ConcreteContour)i.next();
				if (c.equals(newC.getContour())) {
					return newC;
				}
			}
		} else if (item instanceof ConcreteSpiderFoot) {
			// Get concrete zone and concrete spider
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)item;
			ConcreteZone zone = cud.getConcreteZone(foot);
			if (zone == null) return null;
			Zone az = zone.getAbstractZone();
			ConcreteSpider spider = null;
			Iterator i = cud.getConcreteSpiders().iterator();
			while (i.hasNext()) {
				spider = (ConcreteSpider)i.next();
				if (spider.getFeet().contains(foot)) {
					break;
				}
			}
			if (spider == null) return null;
			Spider as = spider.getAbstractSpider(cud);
			// Search for a concrete spider with an equal abstract spider
			ConcreteSpider newSpider = null;
			Iterator j = newCud.getConcreteSpiders().iterator();
			while (j.hasNext()) {
				newSpider = (ConcreteSpider)j.next();
				if (as.equals(newSpider.getAbstractSpider(newCud))) {
					break;
				}
			}
			if (newSpider == null) return null;
			// Search for a foot in the same abstract zone
			ConcreteSpiderFoot newFoot = null;
			Iterator k = newSpider.getFeet().iterator();
			while (k.hasNext()) {
				newFoot = (ConcreteSpiderFoot)k.next();
				ConcreteZone newZone = newCud.getConcreteZone(newFoot);
				if (newZone.getAbstractZone().equals(az)) {
					break; 
				}
			}
			return newFoot;
		}
		return null;
	}
	
	public DecorationBox getBoxAround(Set polygonSet, double padding) {
		// Compute border
		int leftMost = Integer.MAX_VALUE;
		int rightMost = Integer.MIN_VALUE;
		int upperMost = Integer.MIN_VALUE;
		int lowerMost = Integer.MAX_VALUE;
		// For each concrete contour
		Iterator i = polygonSet.iterator();
		while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour) i.next();
			Polygon p = cp.getPolygon();
			p.invalidate();
			Rectangle rect = p.getBounds();
			int boxLeft = rect.x;
			int boxBottom = rect.y;
			int boxRight = boxLeft + rect.width;
			int boxTop = boxBottom + rect.height;
			if (leftMost > boxLeft) {
				leftMost = boxLeft;
			}
			if (lowerMost > boxBottom) {
				lowerMost = boxBottom;
			}
			if (rightMost < boxRight) {
				rightMost = boxRight;
			}
			if (upperMost < boxTop) {
				upperMost = boxTop;
			}
		}
//		DecorationSet decorationSet = _diagram.getDecorationSet();
//		i = decorationSet.getBoxes().iterator();
//		while(i.hasNext()){
//			DecorationBox b = (DecorationBox) i.next();
//			//System.out.println("assessing contour polygon " + cp);
//			Polygon p = b.getPolygon();
//			p.invalidate();
//			Rectangle rect = p.getBounds();
//			int boxLeft = rect.x;
//			int boxBottom = rect.y;
//			int boxRight = boxLeft + rect.width;
//			int boxTop = boxBottom + rect.height;
//			if (leftMost > boxLeft) {
//				leftMost = boxLeft;
//			}
//			if (lowerMost > boxBottom) {
//				lowerMost = boxBottom;
//			}
//			if (rightMost < boxRight) {
//				rightMost = boxRight;
//			}
//			if (upperMost < boxTop) {
//				upperMost = boxTop;
//			}
//		}

		// add padding round box
		int strictWidth = rightMost - leftMost;
		rightMost = rightMost + (int) (((double)strictWidth) *padding);
		leftMost = leftMost - (int) (((double)strictWidth) *padding);
		int strictHeight = upperMost - lowerMost;
		upperMost = upperMost + (int) (strictHeight *padding);
		lowerMost = lowerMost - (int) (strictHeight *padding);
		Polygon p =
			new Polygon(
				new int[] { leftMost, rightMost, rightMost, leftMost },
				new int[] { lowerMost, lowerMost, upperMost, upperMost },
				4);
		if (polygonSet.size() == 0) {
			p =
				new Polygon(
					new int[] {
						-ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						-ConcreteDiagram.SCALE },
					new int[] {
						-ConcreteDiagram.SCALE,
						-ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE },
					4);
		}
		DecorationBox box = new DecorationBox("", "", p);
		return box;
	}

	protected void smooth(ConcreteUnitaryDiagram cud, DiagramViewerInterface viewer) {
		// Update diagram
		cud.refreshZones(true);
		cud.notifyObservers();
		
		// Lauch and join smoothing thread	
		MetricSmoother smoother = new MetricSmoother(cud, viewer, 10);
		Thread smootherThread = new Thread(smoother);
		smootherThread.start();
		try {
			smootherThread.join();
		} catch (Exception e) {
			System.out.println("Smoother thread be joined");
			e.printStackTrace();
		}
		
		// Embed spider		
		SpiderPlacer sp = new SpiderPlacer(viewer);
		sp.embed();
	}
	
	// Local variables
	private String text = new String();
	private DiagramViewerInterface viewer;
	private boolean isSmooth = false;
	private EditorFrame frame;
}

