package rwd.gui.editor.listeners;

import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JOptionPane;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.gui.editor.DiagramCanvas;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class DrawNewContourListener implements ActionListener {
	public DrawNewContourListener(EditorFrame frame) {
		this.frame = frame;
	}
	
	public DrawNewContourListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		DiagramCanvas dc = viewer.getDiagramCanvas();
		if (dc == null) return;
		int x = (int)dc.getCurrentX();
		int y = (int)dc.getCurrentY();
		addNewContour(x, y, 500);
	}

	public void addNewContour(int x, int y) {
		addNewContour(x, y, 500);	
	}
	
	public void addNewContour(int x, int y, int radius) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		String label = "";
		label = JOptionPane.showInputDialog("Enter a contour name");
		if (label != null) label = label.trim();

		int numPoints = 0;
		while (numPoints <= 2) {
			String input =
				JOptionPane.showInputDialog(
					null,
					"Number of control points",
					"20");
			if (input == null) {
				return;
			}
			try {
				numPoints = Integer.parseInt(input);
			} catch (NumberFormatException nfe) {
				// Do nothing; keep default value.
			}
		}

		int[] xs = new int[numPoints];
		int[] ys = new int[numPoints];
		for (int i = 0; i < numPoints; i++) {
			xs[i] =
				(int) (x+radius
					* Math.cos(Math.PI * 2 * i / (double) numPoints));
			ys[i] =
				(int) (y+radius
					* Math.sin(Math.PI * 2 * i / (double) numPoints));
		}
		Polygon polygon = new Polygon(xs, ys, numPoints);
		Contour contour = Contour.getContour(label);
		ConcreteContour cp = new ConcreteContour(contour, polygon);
		viewer.add(cp);

		viewer.redrawImage();
	}

	// Local variables
	private EditorFrame frame;
	private DiagramViewerInterface viewer;
}
