package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBox;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.TextBoxDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EditTextBoxListener implements ActionListener {
	public EditTextBoxListener(EditorFrame frame, int type) {
		this.frame = frame;
		this.type = type;
	}
	
	public EditTextBoxListener(DiagramViewerInterface viewer, int type, int scale) {
		this.viewer = viewer;
		this.type = type;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		edit();
	}
	
	public void edit() {
		// Get the text box
		java.util.List items = viewer.getDiagramCanvas().getSelectedItems();
		if (items != null && !items.isEmpty()) {
			DiagramItem item = (DiagramItem)items.iterator().next();
			if (item instanceof ConcreteTextBox) {
				// Create a dialog to input text
				ConcreteTextBox textBox = (ConcreteTextBox)item;
				TextBoxDialog dialog = new TextBoxDialog(frame, "Comment", textBox.getText());
				int returnVal = dialog.showOpenDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String comment = dialog.getText();
					if (comment != null && comment.length() != 0) {
						// Set new text
						textBox.setText(comment);
						
						// Repaint image
						viewer.redrawImage();
					}
				}						
			}
		}
	}

	public static final int ZOOM_IN = 1;
	public static final int ZOOM_OUT = 2;
	public static final int ZOOM = 3;
	 
	// Local variables
	private EditorFrame frame;
	private DiagramViewerInterface viewer;
	private int type;
}
