package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;

import rwd.gui.editor.EditorFileFilter;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.readers.DiagramReader;
import rwd.gui.editor.readers.SerialReader;
import rwd.gui.editor.readers.XmlReader;

/**
 * @author Octavian Patrascoiu
 *
 */
public class FileOpenListener implements ActionListener {
	public FileOpenListener(EditorFrame frame) {
		this.frame = frame;
	}

	public void actionPerformed(ActionEvent e) {
		// Create chooser
		JFileChooser chooser = new JFileChooser(frame.getDirectory());
		chooser.setDialogTitle("Open diagram");
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.setApproveButtonText("Open");
		// Add filters
		EditorFileFilter filter = addFilter(chooser, "Serialized Format (.cd)", "cd");
		addFilter(chooser, "Extended Markup Language (.xml)", "xml");
		chooser.setFileFilter(filter);
		// Choose file
		int returnVal = chooser.showOpenDialog(null);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			frame.setDirectory(file.getParentFile());
			open(file);
		}
	}
	
	protected EditorFileFilter addFilter(JFileChooser chooser, String name, String ext) {
		EditorFileFilter filter = new EditorFileFilter(name);
		filter.addExtension(ext);
		chooser.addChoosableFileFilter(filter);
		return filter;
	}
	
	protected void open(File file) {
		// Create reader
		String fileName = file.getName();
		DiagramReader reader = null;
		if (fileName.endsWith(".xml")) {
			reader = new XmlReader();
		} else if (fileName.endsWith(".cd")) {
			reader = new SerialReader();
		}
		// Read
		reader.read(file.getAbsolutePath());
		// Add viewer
		frame.addViewer(
			fileName,
			reader.getConcreteDiagram());
	    }
	
	// Local variables
	protected EditorFrame frame;
}
