package rwd.gui.editor.listeners;

import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.PathIterator;
import java.util.Iterator;

import javax.swing.JFileChooser;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.AddPointsDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class FormatAddPointsListener implements ActionListener {
	public FormatAddPointsListener(EditorFrame frame) {
		this.frame = frame;
	}

	public FormatAddPointsListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		// Create a dialog to choose number of points
		AddPointsDialog dialog = new AddPointsDialog(frame, "Add more points", frame.getLog());
		int returnVal = dialog.showOpenDialog(frame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			// Get other options
			int numPoints = dialog.getPointNum();
				
			// Reset log
			frame.getLog().reset();

			// Add points and equalise it
			addMorePoints(viewer.getDiagramCanvas().getSelectedItems(), dialog.getPointNum());
		}
	}

	public void addMorePoints(java.util.Collection items, int n) { 
		// iterate through polys and check they have >=8 points
		// otherwise add some at midpoints 
		Iterator it = items.iterator();
		while (it.hasNext()) {
			boolean changed = false;
			DiagramItem item = (DiagramItem) (it.next());
			if (item instanceof ConcreteContour) {
				ConcreteContour cc = (ConcreteContour)item; 
				Polygon p = cc.getPolygon();
				int npoints;
				while ((npoints = p.npoints) < n) {
					changed = true;
					int[] newxs = new int[2 * npoints];
					int[] newys = new int[2 * npoints];
					int evenIndex = 0;
					PathIterator pi = p.getPathIterator(null);
					double[] sixDoubles = new double[6];
					int oldx, nextx, oldy, nexty;
					pi.currentSegment(sixDoubles);
					nextx = (int) (sixDoubles[0]);
					nexty = (int) (sixDoubles[1]);
					int firstx = nextx;
					int firsty = nexty;
					while (evenIndex < 2 * npoints - 2) {
						oldx = nextx;
						oldy = nexty;
						// use the "last" coords for old(x, y)
						// now pick up the "next" coords for next(x, y)
						pi.next();
						pi.currentSegment(sixDoubles);
						nextx = (int) (sixDoubles[0]);
						nexty = (int) (sixDoubles[1]);
						newxs[evenIndex] = oldx;
						newys[evenIndex] = oldy;
						newxs[evenIndex + 1] = (oldx + nextx) / 2;
						newys[evenIndex + 1] = (oldy + nexty) / 2;
						evenIndex = evenIndex + 2;
					}
					oldx = nextx;
					oldy = nexty;
					nextx = firstx;
					nexty = firsty;
					newxs[evenIndex] = oldx;
					newys[evenIndex] = oldy;
					newxs[evenIndex + 1] = (oldx + nextx) / 2;
					newys[evenIndex + 1] = (oldy + nexty) / 2;

					p = new Polygon(newxs, newys, 2 * npoints);
					cc.setPolygon(p);
				}
			}
		}
		viewer.redrawImage();
	}

	// Local variables
	protected EditorFrame frame;
	private DiagramViewerInterface viewer;
}
