package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import rwd.gui.editor.EditorFrame;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class MetricsAverageWeightListener implements ActionListener {
	public MetricsAverageWeightListener(EditorFrame frame) {
		this.frame = frame;
	}

	public void actionPerformed(ActionEvent e) {
		List averageWeight = computeAverageWeight("metrics.txt");
		for(int i=0; i<averageWeight.size(); i++) {
			String s = averageWeight.get(i).toString();
			ILog log = frame.getLog();
			if (log == null)
				log = new OutputStreamLog(System.out); 
			log.reportMessage(s);
		}
	}
	
	protected List computeAverageWeight(String file) {
		List res = new Vector();
		File fin = new File("metrics.txt");
		try {
			// Open file
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
			// Read each line
			String line;
			while ((line = in.readLine()) != null) {
				if (line.length() != 0) {
					String pair[] = line.split(":");
					if (pair.length == 2) {
						// First is value
						String name = pair[0];
						// Second is metric
						String value = pair[1];
						// Increase the number of appereances
						Integer oldNum = (Integer)name2num.get(name);
						if (oldNum == null) {
							oldNum = new Integer(0);
						}
						int num = oldNum.intValue()+1;
						name2num.put(name, new Integer(num)); 
						// Add value to sum and store it
						Double oldSum = (Double)name2sum.get(name);
						if (oldSum == null) {
							oldSum = new Double(0);
						}
						System.out.println(name+":"+value);
						double sum = oldSum.doubleValue() + Double.parseDouble(value);
						name2sum.put(name, new Double(sum)); 
					}
				}
			}
			// Compute average and print it
			ILog log = frame.getLog();
			Iterator i = name2sum.keySet().iterator();
			while (i.hasNext()) {
				String name = (String)i.next();
				Double sum = (Double)name2sum.get(name);
				Integer num = (Integer)name2num.get(name);
				double average = sum.doubleValue()/num.intValue();
				double weight = 10/average;
//				log.reportMessage("Average value('"+name+"') = "+average);				
				log.reportMessage("Average weight('"+name+"') = "+weight);				
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}
	protected Map name2sum = new LinkedHashMap();
	protected Map name2num = new LinkedHashMap();
	
	// Local variables
	protected EditorFrame frame;
}
