package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JFileChooser;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.ApplyRuleDialog;
import rwd.gui.editor.dialogs.TextBoxDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.rules.CDRule;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu, rnb
 * @version 1.7.05
 *
 */
public class ReasoningApplyRuleListener
	implements ActionListener 
{
    public ReasoningApplyRuleListener(EditorFrame frame) {
        this.frame = frame;
    }

    public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		// Create a dialog to choose rule
		ApplyRuleDialog dialog = new ApplyRuleDialog(frame, "Apply rule");
		int returnVal = dialog.showOpenDialog(frame);
		// The rule
		CDRule rule = null;
		// The arguments
		java.util.List args = new Vector();
		args.add(viewer);
		args.addAll(viewer.getSelectedItems());
		// The extra arguments
		String extraArgs = "";
		// Choose rule
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			rule = (CDRule)dialog.getRule();
			if (rule != null) {
				if (rule.needsExtraArgs()) {
					// Create a dialog to input text
					TextBoxDialog dialog1 = new TextBoxDialog(frame, "Extra arguments", "");
					int returnVal1 = dialog1.showOpenDialog(frame);
					if (returnVal1 == JFileChooser.APPROVE_OPTION) {
						extraArgs = dialog1.getText();
					}		
				}
			}
		}
		
		// Apply rule
		if (rule == null) return;
		ILog log = frame.getLog();
				
		// Check if the rule can be applied
		if (rule.canApplyTo(args, extraArgs)) {
			if (log != null) {
				log.reportMessage("Apply \""+rule.toString()+"\" on arguments:");
				for(int i=1; i<args.size(); i++) {
					Object obj = args.get(i);
					if (obj instanceof DiagramItem) {
						DiagramItem di = (DiagramItem) obj;
						log.reportMessage("  " + di.id());
					}
					else
						log.reportMessage("  " + obj.toString());
				}
				if (extraArgs.length()>0)
					log.reportMessage("and extra arguments: \""+extraArgs+"\"");
			}
			ConcreteUnitaryDiagram cud1 = rule.applyRule(args, extraArgs);
			frame.addViewer(cud1);
		} else {
			if (log != null) {
				log.reportMessage("Cannot apply rule \""+rule.name+"\" on arguments:");
				for(int i=1; i<args.size(); i++) {
					Object obj = args.get(i);
					if (obj instanceof DiagramItem) {
						DiagramItem di = (DiagramItem) obj;
						log.reportMessage("  " + di.id());
					}
					else
						log.reportMessage("  " + obj.toString());
				}
				if (extraArgs.length()>0)
					log.reportMessage("and extra arguments: \""+extraArgs+"\"");
			}
		}
		// Repaint image
		viewer.redrawImage();
	}

    // Local variables
    protected EditorFrame frame;
	protected DiagramViewerInterface viewer;
}
