package rwd.gui.editor.listeners.diagramcontrollers;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * The interface for a diagram controller state.
 * 
 * @author Richard Bosworth
 * 
 * @version 15-Apr-2005 15:11
 */
public interface DiagramControllerState
	extends MouseMotionListener, MouseListener, KeyListener
{
	/**
	 * Note: For a state, a mouse click represents a user gesture sequence
	 * <P>
	 * mousePressed mouseDragged* mouseReleased
	 * <P>
	 * where the mouse is released within a few pixels of the mouse press.
	 * <P>
	 * The original mousePressed, mouseDragged and MouseReleased events
	 * are intercepted by the context and not passed on. 
	 * 
	 * @param e mouseClicked event (may not have a MOUSE_CLICKED id)
	 */
	public void mouseClicked(MouseEvent e);
	
	/**
	 * @param e mousePressed event (the start of a drag sequence).
	 *          May not have a MOUSE_PRESSED id.
	 */
	public void mousePressed(MouseEvent e); 

	/**
	 * @param e mouseReleased event (the end of a drag sequence).
	 */
	public void mouseReleased(MouseEvent e); 

	/**
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e);
	
	/**
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e);
	
	/**
	 * @param e mouseDragged event, one of a series forming a drag sequence
	 */
	public void mouseDragged(MouseEvent e); 
	
	/**
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e);

	/**
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e);
	
	/**
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e);
	
	/**
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e);
	
}
