package rwd.gui.editor.readers;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.util.Vector;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.constraintdiagram.ConcreteArcSet;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBox;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBoxSet;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteContourSet;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.ConcreteZoneSet;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.eulerdiagram.DecorationSet;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFeetAndLegsSet;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;

/**
 * @author Octavian Patrascoiu,rnb
 * @version 11.8.05   13:02
 *
 */
class XmlHandler implements ContentHandler {
	public XmlHandler(XmlReader reader) {
		this.reader = reader;
	}
	public void setDocumentLocator(Locator locator) {
		this.locator = locator;
	}
	public void startDocument() throws SAXException {
	}
	public void endDocument() throws SAXException {
		int i=4;
	}
	public void startPrefixMapping(String prefix, String uri) throws SAXException {
	}
	public void endPrefixMapping(String prefix) throws SAXException {
	}
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
		text = new String();
		// Viewer
		if (localName.equals("viewer")) {
			// Get title
			viewerTitle = atts.getValue("title");
			// Initialise the list of decoration boxes
			decorationBoxes = new Vector();
			// Initialise the list of contours
			concreteContours = new Vector();
			// Initialise the list of concrete zones
			concreteZones = new Vector();
			// Initialise the list of spider feet
			spiderFeet = new Vector();
			// Initialise the list of spider legs
			spiderLegs = new Vector();
			// Initialise the list of arcs
			arcs = new Vector();
			// Initialise the list of texts
			texts = new Vector();
		// Decoration box
		} else if (localName.equals("box")) {
			boxLabel = atts.getValue("label");
			itemId = atts.getValue("id");
		// Reading forest
		} else if (localName.equals("reading")) {
			readingForest = "";
		// Polygon
		} else if (localName.equals("polygon")) {
			// Initialize the list of coordinates
			polygonXList = new Vector();
			polygonYList = new Vector();
		// Point
		} else if (localName.equals("point")) {
			// Add coordinates
			polygonXList.add(Integer.valueOf(atts.getValue("x")));
			polygonYList.add(Integer.valueOf(atts.getValue("y")));
		// Contour
		} else if (localName.equals("contour")) {
			// Initialize the label attributes
			initLabel();
			contourLabel = atts.getValue("label");
			itemId = atts.getValue("id");
		// Zone
		} else if (localName.equals("zone")) {
			inContours = new Vector();
			outContours = new Vector();
		// Zone
		} else if (localName.equals("in")) {
			zoneContours = new Vector();
		} else if (localName.equals("out")) {
			zoneContours = new Vector();
		// Contour in <zone>
		} else if (localName.equals("zone-contour")) {
			// Retrieve contours and add it
			itemId = atts.getValue("ref");
			Object contour = id2item.get(itemId);
			zoneContours.add(contour);
		// Color value
		} else if (localName.equals("color")) {
		// Foot
		} else if (localName.equals("foot")) {
			// Initialize the label attributes
			initLabel();
			// Get attributes
			itemId = atts.getValue("id");
			int footType = Integer.parseInt(atts.getValue("type"));
			int footX = Integer.parseInt(atts.getValue("x"));
			int footY = Integer.parseInt(atts.getValue("y"));
			// Create a foot and store id-mapping
			spiderFoot = new ConcreteSpiderFoot("", footType);
			spiderFoot.setLocation(footX, footY);
			id2item.put(itemId, spiderFoot);
			// Add foot
			spiderFeet.add(spiderFoot);
		// Leg
		} else if (localName.equals("leg")) {
			// Get attributes
			int legType = Integer.parseInt(atts.getValue("type"));
			String footSourceId = atts.getValue("source");
			String footTargetId = atts.getValue("target");
			// Create leg
			ConcreteSpiderFoot source = (ConcreteSpiderFoot)id2item.get(footSourceId);
			ConcreteSpiderFoot target = (ConcreteSpiderFoot)id2item.get(footTargetId);
			ConcreteSpiderLeg leg = new ConcreteSpiderLeg("", source, target, legType);
			// Add it
			spiderLegs.add(leg);
		// Arc
		} else if (localName.equals("arc")) {
			// Initialize the label attributes
			initLabel();
			// Get attributes
			arcType = Integer.parseInt(atts.getValue("type"));
			// Initiliaze the list of ends
			arcEnds = new Vector();
		// End
		} else if (localName.equals("end")) {
			// Get attributes
			String idStr = atts.getValue("id");
			String xStr = atts.getValue("x");
			String yStr = atts.getValue("y");
			int endX = Integer.parseInt(xStr);
			int endY = Integer.parseInt(yStr);
			if (idStr != null) {
				DiagramItem item = (DiagramItem)id2item.get(idStr);
				// Create and add end
				ConcreteArcEnd end = new ConcreteArcEnd(item);
				arcEnds.add(end);
				end.setLocation(endX, endY);		
			} else {
				// Create and add end
				ConcreteArcEnd end = new ConcreteArcEnd(endX, endY);
				arcEnds.add(end);
			}
		// Label
		} else if (localName.equals("label")) {
			// Set label and labelPosition
			label = atts.getValue("name");
			double x = Double.parseDouble(atts.getValue("x"));
			double y = Double.parseDouble(atts.getValue("y"));
			labelPosition = new Point2D.Double(x, y);
		// Text
		} else if (localName.equals("comment")) {
			// Get attributes
			textType = Integer.parseInt(atts.getValue("type"));
		}
	}
	public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
		// Viewer
		if (localName.equals("viewer")) {
			// Set title
			reader.title = viewerTitle;
			// Create diagram
			reader.diagram = new ConcreteDiagram();
			// Set decoration boxes
			reader.diagram.setDecorationBoxes(new DecorationSet(decorationBoxes));
			// Set concrete contours
			reader.diagram.setConcreteContourSet(new ConcreteContourSet(concreteContours));
			// Set contour zones
			reader.diagram.setConcreteZoneSet(new ConcreteZoneSet(concreteZones));
			// Set feet and legs
			reader.diagram.setFeetAndLegsSet(new ConcreteSpiderFeetAndLegsSet(spiderFeet, spiderLegs));
			// Set arcs
			reader.diagram.setArcSet(new ConcreteArcSet(arcs));
			// Set texts
			reader.diagram.setTextSet(new ConcreteTextBoxSet(texts));
		// Decoration box
		} else if (localName.equals("box")) {
			// Create box and add it
			decorationBox = new DecorationBox(boxLabel, readingForest, polygon); 
			decorationBoxes.add(decorationBox);
			// Store box
			id2item.put(itemId, decorationBox);
		// Reading forest
		} else if (localName.equals("reading")) {
			readingForest = text;
		// Contour
		} else if (localName.equals("contour")) {
			// Create contour and add it
			concreteContour = new ConcreteContour(Contour.getContour(contourLabel), polygon);
			concreteContours.add(concreteContour);
			// Set label and label location
			if (labelPosition != null)
				concreteContour.setLabelPosition(labelPosition);
			else
				concreteContour.autoPlaceLabel();
			// Store box
			id2item.put(itemId, concreteContour);
		// Zone
		} else if (localName.equals("zone")) {
			zone = new ConcreteZone(inContours, outContours);
			concreteZones.add(zone);
			zone.setBackColor(color);
		// Zone
		} else if (localName.equals("in")) {
			inContours.addAll(zoneContours);
		} else if (localName.equals("out")) {
			outContours.addAll(zoneContours);
		} else if (localName.equals("zone-contour")) {
		} else if (localName.equals("color")) {
			// Compute r, g, and b and color
			String rgb[] = text.split(",");
			Integer r = Integer.valueOf(rgb[0]);
			Integer g = Integer.valueOf(rgb[1]);
			Integer b = Integer.valueOf(rgb[2]);
			color = new Color(r.intValue(), g.intValue(), b.intValue());
		// Foot
		} else if (localName.equals("foot")) {
			// Set label and label location
			spiderFoot.setLabel(label);
			spiderFoot.setLabelPosition(labelPosition);
		// Arc
		} else if (localName.equals("arc")) {
			// Compute start and end
			ConcreteArcEnd start = (ConcreteArcEnd)arcEnds.get(0);
			if (arcType == ConcreteArc.QUAD) {
				ConcreteArcEnd middle = (ConcreteArcEnd)arcEnds.get(1);
				ConcreteArcEnd end = (ConcreteArcEnd)arcEnds.get(2);
				// Create arc and add it
				ConcreteArc arc = new ConcreteArc(start, end, arcType);
				arc.setMiddle(middle);
				arcs.add(arc);
				// Set label and label location
				arc.setLabel(label);
				arc.setLabelPosition(labelPosition);
			} else {
				ConcreteArcEnd end = (ConcreteArcEnd)arcEnds.get(1);
				// Create arc and add it
				ConcreteArc arc = new ConcreteArc(start, end, arcType);
				arcs.add(arc);
				// Set label and label location
				arc.setLabel(label);
				arc.setLabelPosition(labelPosition);
			}
		// Comment
		} else if (localName.equals("comment")) {
			// Compute location
			int x = ((Integer)polygonXList.get(0)).intValue();
			int y = ((Integer)polygonYList.get(0)).intValue();
			// Create text and add it
			ConcreteTextBox textBox = new ConcreteTextBox(textType, x, y, text);
			texts.add(textBox);
		// Comment body
		} else if (localName.equals("text")) {
			commentText = text;
		// Polygon - list of points
		} else if (localName.equals("polygon")) {
			// Create a new polygon
			polygon = new Polygon();
			// Add points
			for(int i=0; i<polygonXList.size(); i++) {
				int x = ((Integer)polygonXList.get(i)).intValue();
				int y = ((Integer)polygonYList.get(i)).intValue();
				polygon.addPoint(x, y);
			}
		// Key in zone colors
		} else if (localName.equals("key")) {
			key = text;
		// Color value
		} else if (localName.equals("value")) {
			// Compute r, g, and b and color
			String rgb[] = text.split(",");
			Integer r = Integer.valueOf(rgb[0]);
			Integer g = Integer.valueOf(rgb[1]);
			Integer b = Integer.valueOf(rgb[2]);
			color = new Color(r.intValue(), g.intValue(), b.intValue());
			// Set color
			//reader.zoneColors.put(key, color); 
            // NEEDS SOME FIX
		} else {
		}
	}
	public void characters(char ch[], int start, int length) throws SAXException {
		String str = new String(ch, start, length);
		text += str;
	}
	public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
	}
	public void processingInstruction(String target, String data) throws SAXException {
	}
	public void skippedEntity(String name) throws SAXException {
	}

	protected void initLabel() {
		label = "";
		labelPosition = null;
	}
	// Local propeerties
	protected XmlReader reader;
	protected String text;
	protected Locator locator;
	
	protected Vector decorationBoxes;
	protected Vector concreteContours;
	protected Vector concreteZones;
	protected Vector inContours;
	protected Vector outContours;
	protected Vector zoneContours;
	protected Vector spiderFeet;
	protected Vector spiderLegs;
	protected Vector arcs;
	protected Vector arcEnds;
	protected Vector texts;

	protected String viewerTitle;
	protected String boxLabel;
	protected String contourLabel;
	protected List polygonXList;
	protected List polygonYList;
	protected int arcType;
	protected int textType;
	protected String commentText;
	protected String readingForest;
		
	protected DecorationBox decorationBox;
	protected ConcreteContour concreteContour;
	protected ConcreteZone zone;
	protected String itemId;
	protected Map id2item = new HashMap();
	protected ConcreteSpiderFoot spiderFoot;
	protected ConcreteSpiderLeg spiderLeg;
	protected String label = "";
	protected Point2D labelPosition;
	protected Polygon polygon;
	protected String key;
	protected Color color;
	protected String id;
	
}

