package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class ContourArea
	extends EulerMetric 
{
	public ContourArea(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	/**
	 * Score is given by calculating the difference in squares from
	 * the mean area. Ergo, the lowest (best) score is achieved when
	 * all contours have the same area.
	 */
	public double giveScore() {
		double totalArea = 0.0;
		Set contourPolys = cud.getConcreteContours();
        Iterator i = contourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();
			totalArea += AreaPolygonAdapter.getAreaOf(p);
		}
		double meanArea = totalArea / contourPolys.size();

		double differenceOfSquares = 0.0;
        i = contourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();
			double area = AreaPolygonAdapter.getAreaOf(p);
			differenceOfSquares += (area - meanArea) * (area - meanArea);
		}
		//differenceOfSquares=differenceOfSquares/10000000000000.0;
		//System.out.println("ContourAreaMetric returning "
		//                                    +differenceOfSquares);

		//double diagramArea = DiagramAreaMetric.getDiagramArea(polys);

		return differenceOfSquares / (totalArea * totalArea);
	}
}
