package rwd.metrics;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.util.List;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.DecorationBox;


public class ContourClosenessSegmentPoint
	extends EulerMetric 
{
	public ContourClosenessSegmentPoint(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}
	
	protected double giveScore() {
		List contours = new Vector(cud.getConcreteContours());
		double total = 0.0;
		for(int i=0; i<contours.size(); i++) {
			ConcreteContour cp1 = (ConcreteContour)contours.get(i); 
			Polygon p1 = cp1.getPolygon();
			for(int j=i+1; j<contours.size(); j++) {
				ConcreteContour cp2 = (ConcreteContour)contours.get(j); 
				Polygon p2 = cp2.getPolygon();
				total += computePairTotal(p1, p2);
			}
		}

		return total;
	}

	protected double computePairTotal(Polygon p1, Polygon p2) {
		double pairTotal = 0.0;

		// get closest separation value
		//double closest = Double.MAX_VALUE;
		int[] xs1 = p1.xpoints;
		int[] ys1 = p1.ypoints;
		int numPts1 = p1.npoints;
		int[] xs2 = p2.xpoints;
		int[] ys2 = p2.ypoints;
		int numPts2 = p2.npoints;
		int xbefore, ybefore;
		int x, y;
		int xafter, yafter;
		// for each point on the first polygon....
		// loop n-1 times
		for (int pi = 0; pi <= numPts1 - 1; pi++) {
			xbefore = xs1[(pi - 1 + numPts1) % numPts1];
			ybefore = ys1[(pi - 1 + numPts1) % numPts1];
			x = xs1[pi];
			y = ys1[pi];
			xafter = xs1[(pi + 1) % numPts1];
			yafter = ys1[(pi + 1) % numPts1];
			// (x,y) is a vertex on the first polygon
			// for each point of the second polygon...
			int x1 = xs2[numPts2 - 1];
			int y1 = ys2[numPts2 - 1];
			// loop n-1 times
			for (int pj = 0; pj <= numPts2 - 1;	pj++) { 
				int x2 = xs2[pj];
				int y2 = ys2[pj];
				//System.out.println("pt ("+x+","+y+") edge ("+x1+","+y1+")-("+x2+","+y2+")");
				// (x1,y1)-(x2,y2) is an edge in the 2nd polygon
				int vx = x2 - x1;
				int vy = y2 - y1;
				// v is vector along edge
				double length = Math.sqrt(vx * vx + vy * vy);
				sumOfAllLengths += length;
				double nvx = ((double) (vx)) / length;
				double nvy = ((double) (vy)) / length;
				// nv is normed vector along edge
				int pToEx = x - x1;
				int pToEy = y - y1;
				// pToE is vector from point on contour 1 to start of edge
				double pToEProjectedAlongEdgex =
					nvx * (nvx * pToEx + nvy * pToEy);
				double pToEProjectedAlongEdgey =
					nvy * (nvx * pToEx + nvy * pToEy);
				//System.out.println("x-coord of projection along edge is "
				//                                +pToEProjectedAlongEdgex);
				//System.out.println("vx is "+vx);
				// pToEProjectedAlongEdge
				if (0 < (pToEProjectedAlongEdgex * vx)
					&& pToEProjectedAlongEdgex
						* pToEProjectedAlongEdgex
						< vx * vx) {
					//System.out.println("the point/edge combination is relevant");
					if (!Line2D
						.linesIntersect(
							xbefore,
							ybefore,
							x,
							y,
							x1,
							y1,
							x2,
							y2)
						&& !Line2D.linesIntersect(
							xafter,
							yafter,
							x,
							y,
							x1,
							y1,
							x2,
							y2)) { //System.out.println("point is not next to a crossing edge");
						//System.out.println("edges compared are ("
						//       +x1+","+y1+")=("+x2+","+y2+") and ("
						//       +xbefore+","+ybefore+")=("+x+","+y+")");
						//System.out.println("edges compared are ("
						//       +x1+","+y1+")=("+x2+","+y2+") and ("
						//       +xafter+","+yafter+")=("+x+","+y+")");
						double pToEPerpx =
							pToEx - pToEProjectedAlongEdgex;
						double pToEPerpy =
							pToEy - pToEProjectedAlongEdgey;
						double distSqrd =
							pToEPerpx * pToEPerpx
								+ pToEPerpy * pToEPerpy;
						//if(closest > distSqrd)
						//{
						//System.out.println("distSqrd is "+distSqrd);
						//System.out.println("updating total : add "+1.0/((double)distSqrd));
						if (distSqrd != 0)
							pairTotal += 1.0 / (distSqrd * distSqrd);
						// emphasised by squareing 11/5/03
						//}
					}
				}
				x1 = x2;
				y1 = y2;
			}

		}
		//System.out.println("incorporating closest :"+closest);
		//total = total + 1/(closest*closest);
		if (numPts1*numPts2 != 0)
			pairTotal = pairTotal / (numPts1 * numPts2);
		return pairTotal;
	}

	public static void main(String[] args) { //build PolygonSet with null zones is OK
		Polygon p1 =
			new Polygon(
				new int[] { 0, 20, 20, 0 },
				new int[] { 0, 0, 20, 20 },
				4);
		Polygon p2 =
			new Polygon(
				new int[] { 5, 35, 30, 10 },
				new int[] { 5, 10, 30, 25 },
				4);
		ConcreteContour cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		ConcreteContour cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,50,50,0},new int[]{0,0,50,50},4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
		/**/
		System.out.println(
			"" + new ContourClosenessSegmentPoint(cud, 1.0).giveScore());
		////////////////
		p1 =
			new Polygon(
				new int[] { 0, 200, 200, 0 },
				new int[] { 0, 0, 200, 200 },
				4);
		p2 =
			new Polygon(
				new int[] { 50, 350, 300, 100 },
				new int[] { 50, 100, 300, 250 },
				4);
		cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,400,400,0},new int[]{0,0,400,400},4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
		/**/
		System.out.println(
			"" + new ContourClosenessSegmentPoint(cud, 1.0).giveScore());
	}

	//
	// Local properties
	//
	protected double sumOfAllLengths = 0.0;
}
