package rwd.metrics;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class ContourEdgeAngularResolution
	extends EulerMetric 
{
    public ContourEdgeAngularResolution(ConcreteUnitaryDiagram cud, double weighting) {
        super(cud, weighting);
    }

    protected double giveScore() {
        double sumOfAllLengths = 0.0;

        Set contourPolys = cud.getConcreteContours();

        if (contourPolys.size() < 2) {
            return (0.0);
        }
        double total = 0.0;

        Iterator i = contourPolys.iterator();
        while (i.hasNext()) {
            ConcreteContour cp1 = (ConcreteContour) i.next();

            // only interate through pairs once
            Iterator j = contourPolys.iterator();
            if(j.hasNext()){
                ConcreteContour cp2 = (ConcreteContour)j.next();
                while(cp2!=cp1){
                    cp2 = (ConcreteContour)j.next();
                }
            }
            while(j.hasNext()){
                ConcreteContour cp2 = (ConcreteContour)j.next();
                if (cp1 != cp2) {

                    Polygon p1 = cp1.getPolygon();
                    Polygon p2 = cp2.getPolygon();
                    double pairTotal = 0.0;

                    // get closest separation value
                    //double closest = Double.MAX_VALUE;
                    int[] xs1 = p1.xpoints;
                    int[] ys1 = p1.ypoints;
                    int numPts1 = p1.npoints;
                    int[] xs2 = p2.xpoints;
                    int[] ys2 = p2.ypoints;
                    int numPts2 = p2.npoints;
                    int x1start, y1start, x1end, y1end;
                    int x2start, y2start, x2end, y2end;
                    // for each point on the first polygon....
                    for (int pi = 0; pi <= numPts1 - 1; pi++) {
                        x1start = xs1[pi];
                        y1start = ys1[pi];
                        x1end = xs1[(pi + 1) % numPts1];
                        y1end = ys1[(pi + 1) % numPts1];

                        Line2D.Double line1 =
                            new Line2D.Double(
                                new Point2D.Double(x1start, y1start),
                                new Point2D.Double(x1end, y1end));

                        for (int pj = 0; pj <= numPts2 - 1; pj++) {
                            x2start = xs2[pj];
                            y2start = ys2[pj];
                            x2end = xs2[(pj + 1) % numPts2];
                            y2end = ys2[(pj + 1) % numPts2];

                            Line2D.Double line2 =
                                new Line2D.Double(
                                    new Point2D.Double(x2start, y2start),
                                    new Point2D.Double(x2end, y2end));
                            if (line1.intersectsLine(line2)) {
                                double angleBetweenLines =
                                    getAngleBetweenLines(line1, line2);
                                total =
                                    total
                                        + 1
                                            / (angleBetweenLines
                                                * angleBetweenLines);
                            }
                        }
                    }

                    pairTotal = pairTotal / (numPts1 * numPts2);
                    total = total + pairTotal;
                }
            }
        }
        return total;
    }

    /**
     * Returns the smallest angle (in radians) between the intersection of two lines.
     * The code seems to work, but its a horrid hack.
     */
    double getAngleBetweenLines(Line2D.Double line1, Line2D.Double line2) {

        double x1start = line1.x1;
        double x1end = line1.x2;
        double y1start = line1.y1;
        double y1end = line1.y2;

        double x2start = line2.x1;
        double x2end = line2.x2;
        double y2start = line2.y1;
        double y2end = line2.y2;

        double xstartdiff = x1start - x2start;
        double xenddiff = x1end - x2end;

        double ystartdiff = y1start - y2start;
        double yenddiff = y1end - y2end;

        double angle1 = 0.0;
        double angle2 = 0.0;

        angle1 = Math.atan((y1start - y1end) / (x1start - x1end));
        angle2 = Math.atan((y2start - y2end) / (x2start - x2end));

        if (angle1 < 0) {
            angle1 += Math.PI;
        }
        if (angle2 < 0) {
            angle2 += Math.PI;
        }

        double angle = Math.abs(angle1 - angle2);

        if (angle > Math.PI / 2) {
            angle = Math.PI - angle;
        }

        //double inDegrees1 = Math.toDegrees(angle1);
        //double inDegrees2 = Math.toDegrees(angle2);
        //double degrees = Math.toDegrees(angle);
        //System.out.println("Angle1 "+ inDegrees1+" Angle2 "+inDegrees2+" Diff "+ degrees);

        return angle;
    }

}
