package rwd.metrics;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class ContourPositionComparison
	extends Comparison 
{
	public ContourPositionComparison(ConcreteUnitaryDiagram cud, double weighting, ConcreteUnitaryDiagram moveCud) {
		super(cud, weighting, moveCud);
		usedByPointMove = false;
	}

	protected double giveScore() {

		double total = 0.0;

		Set moveContourPolys = cud.getConcreteContours();
		Set stillContourPolys = stillCud.getConcreteContours();

        Iterator i = stillContourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour stillcp = (ConcreteContour)i.next();
			Polygon stillp = stillcp.getPolygon();

			Point2D.Double stillCentre = polygonCentre(stillp);

            Iterator j = moveContourPolys.iterator();
            while(j.hasNext()){
				ConcreteContour movecp = (ConcreteContour)j.next();

				if(stillcp.getContour().equals(movecp.getContour())) {

					Polygon movep = movecp.getPolygon();

					Point2D.Double moveCentre = polygonCentre(movep);

					double diffX = stillCentre.x-moveCentre.x;
					double diffY = stillCentre.y-moveCentre.y;
					double diffSq = diffX*diffX+diffY*diffY;

					total += diffSq;
				}
			}
		}

		double polygonArea = getPolygonAreaSum(stillContourPolys);
        if(polygonArea!=0){
            return total/polygonArea;
        } else {
            return total;
        }
	}	


	public static Point2D.Double polygonCentre(Polygon p) {

		p.invalidate();

		Rectangle r = p.getBounds();

		double x = r.x + r.width/2;
		double y = r.y + r.height/2;

		return new Point2D.Double(x,y);
	}
	


	public static double getPolygonAreaSum(Set contourPolygons) {

		double areaTotal = 0.0;

        Iterator i = contourPolygons.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();

			areaTotal += AreaPolygonAdapter.getAreaOf(p);
		}
		return areaTotal;
	}


	
}


