package rwd.metrics;

import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.deltas.Delta;
import rwd.deltas.EdgeContourCrossingDelta;

/**
 * @author Octavian Patrascoiu
 * 
 * Edge-Contour border crossing. Edges crossing borders too many times is bad. 
 * 
 * Adding the number of times edges cross contour borders.
 */
public class EdgeContourCrossing
	extends SpiderMetric
{
	public EdgeContourCrossing(
		ConcreteUnitaryDiagram cud,
		double weighting) {
		super(cud, weighting);
	}
	public EdgeContourCrossing(ConcreteUnitaryDiagram cud) {
		super(cud);
	}

	public void initDeltas(ConcreteUnitaryDiagram cud) {
		if (cud == null) return;
		// For each contour and leg
		deltas = new Vector();
		List contours = new Vector(cud.getConcreteContours());
		List legs = new Vector(cud.getConcreteLegs());
		for(int i=0; i<contours.size(); i++) {
			ConcreteContour c = (ConcreteContour)contours.get(i);
			for(int j=0; j<legs.size(); j++) {
				ConcreteSpiderLeg leg = (ConcreteSpiderLeg)legs.get(j);
				// Compute the number of crossings
				Delta delta = new EdgeContourCrossingDelta(leg, c);
				deltas.add(delta);
			}
		}
		// For each contour and arc
		List arcs = new Vector(cud.getConcreteArcs());
		for(int i=0; i<contours.size(); i++) {
			ConcreteContour c = (ConcreteContour)contours.get(i);
			for(int j=0; j<arcs.size(); j++) {
				ConcreteArc arc = (ConcreteArc)arcs.get(j);
				// Compute the number of crossings
				Delta delta = new EdgeContourCrossingDelta(arc, c);
				deltas.add(delta);
			}
		}		
	}
}
