package rwd.metrics;

import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.spiderdiagram.ConcreteSpider;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.deltas.Delta;
import rwd.deltas.EdgeEdgeCrossingDelta;

/**
 * @author Octavian Patrascoiu
 * 
 * Edge-Edge crossing. Edge crossings are bad. 
 * Add the edge crossings in the diagram
 */
public class EdgeEdgeCrossing 
	extends SpiderMetric 
{
	public EdgeEdgeCrossing(
		ConcreteUnitaryDiagram cud,
		double weighting) {
		super(cud, weighting);
	}
	public EdgeEdgeCrossing(ConcreteUnitaryDiagram cud) {
		super(cud);
	}

	protected void initDeltas(ConcreteUnitaryDiagram cud) {
		if (cud == null) return;
		// For each pair of distinct spiders
		deltas = new Vector();
		List spiders = new Vector(cud.getConcreteSpiders());
		for(int i=0; i<spiders.size(); i++) {
			ConcreteSpider sp1 = (ConcreteSpider)spiders.get(i);
			List legs1 = new Vector(sp1.getLegs());
			for(int j=i+1; j<spiders.size(); j++) {
				ConcreteSpider sp2 = (ConcreteSpider)spiders.get(j);
				List legs2 = new Vector(sp2.getLegs());
				// For each pair of legs
				for(int k1=0; k1<legs1.size(); k1++) {
					ConcreteSpiderLeg leg1 = (ConcreteSpiderLeg)legs1.get(k1);
					for(int k2=0; k2<legs2.size(); k2++) {
						ConcreteSpiderLeg leg2 = (ConcreteSpiderLeg)legs2.get(k2);
						// Add delta
						Delta delta = new EdgeEdgeCrossingDelta(leg1, leg2);
						deltas.add(delta);
					}
				}
			}
		}
		// For each pair (leg, arc)
		List legs = new Vector(cud.getConcreteLegs());
		List arcs = new Vector(cud.getConcreteArcs());
		for(int i=0; i<legs.size(); i++) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)legs.get(i);
			ConcreteSpiderFoot source1 = leg.getSource();
			ConcreteSpiderFoot target1 = leg.getTarget();
			for(int j=0; j<arcs.size(); j++) {
				ConcreteArc arc = (ConcreteArc)arcs.get(j);
				ConcreteArcEnd source2 = arc.getStart();
				ConcreteArcEnd target2 = arc.getEnd();
				if (near(source1, source2) || near(source1, target2) || near(target1, source2) || near(target1, target2))
					;
				else {
					// Add delta
					Delta delta = new EdgeEdgeCrossingDelta(leg, arc);
					deltas.add(delta);
				}
			}
		}
		// For each pair (arc, arc)
		for(int i=0; i<arcs.size(); i++) {
			ConcreteArc arc1 = (ConcreteArc)arcs.get(i);
			for(int j=i+1; j<arcs.size(); j++) {
				ConcreteArc arc2 = (ConcreteArc)arcs.get(j);
				// Add delta
				Delta delta = new EdgeEdgeCrossingDelta(arc1, arc2);
				deltas.add(delta);
			}
		}
	}

	protected boolean near(ConcreteSpiderFoot item1, ConcreteArcEnd item2) {
		double x1 = item1.getX();
		double y1 = item1.getY();
		double x2 = item2.getX();
		double y2 = item2.getY();
		double d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
		if (Math.abs(d) < 10)
			return true;
		return false;	
	}
}
