package rwd.metrics;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.deltas.*;

/**
 * @author Octavian Patrascoiu
 *
 * Edge Length. Long edges are bad. Distance between connected nodes. 
 * 
 * Summed square of distance
 */
public class EdgeLength 
	extends SpiderMetric 
{
	public EdgeLength(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
		area = getArea();
	}
	public EdgeLength(ConcreteUnitaryDiagram cud) {
		super(cud);
		area = getArea();
	}

	protected void initDeltas(ConcreteUnitaryDiagram cud) {
		if (cud == null) return;
		// For each leg and
		deltas = new Vector();
		Set items = new LinkedHashSet(cud.getConcreteLegs());
		items.addAll(cud.getConcreteArcs());
		Iterator i = items.iterator();
		while (i.hasNext()) {
			// Make a specialized delta delta(item) where item is either leg or arc  
			DiagramItem item = (DiagramItem)i.next();
			Delta delta = new EdgeLengthDelta(item); 
			deltas.add(delta);
		}				
	}
	public double giveScore() {
		double score = super.giveScore();
		if (area != 0)
			score /= area;
		return score;
	}

}
