package rwd.metrics;

import java.text.NumberFormat;
import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.Event;
import rwd.concretesyntax.Observable;
import rwd.concretesyntax.Observer;
import rwd.deltas.Delta;

public abstract class Metric
	implements Observer
{
	public Metric(ConcreteUnitaryDiagram cud) {
		this.cud = cud;
		this.weighting = 1.0;
		// Init deltas
		initDeltas(cud);
		// Add this metric as observer
		if (cud != null)
			cud.addObserver(this);
		name = getClass().getName().substring(12);
	}

	public Metric(ConcreteUnitaryDiagram cud, double weighting) {
		this.cud = cud;
		this.weighting = weighting;
		// Init deltas
		initDeltas(cud);
		// Add this metric as observer
		if (cud != null)
			cud.addObserver(this);
		name = getClass().getName().substring(12);
		key = "";
		for(int i=0; i<name.length(); i++) {
			char ch = name.charAt(i);
			if (Character.isUpperCase(ch)) {
				key += ch;
			}
		}
	}

	public double measure() {
		if (NORMALIZE) {
			if (firstMeasurement) {
				firstMeasurement = false;
				double score = giveScore();
				double exp = computeExp(score);
				weighting *= Math.pow(10, exp);  
			}
		}
		return weighting * giveScore();
	}
	
	protected double computeExp(double firstScore) {
		double w = Math.abs(firstScore);
		double exp = 0;
		while (true) {
			if (w > RANGE) {
				w /= 10;
				exp--;
			} else if (0 < w && w < 1) {
				w *= 10;
				exp++;
			} else {
				break;
			}
		}
		return exp;
	}

	public void update(Observable cd, Object arg) {
		if (cd == null) return;
		if (cd instanceof ConcreteUnitaryDiagram) {
			// Add item
			if (arg == Event.ADD_ITEM) {
				initDeltas((ConcreteUnitaryDiagram)cd);
			// Remove item
			} else if (arg == Event.REMOVE_ITEM) {
				initDeltas((ConcreteUnitaryDiagram)cd);
			// Change item
			} else {
				List items = cud.getDiagramItems();
				for (int i = 0; i < items.size(); i++) {
					DiagramItem item = (DiagramItem) items.get(i);
					item.notifyObservers(arg);
				}			
			}
		}
	}

	protected double giveScore() {
		double score = 0.0;
		for(int i=0; i<deltas.size(); i++) {
			Delta d = (Delta)deltas.get(i);
			score += d.evaluate();
		}
		return score;
	}

	protected void initDeltas(ConcreteUnitaryDiagram cd) {
		deltas = new Vector();
	}
	
	public void activateDeltaObservers() {
		// For each delta
		List deltas = getDeltas();
		for (int k = 0; k < deltas.size(); k++) {
			Delta delta = (Delta) deltas.get(k);
			List args = delta.getArgs();
			// For each diagram item
			List items = cud.getDiagramItems();
			for (int i = 0; i < items.size(); i++) {
				DiagramItem item = (DiagramItem) items.get(i);
				if (args.contains(item)) {
					item.addObserver(delta);
					item.setChanged();
					item.notifyObservers();
				}
			}
		}
	}

	public double getMin() {
		return min;
	}

	public double getMax() {
		return max;
	}

	public double getValue() {
		return measure();
	}

	public static String getDisplayValue(double value) {
		if (value == Double.NEGATIVE_INFINITY) return "-oo";
		if (value == Double.MIN_VALUE) return "-oo";
		if (value == Double.POSITIVE_INFINITY) return "+oo";
		if (value == Double.MAX_VALUE) return "+oo";
		NumberFormat f = NumberFormat.getInstance();
		f.setMaximumFractionDigits(2);
		return f.format(value);
	}
	
	//
	// Get/Set methods
	//
	public double getWeighting() {
		return weighting;
	}
	public void setWeighting(double w) {
		weighting = w;
	}
	public List getDeltas() {
		return deltas;
	}

	// Compute the area to divide/multiply
	protected double getArea() {
		if (cud == null) return 0;
		return DiagramArea.getDiagramArea(cud);
	}
	public String getName() {
		return name;
	}
	public String getKey() {
		return key;
	}

	/** Check if the metric is good */
	public boolean isContradiction() {
		double value = getValue();
		if (min != Double.MIN_VALUE || min != Double.NEGATIVE_INFINITY) {
			if (min > value)
				return true;
		}
		if (max != Double.MAX_VALUE || max != Double.POSITIVE_INFINITY) {
			if (max < value)
				return true;
		}
		return false;					
	}
	
	/** Activate deltas */
	public static void activateDeltaObservers(java.util.List metrics) {
		// For each metric
		for (int j = 0; j < metrics.size(); j++) {
			Metric m = (Metric) metrics.get(j);
			m.activateDeltaObservers();
//			double val = m.measure();
//			System.out.println(m.getClass()+":"+val);
		}
	}

	//
	// Local properties
	//
	public boolean usedByPointMove = true;
	public boolean usedByPolygonMove = true;
	protected double weighting;
	protected ConcreteUnitaryDiagram cud; 
	protected List deltas = new Vector();
	protected double area;
	protected String name = "test";
	protected String key = "test";
	protected double min = 0;
	protected double max = Double.MAX_VALUE;
	protected boolean firstMeasurement = true;
	
	public static boolean NORMALIZE = true;
	protected static int RANGE = 10;
}
