package rwd.metrics;

import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.deltas.Delta;
import rwd.deltas.NodeContourPointSeparationDelta;

/**
 * @author Octavian Patrascoiu
 * Node-Zone border separation. Between a node and the closest point of its containing zone. 
 * Nodes too close to borders are bad. 
 * 
 * Inverse of summed square of distance.
 */
public class NodeContourPointSeparation
	extends SpiderMetric 
{
	public NodeContourPointSeparation(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
		area = getArea();
	}
	public NodeContourPointSeparation(ConcreteUnitaryDiagram cud) {
		super(cud);
		area = getArea();
	}

	protected void initDeltas(ConcreteUnitaryDiagram cud) {
		if (cud == null) return;
		// For each concrete zone
		deltas = new Vector();
		Set zones = cud.getConcreteZones();
		Iterator i = zones.iterator();
		while (i.hasNext()) {
			// For each foot in current concrete zone
			ConcreteZone cz = (ConcreteZone)i.next();
			List feet = new Vector(cud.getFeet(cz));
			for(int j=0; j<feet.size(); j++) {
				ConcreteSpiderFoot foot = (ConcreteSpiderFoot)feet.get(j);
				// Compute the distance
				Delta delta = new NodeContourPointSeparationDelta(foot, cz);
				deltas.add(delta);
			}
		}
	}

	public double giveScore() {
		double score = super.giveScore();
		score *= area;
		return score;
	}
}
