package rwd.ocl.constraintdiagram;

import rwd.concretesyntax.*;

import rwd.fopl.ast.Expression;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class ConstraintDiagram2Ocl {
	/** Constructor */
	public ConstraintDiagram2Ocl(ConcreteUnitaryDiagram cud, ILog log) {
		// Initialize input members
		this.log = log;
		this.cud = cud;
	}

	/**
	 * Compute the equivalent FOPL
	 **/
	public String getFOPL(String inputString, ConcreteUnitaryDiagram cud) {
		ReadingTree tree = new ReadingTree(inputString, log);
		return tree.readFOPL(cud);
	}

	/**
	 * Compute the equivalent OCL
	 **/
	public String getOCL(String readingTree, ConcreteUnitaryDiagram cud) {
		ReadingTree tree = new ReadingTree(readingTree, log);
		String rest = tree.readOCL(cud);
		return rest;
	}

	/**
	 * Compute the PTC using FOPL concepts
	 **/
	public String getPTCinFOPL(ConcreteUnitaryDiagram cud) {
		ReadingTree tree = new ReadingTree("()", log);
		String rest = tree.getPTCinFOPL(cud);
		return rest;
	}

	/**
	 * Compute the AST of the semantics
	 **/
	public Expression getAST(String inputString, ConcreteUnitaryDiagram cud) {
		ReadingTree tree = new ReadingTree(inputString, log);
		return tree.readAST(cud);
	}

	/**
	 * Compute the dependence graph
	 **/
	public DependenceGraph getDependenceGraph() {
		DependenceGraph graph = new DependenceGraph(cud);
		return graph;
	}
	
	/**
	 * Debug/release flag 
	 **/
	public boolean isDebug() {
		return debug;
	}
	public void setDebug(boolean b) {
		debug = b;
	}

	//
	// Local properties
	//
	/** 
	 * Debug/release flag 
	 **/
	protected boolean debug = false;
	/**
	 * Concrete diagram 
	 **/
	protected ConcreteUnitaryDiagram cud;
	/**
	 * Log 
	 **/
	protected ILog log;
}
