package rwd.ocl.constraintdiagram;

/**
 * @author Octavian Patrascoiu, rnb
 * @version 28.7.05   12:56
 *
 */
//
// Lexical analyser
//
interface Lexer {
	Token nextToken();
}
class StringLexer 
	implements Lexer 
{
	// constructor
	// -----------
	
	public StringLexer(String input) {
		this.tape = new StringInputTape(input);
		this.ch = tape.nextChar();
	}
	
	
	/*
	 * test:
	 *   no whitespace between tokens
	 *     token is (
	 *     token is )
	 *     token is identifier
	 *     token is none of these
	 *     end of tape
	 *   2 or more whitespaces between tokens
	 *     token is (
	 *     token is )
	 *     token is identifier
	 *     token is none of these
	 *     end of tape
	 */
	public Token nextToken() {
		Token tok = new Token();
		// Skip white spaces
		while (Character.isWhitespace((char)ch)) {
			ch = tape.nextChar();				
		}
		// Get next token
		if (ch == -1) {
			tok.code = Token.END;
		} else if (ch == '(') {
			tok.code = Token.LP;
			tok.info = "(";
			ch = tape.nextChar();				
		} else if (ch == ')') {
			tok.code = Token.RP;
			tok.info = ")";
			ch = tape.nextChar();				
		} else if (Character.isJavaIdentifierStart((char)ch)) {
			String name = "";
			do {
				name += (char) ch;
				ch = tape.nextChar();
			} while (Character.isJavaIdentifierPart((char)ch));
			tok.code = Token.ID;
			tok.info = name;
		} else {
			tok.info += (char) ch;
			ch = tape.nextChar();				
		}
		return tok;
	}

	//
	// Local properties
	//
	protected InputTape tape;
	protected int ch;	// next character (or -1 if end of tape)

	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

	public static void main(String args[]) {
		Lexer lexer = new StringLexer("(a (b_b) (c))*");
		for(int i=0; i<11; i++) {
			Token tok = lexer.nextToken();
			System.out.println(tok.code+":"+tok.info);
		}
		System.out.println();
		lexer = new StringLexer("  (  a  (  b_b  )  (  c  )  )  *  ");
		for(int i=0; i<11; i++) {
			Token tok = lexer.nextToken();
			System.out.println(tok.code+":"+tok.info);
		}
	}
	*/
}
