package rwd.ocl.eulerdiagram;

import java.text.SimpleDateFormat;
import java.util.*;

import rwd.abstractsyntax.eulerdiagram.*;

import rwd.abstractsyntax.spiderdiagram.Utilities;

import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EulerDiagram2Uml {
	/** Constructor */
	public EulerDiagram2Uml(List diagrams, ILog log) {
		// Initialize input members
		this.cds = new LinkedHashSet(diagrams);
		this.log = log;
		// Compute cuds, cd, and ud
		this.cuds = new LinkedHashSet();
		Iterator i = cds.iterator();
		while (i.hasNext()) {
			ConcreteDiagram cd = (ConcreteDiagram)i.next();
			Set cuds = cd.getConcreteUnitaryDiagrams();
			this.cuds.addAll(cuds);
		}
	}

	/** Generate the equivalent model - XMI */
	public String makeXMI(String stringExp) {
		// Compute all the diagram names
		Set diagramNames = new LinkedHashSet();
		StringTokenizer strTok = new StringTokenizer(stringExp, " <=>", false);
		while (strTok.hasMoreTokens()) {
			String token = strTok.nextToken();
			if (token.equals("true"));
			else if (token.equals("false"));
			else if (token.equals("not"));
			else if (token.equals("and"));
			else if (token.equals("or"));
			else diagramNames.add(token);
		}
		// Compute all the concrete unitary diagrams used in expression
		Set cuds = new LinkedHashSet();
		Iterator i = diagramNames.iterator();
		while (i.hasNext()) {
			// Compute the diagram
			String dn = (String)i.next();
			ConcreteUnitaryDiagram cud = getConcreteUnitaryDiagram(this.cuds, dn);
			if (cud == null) {
				log.reportError("Cannot find diagram '"+dn+"'");				
			} else {
				cuds.add(cud);
			}
		}
		// Compute all the UML types
		Set umlTypes = getAllUMLTypes(cuds);
		// Compute date
		Calendar cal = Calendar.getInstance();
		String DATE_FORMAT = "dd MMMMM yyyy HH:mm:ss";
		SimpleDateFormat dateFormatter = new java.text.SimpleDateFormat(DATE_FORMAT);
		Date date = cal.getTime();
		// Compute the xmi
		String xmi = "";
		xmi += "<?xml version = '1.0' encoding = 'UTF-8' ?>\n";
		xmi += "<XMI xmi.version = '1.2' xmlns:UML = 'org.omg.xmi.namespace.UML' timestamp = '"+dateFormatter.format(date)+"'>\n";
		xmi += getXMIHeader();
		xmi += getXMIContent(new Vector(umlTypes));
		xmi += "</XMI>\n";
		return xmi;
	}
	
	/** Search for a CUD */
	protected ConcreteUnitaryDiagram getConcreteUnitaryDiagram(Collection cuds, String name) {
		Iterator i = cuds.iterator();
		while (i.hasNext()) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram)i.next();
			if (cud.getName().equals(name)) {
				return cud;
			}
		}
		return null;
	}

	/** Compute all the UML types for a set of CUDs */
	protected Set getAllUMLTypes(Set cuds) {
		Set types = new LinkedHashSet();
		// Collect together contour names
		Iterator i = cuds.iterator();
		Set contourNames = new LinkedHashSet(); 
		while (i.hasNext()) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram)i.next();
			if (cud != null) {
				Iterator j = cud.getConcreteContours().iterator();
				while (j.hasNext()) {
					ConcreteContour cp = (ConcreteContour)j.next();
					contourNames.add(cp.getLabel());
				}
			}
		}
		// Generate all the in/out combinations - abstract zones
		List powerSet = Utilities.powerSet(new Vector(contourNames));
		for (int k = 0; k < powerSet.size(); k++) {
			Vector subset = (Vector) powerSet.get(k);
			Vector rest = new Vector(contourNames);
			rest.removeAll(subset);
			// Compute the corresponding UML type
			String umlType = getUMLType(subset, rest);
			types.add(umlType);
		}
		return types;
	}
	
	//
	// Static methods used in the mapping process
	//
	
	/**
	 * Model and package name
	 **/	
	public static String modelName = "RwD";
	public static String pkgName = "";

	/**
	 * Compute the initial value of the variable 
	 **/
	public static String getUMLType(ConcreteContour c) {
		return getUMLType(c.getContour());
	}
	public static String getUMLType(Contour c) {
		return modelName+"::"+pkgName+c.getLabel();
	}

	/**
	 * Compute the OCL variable name attached to a zone 
	 **/
	public static String getVarName(ConcreteZone cz) {
		return "set"+getUMLType(cz);
	}
	public static String getVarName(Zone az) {
		return "set"+getUMLType(az);
	}

	/** 
	 * Compute the UML type associated with a zone 
	 **/
	public static String getUMLType(ConcreteZone cz) {
		// Get contours
		Zone az = cz.getAbstractZone();
		return getUMLType(az.getContainingContours(), az.getExcludingContours());
	}
	public static String getUMLType(Zone az) {
		// Get contours
		return getUMLType(az.getContainingContours(), az.getExcludingContours());
	}

	/**
	 * Compute the UML type associated with a zone 
	 **/
	public static String getUMLType(List in, List out) {
		// Get contours
		List inc = sort(in);
		// Make the name of the type
		String type = null;
		if (inc.isEmpty()) type = pkgName+"Out";
		else type = pkgName+EulerDiagram2Uml.makeString(inc, "_");
		return type;
	}

	/**
	 * Sort a list of comparable 
	 **/
	protected static List sort(List list) {
		// Sort them
		boolean ok = false;
		do {
			ok = true;
			for(int i=0; i<list.size()-1; i++) {
				Comparable c1 = (Comparable)list.get(i);
				Comparable c2 = (Comparable)list.get(i+1);
				if (c1.compareTo(c2) > 0) {
					list.set(i, c2);
					list.set(i+1, c1);
					ok = false;
				}
			}
			
		}
		while (!ok);
		return list;
	}

	/** 
	 * Compute the initial value of a variable attached to a zone 
	 **/
	public static String getInitValue(ConcreteZone cz) {
		// Compute initial collection
		String set = "OclAny.allInstances()";
		// Compute selection predicate
		List selPred = new Vector();
		Zone az = cz.getAbstractZone();
        
		List inCont = az.getContainingContours();
		for(int i=0; i<inCont.size(); i++) {
			Contour c1 = (Contour)inCont.get(i);
			String cName = EulerDiagram2Uml.getUMLType(c1);
			selPred.add("x.oclIsKindOf("+cName+")");
		}
		List exCont = az.getExcludingContours();
		for(int i=0; i<exCont.size(); i++) {
			Contour c1 = (Contour)exCont.get(i);
			String cName = EulerDiagram2Uml.getUMLType(c1);
			selPred.add(" not x.oclIsKindOf("+cName+")");
		}
		String selPredStr = "";
		for(int i=0; i<selPred.size(); i++) {
			if (i != 0) selPredStr += " and ";
			selPredStr += selPred.get(i);
		}
		// Else, compute selected set
		set += "->select(x : OclAny | "+selPredStr+")";
		return set;
	}
	public static String getInitValue(Zone az) {
		// Compute initial collection
		String set = "OclAny.allInstances()";
		// Compute selection predicate
		List selPred = new Vector();
		List inCont = az.getContainingContours();
		for(int i=0; i<inCont.size(); i++) {
			Contour c1 = (Contour)inCont.get(i);
			String cName = EulerDiagram2Uml.getUMLType(c1);
			selPred.add("x.oclIsKindOf("+cName+")");
		}
		List exCont = az.getExcludingContours();
		for(int i=0; i<exCont.size(); i++) {
			Contour c1 = (Contour)exCont.get(i);
			String cName = EulerDiagram2Uml.getUMLType(c1);
			selPred.add(" not x.oclIsKindOf("+cName+")");
		}
		String selPredStr = "";
		for(int i=0; i<selPred.size(); i++) {
			if (i != 0) selPredStr += " and ";
			selPredStr += selPred.get(i);
		}
		// Else, compute selected set
		set += "->select(x : OclAny | "+selPredStr+")";
		return set;
	}
	public static String getInitValue4Out(Vector exCont) {
		// Compute initial collection
		String set = "OclAny.allInstances()";
		// Compute selection predicate
		List selPred = new Vector();
		for(int i=0; i<exCont.size(); i++) {
			Contour c1 = (Contour)exCont.get(i);
			String cName = EulerDiagram2Uml.getUMLType(c1);
			selPred.add(" not x.oclIsKindOf("+cName+")");
		}
		String selPredStr = "";
		for(int i=0; i<selPred.size(); i++) {
			if (i != 0) selPredStr += " and ";
			selPredStr += selPred.get(i);
		}
		// Else, compute selected set
		set += "->select(x : OclAny | "+selPredStr+")";
		return set;		
	}

	/**
	 * Make a string from a collection using the delimiter
	 **/
	public static String makeString(Collection objs, String del) {
		String res = "";
		Iterator i = objs.iterator();
		while (i.hasNext()) {
			Object obj = i.next();
			res += obj;
			if (i.hasNext()) res += del;
		}
		return res;		
	}
	
	/**
	 * Compute XMI header 
	 **/
	protected String tab = "  ";
	protected String getXMIHeader() {
		// Compute header
		String header = "";
		header += "<XMI.header>\n";
		header += tab+"<XMI.documentation>\n";
		header += tab+tab+"<XMI.exporter>XMI Writer</XMI.exporter>\n";
		header += tab+tab+"<XMI.exporterVersion>1.0</XMI.exporterVersion>\n";
		header += tab+"</XMI.documentation>\n";
		header += "</XMI.header>\n";
		return header;
	}
	
	/**
	 * Compute the XMI content of the UML model 
	 **/
	protected String getXMIContent(List types) {
		String content = "";
		// Open content
		content += "<XMI.content>\n";
		// Open model
		content += tab+"<UML:Model xmi.id = '"+getNewId()+"' name = '"+modelName+"' isSpecification = 'false' isRoot = 'false' isLeaf = 'false' isAbstract = 'false'>\n";
		// Save an id for tagged value definition
		String tagDefRef = getNewId();
		// Add tagged value
		content += getXMI4TaggedValue(tab+tab, getNewId(), tagDefRef);
		// Open namespace
		content += tab+tab+"<UML:Namespace.ownedElement>\n";
		// Add all classes
		for(int i=0; i<types.size(); i++) {
			String type = (String)types.get(i);
			content += getXMI4Class(tab+tab+tab, type, getTypeId(type), tagDefRef);
		}
		// Add all inheritances
		for(int i=0; i<types.size(); i++) {
			String type = (String)types.get(i);
			content += getXMI4InheritanceRelations(tab+tab+tab, type, getTypeId(type), tagDefRef);
		}
		// Close Namespace
		content += tab+tab+"</UML:Namespace.ownedElement>\n";
		// Close model
		content += tab+"</UML:Model>\n";
		// Add tag value definition 
		content += getXMI4TagDefinition(tab, tagDefRef);
		// Close content		
		content += "</XMI.content>\n";
		return content;
	}
	
	/**
	 * Compute XMI for a taggged value 
	 **/
	protected String getXMI4TaggedValue(String indent, String id, String idRef) {
		String res = "";
		res += indent+"<UML:ModelElement.taggedValue>\n";
		res += indent+tab+"<UML:TaggedValue xmi.id = '"+id+"' isSpecification = 'false' dataValue = '"+id+"'>\n";
		res += indent+tab+tab+"<UML:TaggedValue.type>\n";
		res += indent+tab+tab+tab+"<UML:TagDefinition xmi.idref = '"+idRef+"'/>\n";
		res += indent+tab+tab+"</UML:TaggedValue.type>\n";
		res += indent+tab+"</UML:TaggedValue>\n";
		res += indent+"</UML:ModelElement.taggedValue>\n";
		return res;
	}
	
	/**
	 * Compute XMI for a tagged value definition 
	 **/
	protected String getXMI4TagDefinition(String indent, String id) {
		String res = "";
		res += indent+"<UML:TagDefinition xmi.id = '"+id+"' name = 'element.uuid' isSpecification = 'false' tagType = 'element.uuid'>\n";
		res += indent+tab+"<UML:TagDefinition.multiplicity>\n";
		res += indent+tab+tab+"<UML:Multiplicity xmi.id = '"+getNewId()+"'>\n";
		res += indent+tab+tab+tab+"<UML:Multiplicity.range>\n";
		res += indent+tab+tab+tab+tab+"<UML:MultiplicityRange xmi.id = '"+getNewId()+"' lower = '1' upper = '1'/>\n";
		res += indent+tab+tab+tab+"</UML:Multiplicity.range>\n";
		res += indent+tab+tab+"</UML:Multiplicity>\n";
		res += indent+tab+"</UML:TagDefinition.multiplicity>\n";
		res += indent+"</UML:TagDefinition>\n";
		return res;
	}
	
	/**
	 * Compute XMI for a Class 
	 **/ 
	protected String getXMI4Class(String indent, String name, String id, String tagRef) {
		// Compute the xmi and return it 
		String res = "";
		res += indent+"<UML:Class xmi.id = '"+id+"' name = '"+name+"' visibility = 'public' isSpecification = 'false' isRoot = 'false' isLeaf = 'false' isAbstract = 'false' isActive = 'false'>\n";
		res += getXMI4TaggedValue(indent+tab, getNewId(), tagRef);
		res += getXMI4InheritedTypes(indent+tab, name);
		res += indent+"</UML:Class>\n";
		return res;
	}
	
	/**
	 * Compute all the super classes of a Class 
	 **/ 
	protected List getSuperTypes(String typeName) {
		String types[] = typeName.split("_");
		List res = new Vector();
		for(int i=0; i<types.length; i++) res.add(types[i]);
		return res;
	}
	
	/**
	 * Compute XMI for super classes 
	 **/
	protected String getXMI4InheritedTypes(String indent, String typeName) {
		String res = "";
		List types = getSuperTypes(typeName);
		if (types.size() >= 2) {
			res += indent+"<UML:GeneralizableElement.generalization>\n";
			for(int i=0; i<types.size(); i++) {
				String superType = (String)types.get(i);
				res += indent+tab+"<UML:Generalization xmi.idref = '"+getGeneralizationId(typeName+"->"+superType)+"'/>\n";
			}
			res += indent+"</UML:GeneralizableElement.generalization>\n";
		}
		return res;
	}
	
	/**
	 * Compute XMI for all inheritance relations
	 **/
	protected String getXMI4InheritanceRelations(String indent, String name, String id, String tagRef) {
		// Compute the xmi and return it 
		String res = "";
		// Compute the list of inherited types
		List superTypes = getSuperTypes(name);
		if (superTypes.size() >= 2) {
			// Scan all the inherited types and generate a inheritance relation
			for(int i=0; i<superTypes.size(); i++) {
				String superType = (String)superTypes.get(i);
				res += indent+"<UML:Generalization xmi.id = '"+getGeneralizationId(name+"->"+superType)+"' isSpecification = 'false'>\n";
				res += getXMI4TaggedValue(indent+tab, getNewId(), tagRef);
				res += indent+tab+"<UML:Generalization.child>\n";
				res += indent+tab+tab+"<UML:Class xmi.idref = '"+id+"'/>\n";
				res += indent+tab+"</UML:Generalization.child>\n";
				res += indent+tab+"<UML:Generalization.parent>\n";
				res += indent+tab+tab+"<UML:Class xmi.idref = '"+getTypeId(superType)+"'/>\n";
				res += indent+tab+"</UML:Generalization.parent>\n";
				res += indent+"</UML:Generalization>\n";
			}			
		}
		return res;
	}

	/**
	 * Provide unique ids
	 **/
	private int idNo = 0;
	protected Map type2id = new HashMap();
	protected Map generalization2id = new HashMap();
	protected String getNewId() {
		return "id-"+ ++idNo;
	}
	protected String getTypeId(String typeName) {
		String id = (String)type2id.get(typeName);
		if (id == null) {
			id = getNewId();
			type2id.put(typeName, id);
		} 
		return id;
	}
	protected String getGeneralizationId(String gName) {
		String id = (String)generalization2id.get(gName);
		if (id == null) {
			id = getNewId();
			generalization2id.put(gName, id);
		} 
		return id;
	}
	
	// Members
	/** Concrete diagrams */
	protected Set cds;
	/** Concrete unitary diagrams */
	protected Set cuds;
	/** Log */
	protected ILog log;
	/** Display meaning for missing zones */
	protected static boolean MEANING_FOR_MISSING_ZONES = true;
}
