package rwd.ocl.spiderdiagram;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import rwd.gui.editor.EditorFileFilter;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class Ocl2SpiderDiagramDialog
	extends JDialog
{
	public Ocl2SpiderDiagramDialog(JFrame parent, String title, ILog log) {
		super(parent, title);	
		getContentPane().setLayout(new GridBagLayout());
		this.log = log;
		
		//
        //--- Create and fill PROJECT panel
        //
		generalPanel = new JPanel(new GridBagLayout());
	    //--- Give the panel a border gap of 5 pixels ---
	    generalPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
	    //--- Get the constraints ---
	    GridBagConstraints c = new GridBagConstraints();
	    //--- Add space around all components to avoid clutter ---
	    c.insets = new Insets(2, 2, 2, 2);
	    c.ipadx = 5;
	    c.ipady = 5;
	    c.weightx = 0.0;
	    c.weighty = 0.0;
		//--- Initialize common variables ---
		int line = 0;
		int column = 0;
    	EmptyBorder border = new EmptyBorder(new Insets(0, 0, 0, 10));
    	Dimension buttonSize = new Dimension(80, 25);
    	int fieldSize = 30;
		JLabel label;
		JButton button;
		File dir = new File(System.getProperty("user.dir"));
        inputChooser = new JFileChooser(dir);	

//    	//--- XMI label ---
//    	line = 0;
//    	column = 0;
//		label = new JLabel("Input XMI file");
//	    label.setBorder(border);
//	    c.gridx = column;
//	    c.gridy = line;
//	    c.anchor = GridBagConstraints.WEST;
//	    generalPanel.add(label, c);
//		//--- XMI area ---
//		xmiField = new JTextField(fieldSize);
//		c.gridx = column+1;
//		c.gridy = line;
//		generalPanel.add(xmiField, c);
//		//--- XMI button ---
//		button = new JButton();
//		button.setPreferredSize(buttonSize);
//		button.setText("Browse");
//		button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
//		button.setVerticalAlignment(javax.swing.SwingConstants.TOP);
//		button.addActionListener(new java.awt.event.ActionListener() {
//			public void actionPerformed(java.awt.event.ActionEvent evt) {
//				chooseXMIAction(evt);
//			}
//		});
//		c.gridx = column+2;
//		c.gridy = line++;
//		generalPanel.add(button, c);
    	//--- OCL label ---
		label = new JLabel("OCL file");
	    label.setBorder(border);
	    c.gridx = column;
	    c.gridy = line;
	    c.anchor = GridBagConstraints.WEST;
	    generalPanel.add(label, c);
        //--- OCL area ---
        oclField = new JTextField(fieldSize);
	    c.gridx = column+1;
	    c.gridy = line;
	    generalPanel.add(oclField, c);
    	//--- LICENSE button ---
        button = new JButton();
        button.setPreferredSize(buttonSize);
        button.setText("Browse");
        button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        button.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chooseOCLAction(evt);
            }
        });
	    c.gridx = column+2;
	    c.gridy = line++;
	    generalPanel.add(button, c);        
    
		//
        //--- Create and fill CONFIRM panel ---
        //
        JPanel confirmPanel = new JPanel(new GridBagLayout());
        //--- OK button ---
		JButton okButton = new JButton("OK");
		okButton.setPreferredSize(buttonSize);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
	    c.gridx = 0;
	    c.gridy = 0;
		confirmPanel.add(okButton);
        //--- Cancel button ---
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setPreferredSize(buttonSize);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });
	    c.gridx = 0;
	    c.gridy = 1;
		confirmPanel.add(cancelButton);

	    //--- Add TAB panel to MAIN panel ---
	    c.gridx = 0;
	    c.gridy = 0;
		getContentPane().add(generalPanel, c);

		//--- Add CONFIRM to MAIN panel ---		
	    c.gridx = 0;
	    c.gridy = 1;
	    c.anchor = GridBagConstraints.CENTER;
	    getContentPane().add(confirmPanel, c);
    	
		// Add extensions for filter
//		xmiFilter.addExtension("xmi");
//		xmiFilter.addExtension("xml");
		oclFilter.addExtension("ocl");
		
        // Put frame in the middle of the screen
        pack();
        Dimension screen = getToolkit().getScreenSize();
        int width  = this.getSize().width+20;
        int height = this.getSize().height+20;
        int left = (screen.width - width)/2;
        int top = (screen.height - height)/2;
        setBounds(left, top, width, height);
	}

	//--- Choose the XMI file ---
    protected void chooseXMIAction(java.awt.event.ActionEvent evt) {
    	//--- Create INPUT file chooser ---
//        inputChooser.setFileFilter(xmiFilter);
        // Add your handling code here:
        int returnVal = inputChooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File inputFile = inputChooser.getSelectedFile();
//            xmiField.setText(inputFile.getAbsolutePath());            
        }          
    }

	//--- Choose the OCL file ---
    protected void chooseOCLAction(java.awt.event.ActionEvent evt) {
    	//--- Create INPUT file chooser ---
        inputChooser.setFileFilter(oclFilter);
        // Add your handling code here:
        int returnVal = inputChooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File inputFile = inputChooser.getSelectedFile();
            oclField.setText(inputFile.getAbsolutePath());            
        }          
    }

	//--- OK action ---
	protected int returnVal = JFileChooser.CANCEL_OPTION;
    protected void okAction(java.awt.event.ActionEvent evt) {
    	returnVal = JFileChooser.APPROVE_OPTION;
//    	String xmiFileName = getXMIFileName();
    	String oclFileName = getOCLFileName();
		if (oclFileName != null && oclFileName.length() != 0)  	
//    		if (xmiFileName != null && xmiFileName.length() != 0)
				dispose();
    }

	//--- Cancel action ---
    protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
    	dispose();
    }

//	//--- Get XMI file name ----
//	public String getXMIFileName() {
//		return xmiField.getText();
//	}

	//--- Get XMI file name ----
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}
	
	//--- Get OCL file name ----
	public String getOCLFileName() {
		return oclField.getText();
	}

    protected JFileChooser inputChooser;

	protected JPanel generalPanel;
//	protected JTextField xmiField;
	protected JTextField oclField;	

//	protected EditorFileFilter xmiFilter = new EditorFileFilter("XMI Filter");
	protected EditorFileFilter oclFilter = new EditorFileFilter("*.ocl");
	
	protected ILog log;
}
