/***********************************************************************
 * Copyright (c) 2003 University of Kent.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.cs.kent.ac.uk/projects/ocl/CPL.html
 ***********************************************************************/

/**
 * 
 * @author Octavian Patrascoiu
 *
 */

package rwd.ocl.spiderdiagram.parser;

import java.io.Reader;

import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.ocl20.syntax.ast.contexts.ContextDeclarationAS;

public interface OclParser {
  /** 
   * Parse the given string 
   * without debugging info, 
   * reporting errors on default log: System.out 
   * 
   * The result is null if an error is reported 
   * */
  public ContextDeclarationAS parse(String body);

  /** 
   * Parse the given string 
   * without debugging info, 
   * reporting errors on given log 
   * 
   * The result is null if an error is reported
   *  
   * */
  public ContextDeclarationAS parse(String body, ILog log);

  /** 
   * Parse the given string 
   * with/without debugging info, 
   * reporting errors on given log
   * 
   * The result is null if an error is reported 
   *  
   * */
  public ContextDeclarationAS parse(String body, ILog log, boolean debug);

  /** 
   * Parse the given file 
   * without debugging info, 
   * reporting errors on default log: System.out
   *  
   * The result is null if an error is reported 
   * 
   * */
  public ContextDeclarationAS parse(Reader reader);

  /** 
   * Parse the given file 
   * without debugging info, 
   * reporting errors on given log
   *  
   * The result is null if an error is reported 
   * 
   * */
  public ContextDeclarationAS parse(Reader reader, ILog log);

  /** 
   * Parse the given file 
   * with/without debugging info, 
   * reporting errors on given log
   * 
   * The result is null if an error is reported
   *  
   * */
  public ContextDeclarationAS parse(Reader reader, ILog log, boolean debug);

  /** 
   * Check if the parser encountered an error 
   * 
   * */
  public boolean hasErrors();

  /** 
   * Check if the parser encountered a warning 
   * 
   * */
  public boolean hasWarnings();
}
