package rwd.rules;

import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.choicespec.*;
import rwd.rules.inputspec.RuleAddContour;

/**
 * @author jaf28
 */
public class UnitaryProofGetterRandom implements UnitaryProofGetter {
	private int maxNumContours;
	UnitaryDiagramGetterRandom dgr;
	RuleWithChoice[] rulesWithChoice;
	Vector maxContourSet;

	public UnitaryProofGetterRandom(
		int numContours,
		int maxNumContours,
		int numSpiders) {
		this.maxNumContours = maxNumContours;
		dgr = new UnitaryDiagramGetterRandom(numContours, numSpiders);
		rulesWithChoice =
			new RuleWithChoice[] {
				(RuleWithChoice) RuleUnshadeZone.getRule(),
				(RuleWithChoice) RuleIntroduceShadedZone.getRule(),
				(RuleWithChoice) RuleDeleteShadedZone.getRule(),
				(RuleWithChoice) RuleAddSpiderFoot.getRule(),
				(RuleWithChoice) RuleDeleteSpider.getRule(),
				(RuleWithChoice) RuleDeleteContour.getRule(),
				};
		maxContourSet = new Vector();

		if (numContours > 0) {
			maxContourSet.add(Contour.getContour("a"));
		}
		if (numContours > 1) {
			maxContourSet.add(Contour.getContour("b"));
		}
		if (numContours > 2) {
			maxContourSet.add(Contour.getContour("c"));
		}
		if (numContours > 3) {
			maxContourSet.add(Contour.getContour("d"));
		}
		if (numContours > 4) {
			System.out.println("too many contours to handle");
			System.exit(0);
		}

	}


    private int numAttempts=0;
	public Proof getProof(int depth) {

		// get a random start diagram
		UnitaryDiagram d = (UnitaryDiagram) dgr.getDiagram();

		// start off proof
		Proof p = new Proof(d);
		for (int numSteps = 0; numSteps < depth; numSteps++) {
			d = (UnitaryDiagram) p.getLastDiagram();
			//System.out.print("got diag to work from  ");
			// choose a random rule
			int numberOfRules = rulesWithChoice.length + 1;
			//System.out.println(
			//	"there are " + numberOfRules + " rules");
			ruleSearch : for (
				int ruleAttempt = 0; ruleAttempt < 10; ruleAttempt++) {
				//System.out.println(
				//	"seeking a rule to apply...");
				int ruleNumber = (int) ((Math.random() * (numberOfRules)));
				//System.out.println("rule " + ruleNumber);
				if (ruleNumber == 0) {
					Rule rac = RuleAddContour.getRule();
					// find out if it's applicable
					Vector conts = (Vector) maxContourSet.clone();
					//System.out.println("in doStepsTo d1 is "+d1);
					conts.removeAll(d.getCloneOfContourSet());
					for (int ci = 0; ci < conts.size(); ci++) {
						Contour ct = (Contour) conts.get(ci);
						if (!d.getCloneOfContourSet().contains(ct)) {
							// apply rule!
							RuleAppliesWithString rws =
								new RuleAppliesWithString(d, "" + ct);
							UnitaryDiagram newEnd =
								(UnitaryDiagram) rac.applyRule(rws);
							if (newEnd.equals(p.getFirstDiagram())) {
								continue ruleSearch;
							} else {
								p.addStepAtEnd(rac, rws, newEnd);
								//System.out.println(
								//	""+rac+" applied");
								//System.out.print("" + ruleNumber);
								break ruleSearch;
							}
						}
					}
				} else {
					RuleWithChoice rwc = rulesWithChoice[ruleNumber - 1];
					Vector specs = rwc.getApplicationPossibilities(d);
					int numWays = specs.size();
					if (numWays == 0) {
						continue ruleSearch;
					} else {
						int wayNumber = (int) (Math.random() * numWays);
						RuleApplicationSpec ras =
							(RuleApplicationSpec) specs.get(wayNumber);
						Diagram newEnd = rwc.applyRuleToPart(d, ras);
						if (newEnd.equals(p.getFirstDiagram())) {
							continue ruleSearch;
						} else {
							p.addStepAtEnd(rwc, ras, newEnd);
							//System.out.println(
							//	"" + rwc + " applied");
							//System.out.print("" + ruleNumber);
							break ruleSearch;
						}
					}
				}
			}
		}
        if(p.numSteps()==depth){
            numAttempts = 0;
            return p;
        } else if(numAttempts<20){
            numAttempts ++;
            return getProof(depth);
        } else {
            return null; // can't find proof with this spec
        }
	}
    
	public static void main(String args[]) {
		// build a proof generator
		Proof p = new UnitaryProofGetterRandom(3, 3, 3).getProof(1);
		System.out.println("proof is " + ProofFileUser.toSaveString(p));
		for (int i = 0; i < 200; i++) {
			// get random proofs
			// save them to file
			ProofFileUser.addProofFreshFile(p, "test");
			// read it back
			p = ProofFileUser.getProofFromStart("test");
			ProofFileUser.addProofAppend(p, "test2");
			// read it back
			p = ProofFileUser.getProofFromStart("test2");
			System.out.println("done " + i);
		}
		System.out.println("done OK");
	}

}
