package rwd.rules.autoproof;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import java.util.Vector;

import rwd.rules.*;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDandDandD;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.choicespec.RuleUnshadeZone;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.gui.ProofManager;
import rwd.rules.application.RuleAppliesToSpider;
import rwd.rules.choicespec.RuleDeleteSpider;
import rwd.rules.application.RuleAppliesToDiagramAndDiagram;
import rwd.rules.choicespec.RuleExcludedMiddle;
import rwd.rules.choicespec.RuleReverseExcludedMiddle;
import rwd.rules.choicespec.RuleWithChoice;
import rwd.rules.choicespec.RuleReverseOrIdempotent;
import rwd.rules.inputspec.RulePToPOrRubbish;

public class AutoProofWriterShortenedWithNegation {
    
    private static boolean debug = false;

    private static Proof concatenateProofs(
        Proof p1,
        Proof p2) { //ProofManager.showNewProof(p1,false);
        //ProofManager.showNewProof(p2,false);
        Proof result = p1.cloneProof();
        //ProofManager.showNewProof(p1,false);
        //ProofManager.showNewProof(p2,false);
        if (!result.getLastDiagram().equals(p2.getFirstDiagram())) {
            debug(
                "result.getLastDiagram() is "
                    + DiagramFileUser.toSaveString(result.getLastDiagram()));
            debug(
                "p2.getFirstDiagram() is "
                    + DiagramFileUser.toSaveString(p2.getFirstDiagram()));
            //Diagram d1 = (Diagram)result.getLastDiagram();
            //Diagram d2 = (Diagram)p2.getFirstDiagram();
            //debug(""+cd1.getChildren().size());
            //debug(""+cd2.getChildren().size());
            return null;
        }
        while (p1.goToNextStep()) {
        }
        p2.goToStart();
        if (result.trashLastStep()) {
            result.addStepAtEnd(
                p1.getPreviousRule(),
                p1.getPreviousRuleApplicationSpec(),
                p2.getFirstDiagram());
        } else {
            return p2;
        }
        while (p2.goToNextStep()) {
            result.addStepAtEnd(
                p2.getPreviousRule(),
                p2.getPreviousRuleApplicationSpec(),
                p2.getCurrentDiagram());
        }
        return result;
    }

    private static Proof startToAlpharised(Diagram d, Diagram other) {

        debug("vennifying ...");
        Proof result = VennMaker.addZones(d);
        //new ProofViewer(result,false,true).show();
        debug("done");
        debug("splitting spiders ...");
        result =
            concatenateProofs(
                result,
                SpiderSplitter.splitAllSpiders(result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();
        debug("done");
        debug("pushNotsToLeaves");
        result =
            concatenateProofs(
                result,
                PushNotsToLeaves.pushNotsToLeaves(result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();
        debug("done");
        debug("get rid of nots");
        result =
            concatenateProofs(
                result,
                NotEliminator.getRidOfNots(result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();        
        debug("done");        
        //ProofManager.showNewProof(result,true,false);
        //ProofFileUser.saveToFile(result);
        debug("match contours again");
        result =
            concatenateProofs(
                result,
                ContourSetEqualiser.makeContourSetsEqual(result.getLastDiagram(),result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();        
        debug("done");        
        //ProofManager.showNewProof(result,true,false);
        //ProofFileUser.saveToFile(result);                
        debug("turning to DNF ...");
        result =
            concatenateProofs(
                result,
                MakerOfDisNF.turnToDisjNF(result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();
        debug("done");
        debug("applying 'combine' ...");
        result =
            concatenateProofs(
                result,
                Combiner.combineWherePossible(result.getLastDiagram()));
        //ProofManager.showNewProof(result,false,true);
        debug("done");
        debug("removing occurences of False ...");
        result =
            concatenateProofs(
                result,
                FalseRemover.removeFalse(result.getLastDiagram()));
        //new ProofViewer(result,false,true).show();
        debug("done");
        //debug("after combining where possible ~~~~~~~~~~~~~~~~~~");
        //debug(""+result.getLastDiagram().toDebugString());
        //debug("after removing false ~~~~~~~~~~~~~~~~~~");
        //debug(""+result.getLastDiagram().toDebugString());
        debug("removing duplicates ...");
        result =
            concatenateProofs(
                result,
                DuplicatesRemover.removeDupes(result.getLastDiagram()));
        //ProofManager.showNewProof(result,false,true);
        debug("done");
        return result; // concatenates steps
    }

    public static Proof writeProof(Diagram premis, Diagram conclusion)
        throws NoProofException {
        debug("trying to generate proof");
        //debug("from premis =  "+premis.toDebugString());
        //debug("to conclusion =  "+conclusion.toDebugString());
        //debug("go through to alpharise with premis...");

        Proof start =
            ContourSetEqualiser.makeContourSetsEqual(premis, conclusion);
        Proof endBackwards =
            ContourSetEqualiser.makeContourSetsEqual(conclusion, premis);
        debug("made contour sets equal");
        Diagram lhstmp = start.getLastDiagram();
        Diagram rhstmp = endBackwards.getLastDiagram();
        start = concatenateProofs(start, startToAlpharised(lhstmp, rhstmp));
        debug("gone through to alpharised for lhs");
        endBackwards =
            concatenateProofs(endBackwards, startToAlpharised(rhstmp, lhstmp));
        debug("gone through to alpharised for rhs");
        //ProofManager.showNewProof(start,false);
        //ProofManager.showNewProof(endBackwards,false);
        //if(1==1){return null;}
        debug(
            "start has "
                + start.getLastDiagram().getUnitaryParts().size()
                + " children");
        debug(
            "endBackwards has "
                + endBackwards.getLastDiagram().getUnitaryParts().size()
                + " children");
        Vector rhsChildren;
        Diagram rhs = endBackwards.getLastDiagram();
        try { // we know that this is an or-junction of unitary alpha-diagrams
            rhsChildren = ((OrCompoundDiagram) rhs).getChildren();
        } catch (ClassCastException ex) {
            rhsChildren = new Vector();
            rhsChildren.add(endBackwards.getLastDiagram());
        }
        int[] numberOfTimesUsedInExcMiddleReversed =
            new int[rhsChildren.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            numberOfTimesUsedInExcMiddleReversed[i] = 0;
        }
        Vector appPossibilities =
            (
                (RuleWithChoice) RuleReverseExcludedMiddle
                    .getRule())
                    .getApplicationPossibilities(
                rhs);
        for (int i = 0; i < appPossibilities.size(); i++) {
            RuleAppliesToDandDandD raddd =
                (RuleAppliesToDandDandD) appPossibilities.get(i);
            UnitaryDiagram ud1 = (UnitaryDiagram) raddd.other1;
            UnitaryDiagram ud2 = (UnitaryDiagram) raddd.other2;
            //debug("Rule app has ");
            //debug("ud1 is "+ud1);
            //debug("ud2 is "+ud2);
            for (int j = 0; j < rhsChildren.size(); j++) {
                UnitaryDiagram inArray = (UnitaryDiagram) rhsChildren.get(j);
                if (inArray == ud1) {
                    numberOfTimesUsedInExcMiddleReversed[j]++;
                }
                if (inArray == ud2) {
                    numberOfTimesUsedInExcMiddleReversed[j]++;
                }
            }
        }

        //debug("number of times it's possible to apply");
        //debug("rev exc middle = ");
        //for(int i=0; i<rhsChildren.size(); i++)
        //{ debug(""+numberOfTimesUsedInExcMiddleReversed[i]); }
        //debug("");

        //debug("rhs before duplication is "+rhs.toDebugString());
        int shift = 0;
        for (int i = 0; i < rhsChildren.size(); i++) {
            while (numberOfTimesUsedInExcMiddleReversed[i] > 1) {
                numberOfTimesUsedInExcMiddleReversed[i]--;
                Rule r = RulePToPOrRubbish.getRule();
                rhs = endBackwards.getLastDiagram();
                Diagram currentPart = rhs.getPartAt(new int[] { shift + i });
                RuleApplicationSpec ras =
                    new RuleAppliesToDiagramAndDiagram(
                        currentPart,
                        currentPart);
                shift++;
                debug("duplicate a diagram on rhs");
                endBackwards.addStepAtEnd(r, ras, r.applyRuleToPart(rhs, ras));
            }
        }
        rhs = endBackwards.getLastDiagram();
        try {
            rhs = (OrCompoundDiagram) endBackwards.getLastDiagram();
            //debug("rhs after duplication is "+rhs.toDebugString());

            Rule rrem = RuleReverseExcludedMiddle.getRule();
            Vector apps =
                ((RuleWithChoice) rrem).getApplicationPossibilities(rhs);
            while (apps.size() > 0) {
                RuleAppliesToDandDandD raddd =
                    (RuleAppliesToDandDandD) apps.get(0);
                //debug("before applying reverse excluded middle");
                //debug("rhs is "+rhs.toDebugString());
                //debug("before applying reverse excluded middle");
                //debug("spec is or,p,q:"+raddd);

                debug(
                    "merge two diagrams on rhs : exc mid reversed");
                endBackwards.addStepAtEnd(
                    rrem,
                    raddd,
                    rrem.applyRuleToPart(rhs, raddd));
                rhs = (OrCompoundDiagram) endBackwards.getLastDiagram();
                //debug("after applying reverse excluded middle");
                //debug("rhs is "+rhs.toDebugString());
                apps = ((RuleWithChoice) rrem).getApplicationPossibilities(rhs);
            }
        } catch (ClassCastException ex) {
        }
        rhs = endBackwards.getLastDiagram();

        debug(
            "rhs after contraction has "
                + rhs.getUnitaryParts().size()
                + " children");

        try {
            rhsChildren = ((OrCompoundDiagram) rhs).getChildren();
        } catch (ClassCastException ex) {
            rhsChildren = new Vector();
            rhsChildren.add(rhs);
        }
        Vector zones = null;
        try {
            zones = ((UnitaryDiagram) (rhsChildren.get(0))).getCloneOfZoneSet();
        } catch (ClassCastException x) {
            // what to do???
            ProofManager.showNewProof(endBackwards, false, true);
        }
        boolean[][] rhsShadedZones = null;
        try{
        rhsShadedZones =
            new boolean[rhsChildren.size()][zones.size()];
        } catch (NullPointerException x){
            DiagramFileUser.saveToFile(premis);
            DiagramFileUser.saveToFile(conclusion);
        }
        int[][] rhsSpiderCount = new int[rhsChildren.size()][zones.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            UnitaryDiagram ud = (UnitaryDiagram) rhsChildren.get(i);
            for (int j = 0; j < zones.size(); j++) {
                Zone z = (Zone) zones.get(j);
                rhsShadedZones[i][j] = ud.containsShaded(z);
                rhsSpiderCount[i][j] = 0;
                for (int k = 0; k < ud.getSpiders().size(); k++) {
                    Spider s = (Spider) ud.getSpiders().get(k);
                    if (s.getHabitat().contains(z)) {
                        rhsSpiderCount[i][j]++;
                    }
                }
            }
        }
        boolean[] hasBeenMatchedBySomethingOnLHS =
            new boolean[rhsChildren.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            hasBeenMatchedBySomethingOnLHS[i] = false;
        }
        boolean[] isASecondMatchForSomethingOnTheRHS;
        debug("have built info arrays for rhs");
        bigLoop : while (true) // enables start-again
            { 
            debug("at start of loop for exc middle");
            //ProofManager.showNewProof(start,false,true); CRASHES!!
            //ProofFileUser.saveToFile(start);
            //ProofFileUser.saveToFile(endBackwards);
            //debug(""+DiagramFileUser.toSaveString(start.getLastDiagram()));
            //debug(""+DiagramFileUser.toSaveString(endBackwards.getLastDiagram()));
            //ProofManager.showNewProof(endBackwards,false,true);
            //if(1==1){return null;}
            
            for (int i = 0; i < rhsChildren.size(); i++) {
                hasBeenMatchedBySomethingOnLHS[i] = false;
            }
            Vector lhsChildren;
            Diagram lhs = start.getLastDiagram();
            try {
                lhsChildren = ((OrCompoundDiagram) lhs).getChildren();
            } catch (ClassCastException ex) {
                lhsChildren = new Vector();
                lhsChildren.add(lhs);
            }
            isASecondMatchForSomethingOnTheRHS =
                new boolean[lhsChildren.size()];
            for (int i = 0; i < lhsChildren.size(); i++) {
                isASecondMatchForSomethingOnTheRHS[i] = false;
            }
            //debug("the rhs is made up of .........................");
            // take di on the left - iterate through
            // just for debugging...................
            //for(int i=0; i<lhsChildren.size(); i++)
            //{ Diagram temp = (Diagram)lhsChildren.get(i);
            //debug("temp is "+temp.toDebugString());
            //}
            // just for debugging..................
            lhsLoop : for (
                int i = 0;
                    i < lhsChildren.size();
                    i++) { 
                //Diagram temp = (Diagram)lhsChildren.get(i);
                //debug("temp is "+temp.toDebugString());
                UnitaryDiagram di = (UnitaryDiagram) lhsChildren.get(i);
                boolean[] isShaded = new boolean[zones.size()];
                int[] spiderCount = new int[zones.size()];
                for (int j = 0; j < zones.size(); j++) {
                    Zone z = (Zone) zones.get(j);
                    isShaded[j] = di.containsShaded(z);
                    spiderCount[j] = 0;
                    for (int k = 0; k < di.getSpiders().size(); k++) {
                        Spider s = (Spider) di.getSpiders().get(k);
                        if (s.getHabitat().contains(z)) {
                            spiderCount[j]++;
                        }
                    }
                    //debug("spider count for zone "+zones.get(j)+" is "+spiderCount[j]);
                }
                //debug("have built info arrays");
                // if di matches any Dj then take next di
                for (int j = 0; j < rhsChildren.size(); j++) {
                    UnitaryDiagram dj = (UnitaryDiagram) rhsChildren.get(j);
                    if (di.equals(dj)) {
                        if (hasBeenMatchedBySomethingOnLHS[j]) {
                            isASecondMatchForSomethingOnTheRHS[i] = true;
                        }
                        hasBeenMatchedBySomethingOnLHS[j] = true;
                        continue lhsLoop;
                    }
                }
                // if di can reduce to any Dj then reduce it and start again
                for (int j = 0; j < rhsChildren.size(); j++) {
                    //debug("considering rhs diag #"+j);
                    boolean canReduceDiToDj = true;
                    for (int k = 0; k < zones.size(); k++) {
                        if (spiderCount[k] < rhsSpiderCount[j][k]
                            || (isShaded[k]
                                && rhsShadedZones[j][k]
                                && spiderCount[k] != rhsSpiderCount[j][k])
                            || (rhsShadedZones[j][k] && !isShaded[k])) {
                            canReduceDiToDj = false;
                            break;
                        }
                    }
                    if (canReduceDiToDj) {
                        // do the reduction here and then start again
                        // delete shading if necessary
                        // delete spiders
                        //debug("found a rhs partner for lhs diag "+i);
                        for (int k = 0; k < zones.size(); k++) {
                            if (isShaded[k] && !rhsShadedZones[j][k]) {
                                Rule r = RuleUnshadeZone.getRule();
                                RuleApplicationSpec ras =
                                    new RuleAppliesToZone(
                                        di,
                                        (Zone) zones.get(k));
                                int[] coords = lhs.getCoords(di);
                                lhs = r.applyRuleToPart(lhs, ras);
                                di = (UnitaryDiagram) lhs.getPartAt(coords);
                                //debug(
                                //    "about to unshade a zone on lhs");
                                start.addStepAtEnd(r, ras, lhs);
                            }
                        }
                        for (int k = 0; k < zones.size(); k++) {
                            int numExtraSpiders =
                                spiderCount[k] - rhsSpiderCount[j][k];
                            while (numExtraSpiders > 0) {
                                numExtraSpiders--;
                                Spider s = null;
                                for (int q = 0;
                                    q < di.getSpiders().size();
                                    q++) {
                                    s = (Spider) di.getSpiders().get(q);
                                    if (s
                                        .getHabitat()
                                        .contains(zones.get(k))) {
                                        break;
                                    }
                                }
                                Rule r = RuleDeleteSpider.getRule();
                                RuleApplicationSpec ras =
                                    new RuleAppliesToSpider(di, s);
                                int[] coords = lhs.getCoords(di);
                                lhs = r.applyRuleToPart(lhs, ras);
                                di = (UnitaryDiagram) lhs.getPartAt(coords);
                                //debug(
                                //    "about to remove a spider from lhs");
                                start.addStepAtEnd(r, ras, lhs);
                            }
                        }
                        continue bigLoop;
                    }
                }
                //debug("need to work out metrics");
                // build a set forMetricMeasurement containing all Dj
                Vector forMetricMeasurement =
                    (Vector) rhsChildren.clone();
                // if di contradicts any Dj then remove Dj then forMetricMeasurement
                for (int j = 0; j < rhsChildren.size(); j++) {
                    UnitaryDiagram dj = (UnitaryDiagram) rhsChildren.get(j);
                    //debug("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
                    //debug("looking for a contradiction between");
                    //debug("di="+di.toDebugString());
                    //debug("dj="+dj.toDebugString());
                    boolean diContradictsDj = false;
                    for (int k = 0; k < zones.size(); k++) {
                        if ((isShaded[k]
                            && spiderCount[k] < rhsSpiderCount[j][k])
                            || (rhsShadedZones[j][k]
                                && rhsSpiderCount[j][k] < spiderCount[k])) {
                            diContradictsDj = true;
                            break;
                        }
                    }
                    if (diContradictsDj) {
                        forMetricMeasurement.remove(dj);
                    }
                }
                // if forMetricMeasurement is empty then use di to build a model c/ex
                if (forMetricMeasurement.size() == 0) {
                    //debug("can't prove this!!! see "+di.toDebugString());
                    throw new NoProofException(di);
                }
                // measure metrics for di and all Dj in ForMetricMeasurement
                Vector metrics = new Vector();
                for (int j = 0; j < forMetricMeasurement.size(); j++) {
                    UnitaryDiagram dj =
                        (UnitaryDiagram) forMetricMeasurement.get(j);
                    //debug("calculating metrics for "+di.toDebugString()+
                    //                                      " and "+dj.toDebugString());
                    int[] metric = new int[zones.size()];
                    for (int k = 0; k < zones.size(); k++) {
                        metric[k] =
                            rhsSpiderCount[rhsChildren.indexOf(dj)][k]
                                - spiderCount[k];
                        //debug("lhs has "+spiderCount[k]+" spiders in "+zones.get(k));
                        //debug("rhs has "+rhsSpiderCount[rhsChildren.indexOf(dj)][k]+" spiders in "+zones.get(k));
                        if (metric[k] < 0) {
                            metric[k] = 0;
                        }
                        //debug("metric for "+zones.get(k)+" for spiders is "+metric[k]);
                        if (!isShaded[k]
                            && rhsShadedZones[j][k]) { //debug("increment metric for shading");
                            metric[k]++;
                        }
                    }
                    metrics.add(metric);
                }
                int lowestTotal = Integer.MAX_VALUE;
                int lowestIndex = 0;
                // pick the Dj with smallest metric sum
                for (int j = 0; j < metrics.size(); j++) {
                    int[] metric = (int[]) metrics.get(j);
                    int total = 0;
                    for (int k = 0; k < metric.length; k++) {
                        total = total + metric[k];
                    }
                    if (total < lowestTotal) {
                        lowestTotal = total;
                        lowestIndex = j;
                    }
                }
                // use the metric outcome for di, Dj to apply excluded middle to di
                int[] metric = (int[]) metrics.get(lowestIndex);
                UnitaryDiagram target =
                    (UnitaryDiagram) rhsChildren.get(lowestIndex);
                UnitaryDiagram source = di;
                for (int j = 0; j < metric.length; j++) {
                    while (metric[j] > 0) {
                        //debug("metric["+j+"] is "+metric[j]);
                        // metric measurement is +ve
                        // find jth zone
                        Zone z = (Zone) zones.get(j);
                        Rule r = RuleExcludedMiddle.getRule();
                        RuleApplicationSpec ras =
                            new RuleAppliesToZone(source, z);
                        int[] coords = lhs.getCoords(source);
                        // apply excluded middle to source
                        lhs = (OrCompoundDiagram) r.applyRuleToPart(lhs, ras);
                        // add rule to "start"
                        //debug(
                        //    "about to apply excluded middle to lhs");
                        start.addStepAtEnd(r, ras, lhs);
                        if (metric[j] != 1
                            || !rhsShadedZones[rhsChildren.indexOf(target)][j]) {
                            try {
                                coords[coords.length - 1]++;
                            } catch (ArrayIndexOutOfBoundsException x) {
                                coords = new int[] { 1 };
                            }
                        }
                        if (coords.length == 0) {
                            coords = new int[] { 0 };
                        }
                        source = (UnitaryDiagram) lhs.getPartAt(coords);
                        metric[j]--;
                    }
                    //debug("metric["+j+"] is reduced to zero");
                }
                //debug("source should reduce to target next time round");
                //debug("source is "+source.toDebugString());
                //debug("target equals "+target.toDebugString());
                // start again
                continue bigLoop;
            }
            // if you get here then the proof is done
            break;
        }
        // each lhs is equal to an rhs component
        // but the lhs may contain duplicates and may not hit some rhs ones

        Diagram lhs = start.getLastDiagram();
        rhs = endBackwards.getLastDiagram();
        //debug("lhs matched is "+lhs.toDebugString());
        //debug("rhs matched is "+rhs.toDebugString());
        Vector lhsParts = lhs.getUnitaryParts();
        Vector rhsParts = rhs.getUnitaryParts();
        //debug("lhs has "+lhsParts.size()+" children");
        //debug("rhs has "+rhsParts.size()+" children");
        //debug("second matches for rhs:");
        //for(int i=0; i<lhsParts.size(); i++)
        //{ //debug(""+isASecondMatchForSomethingOnTheRHS[i]); }
        //debug("have been hit on rhs:");
        //for(int i=0; i<rhsParts.size(); i++)
        //{ //debug(""+hasBeenMatchedBySomethingOnLHS[i]); }

        Vector thingsToRemove = new Vector();
        for (int i = 0; i < lhsParts.size(); i++) {
            if (isASecondMatchForSomethingOnTheRHS[i]) {
                thingsToRemove.add(lhsParts.get(i));
            }
        }
        Vector thingsToAdd = new Vector();
        for (int i = 0; i < rhsParts.size(); i++) {
            if (!hasBeenMatchedBySomethingOnLHS[i]) {
                thingsToAdd.add(rhsParts.get(i));
            }
        }
        // pair off equal things to remove and things to add
        Vector dontNeedToRemove = new Vector();
        Vector dontNeedToAdd = new Vector();
        removeLoop : for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            for (int j = 0; j < thingsToAdd.size(); j++) {
                UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(j);
                if (toRemove.equals(toAdd) && !dontNeedToAdd.contains(toAdd)) {
                    dontNeedToRemove.add(toRemove);
                    dontNeedToAdd.add(toAdd);
                    continue removeLoop;
                }
            }
        }
        thingsToRemove.removeAll(dontNeedToRemove);
        thingsToAdd.removeAll(dontNeedToAdd);
        //debug("must remove "+thingsToRemove.size()+" things from lhs");
        for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            Vector children = new Vector(); // will find 2 matches
            boolean hasFoundFirst = false;
            for (int j = 0;
                j < ((OrCompoundDiagram) lhs).getChildren().size();
                j++) {
                UnitaryDiagram child =
                    (UnitaryDiagram) (((OrCompoundDiagram) lhs)
                        .getChildren()
                        .get(j));
                if (child.equals(toRemove)) {
                    children.add(child);
                    if (hasFoundFirst) {
                        break;
                    } // arraylist children number 2 already
                    else {
                        hasFoundFirst = true;
                    }
                }
            }
            //debug("identify "+children.size()+" children");
            Rule rroi = RuleReverseOrIdempotent.getRule();
            //debug("2. provides d,[d2,d3,..]");
            RuleAppliesToDiagramAndDiagrams radd =
                new RuleAppliesToDiagramAndDiagrams(lhs, children);
            lhs = rroi.applyRuleToPart(lhs, radd);
            debug("about to merge two duplicates on lhs");
            start.addStepAtEnd(rroi, radd, lhs);
        } // end of thingsToRemove loop
        //debug("must add "+thingsToAdd.size()+" things to lhs");
        for (int i = 0; i < thingsToAdd.size(); i++) {
            UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(i);
            Rule rptprr = RulePToPOrRubbish.getRule();
            RuleAppliesToDiagramAndDiagram radd =
                new RuleAppliesToDiagramAndDiagram(lhs, toAdd);
            lhs = rptprr.applyRuleToPart(lhs, radd);
            debug("about to create two duplicates on lhs");
            start.addStepAtEnd(rptprr, radd, lhs);
        } // end of thingsToRemove loop
        //debug("after addition, lhs has "+lhsParts.size()+" children");

        // invert end-bit and concatenate and show proof
        debug("invert last part of proof before concatenating");
        Proof endBit = ProofInverter.invertProof(endBackwards);
        //ProofManager.showNewProof(start,false,true);
        //ProofManager.showNewProof(endBit,false,true);

        debug("concatenate two parts of proof");
        start = concatenateProofs(start, endBit);
        //ProofManager.showNewProof(start,false);
        //ProofManager.showNewProof(invertProof(endBackwards),false);
        debug("returning proof");
        return start;
    }
    private static void debug(String message){
        if(debug){
            System.out.println(message);
        }
    }
}
