package rwd.rules.autoproof;
import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;

import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.inputspec.RuleAddContour;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesWithString;

public class ContourSetEqualiser {
	public static Proof makeContourSetsEqual(Diagram d, Diagram other) {
		Proof p = new Proof(d);
		Vector unitaryPieces = d.getUnitaryParts();
		Vector unionOfContours = new Vector();
		for (int i = 0; i < unitaryPieces.size(); i++) {
			UnitaryDiagram ud = (UnitaryDiagram) unitaryPieces.get(i);
			if (!(ud instanceof FalseDiagram)) {
				Vector contours = ud.getCloneOfContourSet();
				for (int j = 0; j < contours.size(); j++) {
					Contour c = (Contour) contours.get(j);
					if (!unionOfContours.contains(c)) {
						unionOfContours.add(c);
					}
				}
			}
		}
		unitaryPieces = other.getUnitaryParts();
		for (int i = 0; i < unitaryPieces.size(); i++) {
			UnitaryDiagram ud = (UnitaryDiagram) unitaryPieces.get(i);
			if (!(ud instanceof FalseDiagram)) {
				Vector contours = ud.getCloneOfContourSet();
				for (int j = 0; j < contours.size(); j++) {
					Contour c = (Contour) contours.get(j);
					if (!unionOfContours.contains(c)) {
						unionOfContours.add(c);
					}
				}
			}
		}
		//System.out.println("union of contours is "+unionOfContours);
		contourLoop : for (int i = 0; i < unionOfContours.size(); i++) {
			while (true) {
				Diagram result =
					tryToAddContour(d, (Contour) unionOfContours.get(i), p);
				if (result != null) {
					d = result;
					//System.out.println("just added a contour to "+d.toSaveString());
				} else { //System.out.println("continue to get next contour");
					continue contourLoop;
				}
			}
		}
		//ProofManager.showNewProof(p,false);
		return p;
	}

	private static Diagram tryToAddContour(
		Diagram d,
		Contour c,
		Proof p) { //System.out.println("try to add contour "+c+" to "+d.toSaveString());
		if (d instanceof UnitaryDiagram) {
			if (d instanceof FalseDiagram) {
				return null;
			}
			//System.out.println("unitary");
			UnitaryDiagram ud = (UnitaryDiagram) d;
			if (!ud.getCloneOfContourSet().contains(c)) { // add c to ud...
				// and substitute back into d... but d = ud
				//System.out.println("OK - add "+c);
				Rule r = RuleAddContour.getRule();
				RuleApplicationSpec ras =
					new RuleAppliesWithString(d, c.toString());
				d = r.applyRule(ras);
				p.addStepAtEnd(r, ras, d);
				//System.out.println("added "+c
				//                      +" to diagram has become \n"+d.toSaveString());
				return d;
			}
		} else if (d instanceof CompoundDiagram) { //System.out.println("compound");
			Vector comps = d.getUnitaryParts();
			for (int i = 0; i < comps.size(); i++) {
				UnitaryDiagram ud = (UnitaryDiagram) comps.get(i);
				if (!(ud instanceof FalseDiagram)
					&& !ud.getCloneOfContourSet().contains(c)) { // add c to ud...
					// and substitute back into d...
					Rule r = RuleAddContour.getRule();
					RuleApplicationSpec ras =
						new RuleAppliesWithString(ud, c.toString());
					Diagram newud = r.applyRule(ras);
					//System.out.println("newud is "+newud);
					//Vector parentAndPart = d.findPartLike(ud);
					//CompoundDiagram parent = (CompoundDiagram)parentAndPart.get(0);
					CompoundDiagram parent = d.getParentOf(ud);
					Vector children = parent.getChildren();
					Diagram result = d.getClone();
					//System.out.println("start of result is "+result.toSaveString());
					int[] coords = d.getCoords(parent);
					CompoundDiagram newParent =
						(CompoundDiagram) result.getPartAt(coords);
					//             (CompoundDiagram)result.findPartLike(parent).get(1);
					//System.out.println("parent in result is "+newParent.toSaveString());
					Vector resultingChildren = newParent.getChildren();
					resultingChildren.clear();
					for (int j = 0; j < children.size(); j++) {
						Diagram child = (Diagram) children.get(j);
						if (child != ud) {
							//System.out.println("add the old child");
							resultingChildren.add(child.getClone());
						} else { //System.out.println("add the new child");
							resultingChildren.add(newud);
						}
					}
					//System.out.println("after substituting children, children are \n"
					//                   +resultingChildren);
					p.addStepAtEnd(r, ras, result);
					//System.out.println("added "+c
					//            +", to diagram \n"+result.toSaveString());
					return result;
				}
			}
		}
		//System.out.println("can't add to diagram");
		return null;
	}
}
