package rwd.rules.choicespec;
import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;

public class RuleDMPushNotThroughOr extends RuleWithChoice {
    // takes Not (Q or R)
    // takes NotQ and NotR 
    private RuleDMPushNotThroughOr() {
        super("sd: not(QorR) -> notQ and notR");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleDMPushNotThroughOr();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof NotCompoundDiagram)) {
            return false;
        }
        NotCompoundDiagram ncd = (NotCompoundDiagram) d;
        // search for a child which is an "and"
        Vector children = ncd.getChildren();
        Diagram child = (Diagram) (children.get(0));
        return child instanceof OrCompoundDiagram;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (!(d instanceof NotCompoundDiagram)) {
            return result;
        }
        NotCompoundDiagram ncd = (NotCompoundDiagram) d;
        // search for a child which is an "or"
        Vector children = ncd.getChildren();
        Diagram child = (Diagram) (children.get(0));
        if(!(child instanceof OrCompoundDiagram)){
            return result;
        }        
        OrCompoundDiagram ocd = (OrCompoundDiagram) child;
        children = (Vector) (ocd.getChildren().clone());
        Object faveObj = children.get(0);
        int numChildren = children.size();
        Vector QCandidates = Utilities.powerSet(children);
        Vector toRemove = new Vector();
        for (int j = 0; j < QCandidates.size(); j++) {
            Vector QCandidate = (Vector) (QCandidates.get(j));
            if (QCandidate.size()==numChildren || !QCandidate.contains(faveObj)) {
                toRemove.add(QCandidate);
            }
        }
        QCandidates.removeAll(toRemove);
        for (int k = 0; k < QCandidates.size(); k++) {
            Vector Qchildren = (Vector) QCandidates.get(k);
            Vector spec = new Vector();
            spec.add(ocd);
            spec.addAll((Vector)Qchildren.clone());
            for(int i=0; i<spec.size(); i++){
                Diagram tmp = (Diagram)spec.get(i);
                if(tmp==null){
                    System.out.println("diagram is "+tmp);
                }
            }
            result.add(new RuleAppliesToDiagramAndDiagrams(d, spec));
        }
        return result;
    }
    // takes P and (Q or R) and ...
    // becomes ((P and Q) or (P and R)) and ...
    public Diagram applyRule(RuleApplicationSpec r) {
        RuleAppliesToDiagramAndDiagrams radds = (RuleAppliesToDiagramAndDiagrams) r;
        NotCompoundDiagram ncd = (NotCompoundDiagram) radds.d;
        OrCompoundDiagram QorR = (OrCompoundDiagram)radds.others.get(0);
        Vector QChildren = (Vector)radds.others.clone();
        QChildren.remove(QorR);
        Vector RChildren = (Vector)QorR.getChildren().clone();
        RChildren.removeAll(QChildren);
        
        Vector justQ = new Vector();
        if(QChildren.size()>1){
            justQ.add(new OrCompoundDiagram(QChildren,"Q"));
        } else {
            justQ=QChildren;
        }
        NotCompoundDiagram notQ =
            new NotCompoundDiagram(justQ, "not Q");
        Vector justR = new Vector();
        if(RChildren.size()>1){
            justR.add(new OrCompoundDiagram(RChildren,"R"));
        } else {
            justR=RChildren;
        }
        NotCompoundDiagram notR =
            new NotCompoundDiagram(justR, "not R");
        Vector notdiags = new Vector();
        notdiags.add(notQ);
        notdiags.add(notR);
        return new AndCompoundDiagram(
            notdiags,
            "and(" + notQ.getName() + "," + notR.getName() + ")");
    }
}
