package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToContour;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Spider;

public class RuleDeleteContourSimple extends RuleWithChoice {
    private RuleDeleteContourSimple() {
        super("sd: delete contour (simple)");
    }
    private static Rule example = null;
    public static Rule getRule() {
        if (example == null) {
            example = new RuleDeleteContourSimple();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof UnitaryDiagram)) {
            return false;
        } else if (d instanceof FalseDiagram) {
            return false;
        } else {
            if (((UnitaryDiagram) d).getCloneOfContourSet().size() == 0) {
                return false;
            }
        }
        UnitaryDiagram ud = (UnitaryDiagram) d;
        Vector contours = ud.getCloneOfContourSet();
        Vector oldZones = ud.getCloneOfZoneSet();
        Vector allPossOldZones = UnitaryDiagram.allPossibleZones(contours);
        contourLoop : for (int i = 0; i < contours.size(); i++) {
            Contour c = (Contour) contours.get(i);
            //System.out.println("can we delete contour "+c+"?");
            Vector reducedCSet = (Vector) contours.clone();
            reducedCSet.remove(c);
            Vector allPossNewZones =
                UnitaryDiagram.allPossibleZones(reducedCSet);

            //for each possible new zone
            for (int j = 0; j < allPossNewZones.size(); j++) {
                Zone possNewZone = (Zone) allPossNewZones.get(j);
                //System.out.println("consider new zone "+possNewZone);
                // either all old present zones are absent or present
                boolean allAbsent = true;
                boolean allPresent = true;
                oldZoneLoop : for (
                    int k = 0; k < allPossOldZones.size(); k++) {
                    Zone oldZone = (Zone) allPossOldZones.get(k);
                    // find out whether old zone matches new zone
                    if ((oldZone
                        .getContainingContours()
                        .containsAll(possNewZone.getContainingContours())
                        && possNewZone.getContainingContours().containsAll(
                            oldZone.getContainingContours()))
                        || (oldZone
                            .getExcludingContours()
                            .containsAll(possNewZone.getExcludingContours())
                            && possNewZone.getExcludingContours().containsAll(
                                oldZone.getExcludingContours()))) {
                        if (ud.getCloneOfZoneSet().contains(oldZone)) {
                            allAbsent = false;
                        } else {
                            allPresent = false;
                        }
                    }
                }
                //System.out.println("for zone "+possNewZone);
                //System.out.println("allAbsent: "+allAbsent);
                //System.out.println("allPresent: "+allPresent);
                if (!allAbsent && !allPresent) {
                    //System.out.println("so look to delete a different contour");
                    continue contourLoop;
                }
                // either all old present zones are shaded or all unshaded
                boolean allShaded = true;
                boolean allUnshaded = true;
                oldZoneLoop : for (
                    int k = 0; k < allPossOldZones.size(); k++) {
                    Zone oldZone = (Zone) allPossOldZones.get(k);
                    if (!oldZones.contains(oldZone)) {
                        continue oldZoneLoop;
                    }
                    // find out whether old zone matches new zone
                    if ((oldZone
                        .getContainingContours()
                        .containsAll(possNewZone.getContainingContours())
                        && possNewZone.getContainingContours().containsAll(
                            oldZone.getContainingContours()))
                        || (oldZone
                            .getExcludingContours()
                            .containsAll(possNewZone.getExcludingContours())
                            && possNewZone.getExcludingContours().containsAll(
                                oldZone.getExcludingContours()))) {
                        if (ud.containsShaded(oldZone)) {
                            allUnshaded = false;
                        } else {
                            allShaded = false;
                        }
                    }
                }
                //System.out.println("for zone "+possNewZone);
                //System.out.println("allUnshaded: "+allUnshaded);
                //System.out.println("allShaded: "+allShaded);
                if (!allUnshaded && !allShaded) {
                    //System.out.println("so look to delete a different contour");
                    continue contourLoop;
                }
                // also need to check that if a spider has a foot in 
                // all or none matching oldZones
                for (int sp = 0; sp < ud.getSpiders().size(); sp++) {
                    Spider s = (Spider) ud.getSpiders().get(sp);
                    boolean allHaveFeet = true;
                    boolean allNoFeet = true;
                    oldZoneLoop : for (
                        int k = 0; k < allPossOldZones.size(); k++) {
                        Zone oldZone = (Zone) allPossOldZones.get(k);
                        if (!oldZones.contains(oldZone)) {
                            continue oldZoneLoop;
                        }
                        // find out whether old zone matches new zone
                        if ((oldZone
                            .getContainingContours()
                            .containsAll(possNewZone.getContainingContours())
                            && possNewZone.getContainingContours().containsAll(
                                oldZone.getContainingContours()))
                            || (oldZone
                                .getExcludingContours()
                                .containsAll(possNewZone.getExcludingContours())
                                && possNewZone.getExcludingContours().containsAll(
                                    oldZone.getExcludingContours()))) {
                            if (s.getHabitat().contains(oldZone)) {
                                allNoFeet = false;
                            } else {
                                allHaveFeet = false;
                            }
                        }
                    }
                    //System.out.println("for zone "+possNewZone);
                    //System.out.println("for spider "+possNewZone);
                    //System.out.println("allNoFeet: "+allNoFeet);
                    //System.out.println("allHaveFeet: "+allHaveFeet);
                    if (!allNoFeet && !allHaveFeet) {
                        //System.out.println("so look to delete a different contour");
                        continue contourLoop;
                    }
                }
            }
            //System.out.println("can delete contour "+c);
            return true; // can remove this contour
        }
        //System.out.println("can't remove any "+getApplicationPossibilities(d));
        return false; // can't remove any contour
    }

    public Vector getApplicationPossibilities(Diagram d) {
        //System.out.println("getting application possibilities");
        Vector result = new Vector();
        if (!(d instanceof UnitaryDiagram)) {
            return result;
        } else if (d instanceof FalseDiagram) {
            return result;
        } else {
            if (((UnitaryDiagram) d).getCloneOfContourSet().size() == 0) {
                return result;
            }
        }
        UnitaryDiagram ud = (UnitaryDiagram) d;
        Vector contours = ud.getCloneOfContourSet();
        Vector oldZones = ud.getCloneOfZoneSet();
        Vector allPossOldZones = UnitaryDiagram.allPossibleZones(contours);
        //System.out.println("all possible old zones are "+allPossOldZones);
        contourLoop : for (int i = 0; i < contours.size(); i++) {
            Contour c = (Contour) contours.get(i);
            //System.out.println("can we remove contour "+c+"?");
            Vector reducedCSet = (Vector) contours.clone();
            reducedCSet.remove(c);
            Vector allPossNewZones =
                UnitaryDiagram.allPossibleZones(reducedCSet);

            //for each possible new zone
            for (int j = 0; j < allPossNewZones.size(); j++) {
                Zone possNewZone = (Zone) allPossNewZones.get(j);
                //System.out.println("consider new zone "+possNewZone);
                // either all old present zones are absent or all present
                boolean allAbsent = true;
                boolean allPresent = true;
                oldZoneLoop : for (
                    int k = 0; k < allPossOldZones.size(); k++) {
                    Zone oldZone = (Zone) allPossOldZones.get(k);
                    // find out whether old zone matches new zone
                    if ((oldZone
                        .getContainingContours()
                        .containsAll(possNewZone.getContainingContours())
                        && possNewZone.getContainingContours().containsAll(
                            oldZone.getContainingContours()))
                        || (oldZone
                            .getExcludingContours()
                            .containsAll(possNewZone.getExcludingContours())
                            && possNewZone.getExcludingContours().containsAll(
                                oldZone.getExcludingContours()))) {
                        //System.out.println("consider oldzone "+oldZone);
                        if (ud.getCloneOfZoneSet().contains(oldZone)) {
                            //System.out.println("it's present");
                            allAbsent = false;
                        } else {
                            //System.out.println("it's absent");
                            allPresent = false;
                        }
                    }
                }
                //System.out.println("for zone "+possNewZone);
                //System.out.println("allAbsent: "+allAbsent);
                //System.out.println("allPresent: "+allPresent);
                if (!allAbsent && !allPresent) {
                    //System.out.println("so look to delete a different contour");
                    continue contourLoop;
                }
                // either all old present zones are shaded or all unshaded
                boolean allShaded = true;
                boolean allUnshaded = true;
                oldZoneLoop : for (
                    int k = 0; k < allPossOldZones.size(); k++) {
                    Zone oldZone = (Zone) allPossOldZones.get(k);
                    if (!oldZones.contains(oldZone)) {
                        continue oldZoneLoop;
                    }
                    // find out whether old zone matches new zone
                    if ((oldZone
                        .getContainingContours()
                        .containsAll(possNewZone.getContainingContours())
                        && possNewZone.getContainingContours().containsAll(
                            oldZone.getContainingContours()))
                        || (oldZone
                            .getExcludingContours()
                            .containsAll(possNewZone.getExcludingContours())
                            && possNewZone.getExcludingContours().containsAll(
                                oldZone.getExcludingContours()))) {
                        if (ud.containsShaded(oldZone)) {
                            allUnshaded = false;
                        } else {
                            allShaded = false;
                        }
                    }
                }
                if (!allUnshaded && !allShaded) {
                    continue contourLoop;
                }
                // also need to check that if a spider has a foot in 
                // all or none matching oldZones
                for (int sp = 0; sp < ud.getSpiders().size(); sp++) {
                    Spider s = (Spider) ud.getSpiders().get(sp);
                    boolean allHaveFeet = true;
                    boolean allNoFeet = true;
                    oldZoneLoop : for (
                        int k = 0; k < allPossOldZones.size(); k++) {
                        Zone oldZone = (Zone) allPossOldZones.get(k);
                        if (!oldZones.contains(oldZone)) {
                            continue oldZoneLoop;
                        }
                        // find out whether old zone matches new zone
                        if ((oldZone
                            .getContainingContours()
                            .containsAll(possNewZone.getContainingContours())
                            && possNewZone.getContainingContours().containsAll(
                                oldZone.getContainingContours()))
                            || (oldZone
                                .getExcludingContours()
                                .containsAll(possNewZone.getExcludingContours())
                                && possNewZone.getExcludingContours().containsAll(
                                    oldZone.getExcludingContours()))) {
                            if (s.getHabitat().contains(oldZone)) {
                                allNoFeet = false;
                            } else {
                                allHaveFeet = false;
                            }
                        }
                    }
                    if (!allNoFeet && !allHaveFeet) {
                        continue contourLoop;
                    }
                }
            }
            result.add(new RuleAppliesToContour(ud, c));
            // can remove this contour
        }
        //System.out.println("returning result with "+result.size()+" options");
        return result;
    }
    public Diagram applyRule(RuleApplicationSpec r) {
        return RuleDeleteContour.getRule().applyRule(r);
    }
}
