package rwd.rules.choicespec;

import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToZone;
public class RuleDeleteShadedZone extends RuleWithChoice {
	private RuleDeleteShadedZone() {
		super("sd: delete zone");
	}
	private static Rule example = null;
	public synchronized static Rule getRule() {
		if (example == null) {
			example = new RuleDeleteShadedZone();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector zones = ud.getCloneOfZoneSet();
			Iterator it = ud.getSZIterator();
            zoneLoop:
            while(it.hasNext()){
                Zone z = (Zone)it.next();
				if (ud.aSpiderTouches(z)
					|| z.getContainingContours().size() == 0) {
					continue zoneLoop;
				}
				//System.out.println("considering zone "+z+" for deletion");
				boolean allContoursAreElsewhere = true;
				//System.out.println("all contoursOK:"+allContoursAreElsewhere);
				for (int j = 0; j < z.getContainingContours().size(); j++) {
					Contour c = (Contour) (z.getContainingContours().get(j));
					//System.out.println("considering contour "+c+" in zone "+z);
					boolean isAnotherZone = false;
					//System.out.println("have found another zone :"+isAnotherZone);
					otherZoneLoop : for (int k = 0; k < zones.size(); k++) {
						Zone otherZ = (Zone) (zones.get(k));
						if (otherZ != z) {
							if (otherZ.getContainingContours().contains(c)) {
								isAnotherZone = true;
								//System.out.println("have found another zone :"+isAnotherZone);
								break otherZoneLoop;
							}
						}
					}
					//System.out.println("have found another zone :"+isAnotherZone);
					if (!isAnotherZone) {
						allContoursAreElsewhere = false;
						//System.out.println("all contoursOK:"+allContoursAreElsewhere);
					}
				}
				//System.out.println("all contoursOK:"+allContoursAreElsewhere);
				if (allContoursAreElsewhere) {
					return true;
					//System.out.println("result now has "+result.size()+" options");
				}
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector zones = ud.getCloneOfZoneSet();
            Iterator it = ud.getSZIterator();
            zoneLoop:
            while(it.hasNext()){
                Zone z = (Zone)it.next();
				if (ud.aSpiderTouches(z)
					|| z.getContainingContours().size() == 0) {
					continue zoneLoop;
				}
				//System.out.println("considering zone "+z+" for deletion");
				boolean allContoursAreElsewhere = true;
				//System.out.println("all contoursOK:"+allContoursAreElsewhere);
				for (int j = 0; j < z.getContainingContours().size(); j++) {
					Contour c = (Contour) (z.getContainingContours().get(j));
					//System.out.println("considering contour "+c+" in zone "+z);
					boolean isAnotherZone = false;
					//System.out.println("have found another zone :"+isAnotherZone);
					otherZoneLoop : for (int k = 0; k < zones.size(); k++) {
						Zone otherZ = (Zone) (zones.get(k));
						if (otherZ != z) {
							if (otherZ.getContainingContours().contains(c)) {
								isAnotherZone = true;
								//System.out.println("have found another zone :"+isAnotherZone);
								break otherZoneLoop;
							}
						}
					}
					//System.out.println("have found another zone :"+isAnotherZone);
					if (!isAnotherZone) {
						allContoursAreElsewhere = false;
						//System.out.println("all contoursOK:"+allContoursAreElsewhere);
					}
				}
				//System.out.println("all contoursOK:"+allContoursAreElsewhere);
				if (allContoursAreElsewhere) {
					result.add(new RuleAppliesToZone(d, z));
					//System.out.println("result now has "+result.size()+" options");
				}
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToZone raz = (RuleAppliesToZone) r;
		Diagram d = raz.d;
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) raz.d;
			Zone z = raz.z;
			UnitaryDiagram result = (UnitaryDiagram) (ud.getClone());
			result.removeZone(z);
			result.unshade(z);
			return result;
		}
		return null;
	}
}
