package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleApplicationSpec;

public class RuleDistributeOrOverAnd
    extends RuleWithChoice { // takes P or (Q and R) or ...
    // becomes ((P or Q) and (P or R)) or ...
    // specify P, (Q and R) and Qchildren 
    private RuleDistributeOrOverAnd() {
        super("sd: Por(QandR)or... -> ((PorQ)and(PorR))or...");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleDistributeOrOverAnd();
        }
        return example;
    }
    public boolean isInvertible(){
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof OrCompoundDiagram)) {
            return false;
        }
        OrCompoundDiagram ocd = (OrCompoundDiagram) d;
        //System.out.println("search for a child which is an \"and\"");
        Vector children = ocd.getChildren();
        childLoop : for (int i = 0; i < children.size(); i++) {
            Diagram child = (Diagram) (children.get(i));
            if (!(child instanceof AndCompoundDiagram)) {
                //System.out.println("isn't an AndCompoundDiagram");
                continue childLoop;
            }
            AndCompoundDiagram andChild = (AndCompoundDiagram) child;
            // we have to choose P, one of the other children
            Vector PCandidates = (Vector) (children.clone());
            PCandidates.remove(andChild);
            // now choose Q, a subset of the or-children
            Vector andChildren = andChild.getChildren();
            int numAndChildren = andChildren.size();
            if (numAndChildren == 0) {
                continue childLoop;
            }
            Vector QCandidates = Utilities.powerSet(andChildren);
            Diagram qpart = (Diagram) andChildren.get(0);
            Vector toRemove = new Vector();
            for (int j = 0; j < QCandidates.size(); j++) {
                Vector QCandidate = (Vector) (QCandidates.get(j));
                if (QCandidate.size() == 0
                    || QCandidate.size() == numAndChildren
                    || !QCandidate.contains(qpart)) {
                    toRemove.add(QCandidate);
                }
            }
            QCandidates.removeAll(toRemove);
            //System.out.println("Q-candidates are "+PCandidates);
            for (int j = 0; j < PCandidates.size(); j++) {
                Diagram P = (Diagram)PCandidates.get(j);
                for (int k = 0; k < QCandidates.size(); k++) {
                    Vector Qchildren = (Vector) QCandidates.get(k);
                    return true;
                }
            }
        }
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (!(d instanceof OrCompoundDiagram)) {
            return result;
        }
        OrCompoundDiagram ocd = (OrCompoundDiagram) d;
        //System.out.println("search for a child which is an \"and\"");
        Vector children = ocd.getChildren();
        childLoop : for (int i = 0; i < children.size(); i++) {
            Diagram child = (Diagram) (children.get(i));
            if (!(child instanceof AndCompoundDiagram)) {
                //System.out.println("isn't an AndCompoundDiagram");
                continue childLoop;
            }
            AndCompoundDiagram andChild = (AndCompoundDiagram) child;
            // we have to choose P, one of the other children
            Vector PCandidates = (Vector) (children.clone());
            PCandidates.remove(andChild);
            // now choose Q, a subset of the or-children
            Vector andChildren = andChild.getChildren();
            int numAndChildren = andChildren.size();
            if (numAndChildren == 0) {
                continue childLoop;
            }
            Vector QCandidates = Utilities.powerSet(andChildren);
            Diagram qpart = (Diagram) andChildren.get(0);
            Vector toRemove = new Vector();
            for (int j = 0; j < QCandidates.size(); j++) {
                Vector QCandidate = (Vector) (QCandidates.get(j));
                if (QCandidate.size() == 0
                    || QCandidate.size() == numAndChildren
                    || !QCandidate.contains(qpart)) {
                    toRemove.add(QCandidate);
                }
            }
            QCandidates.removeAll(toRemove);
            //System.out.println("Q-candidates are "+PCandidates);
            for (int j = 0; j < PCandidates.size(); j++) {
                Diagram P = (Diagram)PCandidates.get(j);
                for (int k = 0; k < QCandidates.size(); k++) {
                    Vector Qchildren = (Vector) QCandidates.get(k);
                    Vector spec = new Vector();
                    spec.add(P);
                    spec.add(andChild);
                    spec.addAll(Qchildren);
                    //System.out.println("spec has "+spec.size()+" children");
                    result.add(new RuleAppliesToDiagramAndDiagrams(d,spec));
                }
            }
        }
        return result;
    }
    // takes P and (Q or R) and ...
    // becomes ((P and Q) or (P and R)) and ...
    public Diagram applyRule(RuleApplicationSpec r) {
        RuleAppliesToDiagramAndDiagrams radd =
            (RuleAppliesToDiagramAndDiagrams) r;
        OrCompoundDiagram ocd = (OrCompoundDiagram) radd.d;
        Vector spec = (Vector)radd.others.clone();
        Diagram P = (Diagram) spec.get(0);
        //P is one of the toplevel And-Children
        //System.out.println("andchild is "+spec.get(1));
        CompoundDiagram andChild = (CompoundDiagram) spec.get(1);
        // orChild is (QorR)
        spec.remove(P);
        spec.remove(andChild);
        // now spec is just Q-children.
        Vector rchildren = (Vector)andChild.getChildren().clone();
        rchildren.removeAll(spec);
        //now rchildren is just R-children.
        
        Vector PorQAL = new Vector();
        PorQAL.add(P);
        // used to say PorQAL.addAll(spec);
        PorQAL.add(new AndCompoundDiagram(spec,"old"));
        Diagram PorQ = new OrCompoundDiagram(PorQAL, "PorQ");
        Vector PorRAL = new Vector();
        PorRAL.add(P);
        PorRAL.add(new AndCompoundDiagram(rchildren,"AC"));
        Diagram PorR = new OrCompoundDiagram(PorRAL, "PorR");
        Vector a = new Vector();
        a.add(PorQ);
        a.add(PorR);
        Diagram PorQ_and_PorR = new AndCompoundDiagram(a, "PorQ_and_PorR");
        Vector originalChildren = (Vector) (ocd.getChildren().clone());
        originalChildren.remove(P);
        originalChildren.remove(andChild);
        originalChildren.add(PorQ_and_PorR);
        if (originalChildren.size() > 1) {
            return new OrCompoundDiagram(originalChildren, "outcome");
        } else {
            return (Diagram) originalChildren.get(0);
        }
    }
}
