package rwd.rules.choicespec;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.rules.application.RuleAppliesToDandDandD;

public class RuleReverseExcludedMiddle extends RuleWithChoice {
    private RuleReverseExcludedMiddle() {
        super("sd: excluded middle reversed");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleReverseExcludedMiddle();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof OrCompoundDiagram)) {
            return false;
        }
        OrCompoundDiagram ocd = (OrCompoundDiagram) d;
        // take pairs of children and see
        Vector children = ocd.getChildren();
        d1Loop : for (int i = 0; i < children.size(); i++) {
            Diagram d1 = (Diagram) children.get(i);
            UnitaryDiagram ud1;
            try {
                ud1 = (UnitaryDiagram) d1;
            } catch (ClassCastException e) {
                continue d1Loop;
            }
            d2Loop : for (int j = 0; j < children.size(); j++) {
                Diagram d2 = (Diagram) children.get(j);
                UnitaryDiagram ud2;
                try {
                    ud2 = (UnitaryDiagram) d2;
                } catch (ClassCastException e) {
                    continue d2Loop;
                }
                //System.out.println("can we apply reverse excluded middle to ");
                //System.out.println("ud1"+ud1.toSaveString());
                //System.out.println("ud2"+ud2.toSaveString());
                if (!ud1.getCloneOfZoneSet().containsAll(ud2.getCloneOfZoneSet())
                    || !ud2.getCloneOfZoneSet().containsAll(ud1.getCloneOfZoneSet())) {
                    continue d2Loop;
                }

                Vector spiders1 = ud1.getSpiders();
                Vector spiders2 = ud2.getSpiders();
                for (int is = 0; is < spiders1.size(); is++) {
                    Spider s = (Spider) (spiders1.get(is));
                    if (s.getHabitat().size() == 1) {
                        //System.out.println("found alpha spider "+s);
                        Vector habitats1 = new Vector();
                        for (int js = 0; js < spiders1.size(); js++) {
                            habitats1.add(
                                ((Spider) (spiders1.get(js))).getHabitat());
                        }
                        Vector spiders2temp = (Vector) (spiders2.clone());
                        spiders2temp.add(s);
                        Vector habitats2 = new Vector();
                        for (int js = 0; js < spiders2temp.size(); js++) {
                            habitats2.add(
                                ((Spider) (spiders2temp.get(js))).getHabitat());
                        }
                        if (UnitaryDiagram
                            .zoneBagsMatch(habitats1, habitats2)) {
                            //System.out.println("other spiders match");
                            Zone z = (Zone) (s.getHabitat().get(0));
                            if (!ud1.containsShaded(z)
                                && ud2.containsShaded(z)) {
                                //System.out.println("this zone works for rev excl middle");
                                boolean otherZonesMatch = true;
                                Iterator zi = ud1.getZoneIterator();
                                while(zi.hasNext()){
                                    Zone z1 = (Zone)zi.next();
                                    //System.out.println("does shading match for "+z1);
                                    //System.out.println("doesn't matter if zone is "+z);
                                    //System.out.println("ignore? "+(z==z1));
                                    if (z == z1
                                        || (ud1.containsShaded(z1)
                                            && ud2.containsShaded(z1))
                                        || (!ud1.containsShaded(z1)
                                            && !ud2.containsShaded(z1))) {
                                    } else
                                        /*if(z1!=z &&
                                          (  ud1.containsShaded(z1)&&
                                            !ud2.containsShaded(z1))
                                          ||
                                          ( !ud1.containsShaded(z1)&&
                                             ud2.containsShaded(z1))
                                          )*/ {
                                        otherZonesMatch = false;
                                        //System.out.println("other zones don't match");
                                        break;
                                    }
                                }
                                if (otherZonesMatch) { //System.out.println("OK to apply rwd.rules here");
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }
        //System.out.println(
        //         "leaving method which finds app possibilities:"+result.size());
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (!(d instanceof OrCompoundDiagram)) {
            return result;
        }
        OrCompoundDiagram ocd = (OrCompoundDiagram) d;
        // take pairs of children and see
        Vector children = ocd.getChildren();
        d1Loop : for (int i = 0; i < children.size(); i++) {
            Diagram d1 = (Diagram) children.get(i);
            UnitaryDiagram ud1;
            try {
                ud1 = (UnitaryDiagram) d1;
            } catch (ClassCastException e) {
                continue d1Loop;
            }
            d2Loop : for (int j = 0; j < children.size(); j++) {
                Diagram d2 = (Diagram) children.get(j);
                UnitaryDiagram ud2;
                try {
                    ud2 = (UnitaryDiagram) d2;
                } catch (ClassCastException e) {
                    continue d2Loop;
                }
                //System.out.println("can we apply reverse excluded middle to ");
                //System.out.println("ud1"+ud1.toSaveString());
                //System.out.println("ud2"+ud2.toSaveString());
                if (!ud1.getCloneOfZoneSet().containsAll(ud2.getCloneOfZoneSet())
                    || !ud2.getCloneOfZoneSet().containsAll(ud1.getCloneOfZoneSet())) {
                    continue d2Loop;
                }

                Vector spiders1 = ud1.getSpiders();
                Vector spiders2 = ud2.getSpiders();
                for (int is = 0; is < spiders1.size(); is++) {
                    Spider s = (Spider) (spiders1.get(is));
                    if (s.getHabitat().size() == 1) {
                        //System.out.println("found alpha spider "+s);
                        Vector habitats1 = new Vector();
                        for (int js = 0; js < spiders1.size(); js++) {
                            habitats1.add(
                                ((Spider) (spiders1.get(js))).getHabitat());
                        }
                        Vector spiders2temp = (Vector) (spiders2.clone());
                        spiders2temp.add(s);
                        Vector habitats2 = new Vector();
                        for (int js = 0; js < spiders2temp.size(); js++) {
                            habitats2.add(
                                ((Spider) (spiders2temp.get(js))).getHabitat());
                        }
                        if (UnitaryDiagram
                            .zoneBagsMatch(habitats1, habitats2)) {
                            //System.out.println("other spiders match");
                            Zone z = (Zone) (s.getHabitat().get(0));
                            if (!ud1.containsShaded(z)
                                && ud2.containsShaded(z)) {
                                //System.out.println("this zone works for rev excl middle");
                                boolean otherZonesMatch = true;
                                Iterator zi = ud1.getZoneIterator();
                                while(zi.hasNext()){
                                    Zone z1 = (Zone)zi.next();
                                    //System.out.println("does shading match for "+z1);
                                    //System.out.println("doesn't matter if zone is "+z);
                                    //System.out.println("ignore? "+(z==z1));
                                    if (z == z1
                                        || (ud1.containsShaded(z1)
                                            && ud2.containsShaded(z1))
                                        || (!ud1.containsShaded(z1)
                                            && !ud2.containsShaded(z1))) {
                                    } else
                                        /*if(z1!=z &&
                                          (  ud1.containsShaded(z1)&&
                                            !ud2.containsShaded(z1))
                                          ||
                                          ( !ud1.containsShaded(z1)&&
                                             ud2.containsShaded(z1))
                                          )*/ {
                                        otherZonesMatch = false;
                                        //System.out.println("other zones don't match");
                                        break;
                                    }
                                }
                                if (otherZonesMatch) { //System.out.println("OK to apply rwd.rules here");
                                    result.add(
                                        new RuleAppliesToDandDandD(
                                            d,
                                            ud1,
                                            ud2));
                                }
                            }
                        }
                    }
                }
            }
        }
        //System.out.println(
        //         "leaving method which finds app possibilities:"+result.size());
        return result;
    }

    public Diagram applyRule(RuleApplicationSpec r) {
        try {
            RuleAppliesToDandDandD raddd = (RuleAppliesToDandDandD) r;
            Diagram d = raddd.d;
            OrCompoundDiagram ocd = (OrCompoundDiagram) d;
            UnitaryDiagram d1 = (UnitaryDiagram) raddd.other1;
            UnitaryDiagram d2 = (UnitaryDiagram) raddd.other2;
            // find an alpha spider in one which isn't in the other
            // check shading is OK
            Vector spiders1 = d1.getSpiders();
            Vector spiders2 = d2.getSpiders();
            for (int i = 0; i < spiders1.size(); i++) {
                Spider s = (Spider) (spiders1.get(i));
                if (s.getHabitat().size() == 1) {
                    Vector habitats1 = new Vector();
                    for (int j = 0; j < spiders1.size(); j++) {
                        habitats1.add(
                            ((Spider) (spiders1.get(j))).getHabitat());
                    }
                    Vector spiders2temp = (Vector) (spiders2.clone());
                    spiders2temp.add(s);
                    Vector habitats2 = new Vector();
                    for (int j = 0; j < spiders2temp.size(); j++) {
                        habitats2.add(
                            ((Spider) (spiders2temp.get(j))).getHabitat());
                    }
                    if (UnitaryDiagram.zoneBagsMatch(habitats1, habitats2)) {
                        Zone z = (Zone) (s.getHabitat().get(0));
                        if (!d1.containsShaded(z)
                            && d2.containsShaded(z)) {
                            UnitaryDiagram result =
                                (UnitaryDiagram) d2.getClone();
                            result.unshade(z);
                            if (ocd.countChildren() == 2) {
                                return result;
                            } else {
                                Vector newChildren =
                                    (Vector) (ocd.getChildren().clone());
                                newChildren.remove(d1);
                                newChildren.remove(d2);
                                newChildren.add(result);
                                if (newChildren.size() == 1) {
                                    return result;
                                } else {
                                    return new OrCompoundDiagram(
                                        newChildren,
                                        "outcome");
                                }
                            }
                        }
                    }
                }
            }
        } catch (ClassCastException e) {
        }
        return null;
    }
}
