package rwd.rules.choicespec;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToZone;

/*
 * 
 * @author jf,Richard Bosworth
 * 
 * @version 06-Apr-2005 07:40
 */
public class RuleUnshadeZone extends RuleWithChoice {
    private RuleUnshadeZone() {
        super("sd: erase shading");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleUnshadeZone();
        }
        return example;
    }
    public boolean isInvertible() {
        return false;
    }

    public boolean canApplyTo(Diagram d) {
        if (d instanceof UnitaryDiagram && !(d instanceof FalseDiagram)) {
            UnitaryDiagram ud = (UnitaryDiagram) d;
            return ud.getNumShadedZones() > 0;
        }
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (d instanceof UnitaryDiagram && !(d instanceof FalseDiagram)) {
            UnitaryDiagram ud = (UnitaryDiagram) d;
            Iterator it = ud.getSZIterator();
            while (it.hasNext()) {
                Zone z = (Zone) it.next();
                result.add(new RuleAppliesToZone(ud, z));
            }
        }
        return result;
    }

    public Diagram applyRule(RuleApplicationSpec r) {
        try {
            RuleAppliesToZone raz = (RuleAppliesToZone) r;
            UnitaryDiagram ud = (UnitaryDiagram) raz.d;
            Zone z = raz.z;
            UnitaryDiagram result = (UnitaryDiagram) (ud.getClone());
            result.unshade(z);
            //System.out.println("after removing a shaded zone, the result has "+result.getNumShadedZones()+" shaded zones");
            return result;
        } catch (ClassCastException x) {
        }
        return null;
    }
}
