package rwd.rules.constraintdiagrams;
import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.rules.application.RuleAppliesToSpider;
import rwd.rules.choicespec.RuleWithChoice;

public class RuleDeleteSpider extends RuleWithChoice {
	private RuleDeleteSpider() {
		super("cd: delete spider");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleDeleteSpider();
		}
		return example;
	}
    public boolean isInvertible(){
        return false;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector spiders = ud.getSpiders();
			Vector shadedZones = ud.getCloneOfShadedZones();
			for (int i = 0; i < spiders.size(); i++) {
				Spider s = (Spider) (spiders.get(i));
				Vector habitat = (Vector) (s.getHabitat().clone());
				int oldSize = habitat.size();
				habitat.removeAll(shadedZones);
				if (habitat.size() == oldSize) {
					return true;
				}
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector spiders = ud.getSpiders();
			Vector shadedZones = ud.getCloneOfShadedZones();
			for (int i = 0; i < spiders.size(); i++) {
				Spider s = (Spider) (spiders.get(i));
				Vector habitat = (Vector) (s.getHabitat().clone());
				int oldSize = habitat.size();
				habitat.removeAll(shadedZones);
				if (habitat.size() == oldSize) {
					result.add(new RuleAppliesToSpider(d, s));
				}
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return result;
	}
	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToSpider ras = (RuleAppliesToSpider) r;
		UnitaryDiagram d = (UnitaryDiagram) (ras.d);
		Spider s = ras.s;
		Vector h = s.getHabitat();
		UnitaryDiagram result = (UnitaryDiagram) (d.getClone());
		Vector spiders = result.getSpiders();
		for (int i = 0; i < spiders.size(); i++) {
			Spider s2 = (Spider) (spiders.get(i));
			Vector h2 = s2.getHabitat();
			if (h.containsAll(h2) && h2.containsAll(h)) {
				spiders.remove(s2);
				return result;
			}
		}
		return null;
	}
}
