package rwd.rules.heuristics;

import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
//import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
//import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.rules.Proof;
import rwd.rules.ProofFileUser;
import rwd.rules.ProofGetterRandom;
import rwd.rules.Rule;

import java.util.Vector;

public class HeuristicCalculatorCompound implements HeuristicCalculator {

    public static boolean debug = false;

    public double calcHeuristic(Diagram d1, Diagram d2) { //
        //System.out.println("calcing H for "+DiagramFileUser.toSaveString(d1));
        //System.out.println("calcing H for "+DiagramFileUser.toSaveString(d2));
        try {
            d1 = d1.getClone();
            d2 = d2.getClone();
            double result = 0.0;
            double c1 = 0.0;
            double c2 = 0.0;
            double addZone = 0.0;
            double remZone = 0.0;
            double addShading = 0.0;
            double remShading = 0.0;
            double addSpiders = 0.0;
            double remSpiders = 0.0;
            double notMeasure = 0.0;

            CompoundMeasureCalculator cmc1 = new CompoundMeasureCalculator(d1);
            CompoundMeasureCalculator cmc2 = new CompoundMeasureCalculator(d2);
            Vector[] r1 = cmc1.calcMeasuresList("Contour");
            Vector[] r2 = cmc2.calcMeasuresList("Contour");
            //System.out.println("c1 sets are "+r1[0]+" and "+r2[0]);
            //System.out.println("c2 sets are "+r1[1]+" and "+r2[1]);
            //r1[0] = (Vector) r1[0].clone();
            //r2[0] = (Vector) r2[0].clone();
            Vector r1clone = (Vector) r1[0].clone();
            Vector r2clone = (Vector) r2[0].clone();
            //System.out.print("remove "+r2clone+" from "+r1[0]);
            r1[0].removeAll(r2clone);
            //System.out.println("leaving "+r1[0]);
            //System.out.print("remove "+r1clone+" from "+r2[0]);
            r2[0].removeAll(r1clone);
            //System.out.println("leaving "+r2[0]);
            //System.out.println("symm diff sets are "+r1[0]+" and "+r2[0]);
            c1 = r1[0].size() + r2[0].size();
            
            //r1[1] = (Vector) r1[1].clone();
            //r2[1] = (Vector) r2[1].clone();
            r1clone = (Vector) r1[1].clone();
            r2clone = (Vector) r2[1].clone();
            //System.out.print("remove "+r2clone+" from "+r1[1]);
            r1[1].removeAll(r2clone);
            //System.out.println("leaving "+r1[1]);
            //System.out.print("remove "+r1clone+" from "+r2[1]);
            r2[1].removeAll(r1clone);
            //System.out.println("leaving "+r2[1]);
            //System.out.println("symm diff sets are "+r1[1]+" and "+r2[1]);
            c2 = r1[1].size() + r2[1].size();

            Diagram cf1 = CompoundMeasureCalculator.calcCF(d1, d2);
            Diagram cf2 = CompoundMeasureCalculator.calcCF(d2, d1);

            cmc1 = new CompoundMeasureCalculator(cf1);
            cmc2 = new CompoundMeasureCalculator(cf2);
            r1 = cmc1.calcMeasuresList("Zone");
            r2 = cmc2.calcMeasuresList("Zone");
            int[] addZoneA = new int[] { 0, 0 };
            if (!r2[0].containsAll(r1[0])) {
                addZoneA[0] = 1;
            }
            if (!r2[1].containsAll(r1[1])) {
                addZoneA[1] = 1;
            }
            addZone = Math.max(addZoneA[0], addZoneA[1]);
            //addZone = addZoneA[0] + addZoneA[1]; DeletZone not admissible
            int[] remZoneA = new int[] { 0, 0 };
            if (!r1[0].containsAll(r2[0])) {
                remZoneA[0] = 1;
            }
            if (!r1[1].containsAll(r2[1])) {
                remZoneA[1] = 1;
            }
            remZone = Math.max(remZoneA[0], remZoneA[1]);

            Diagram vf1 = CompoundMeasureCalculator.calcVF(cf1);
            Diagram vf2 = CompoundMeasureCalculator.calcVF(cf2);

            cmc1 = new CompoundMeasureCalculator(vf1);
            cmc2 = new CompoundMeasureCalculator(vf2);
            r1 = cmc1.calcMeasuresList("Shading");
            r2 = cmc2.calcMeasuresList("Shading");

            int[] addShadingA = new int[] { 0, 0 };
            if (!r1[0].containsAll(r2[0])) {
                addShadingA[0] = 1;
                //System.out.println("addShadingA[0] = 1");
                //System.out.println("because !"+r1[0]+" containsAll "+r2[0]);
            }
            if (!r1[1].containsAll(r2[1])) {
                addShadingA[1] = 1;
                //System.out.println("addShadingA[1] = 1");
                //System.out.println("because !"+r1[1]+" containsAll "+r2[1]);
            }
            addShading = Math.max(addShadingA[0], addShadingA[1]);
            int[] remShadingA = new int[] { 0, 0 };
            if (!r2[0].containsAll(r1[0])) {
                remShadingA[0] = 1;
                //System.out.println("remShadingA[0] = 1");
                //System.out.println("because !"+r2[0]+" containsAll "+r1[0]);
            }
            if (!r2[1].containsAll(r1[1])) {
                remShadingA[1] = 1;
                //System.out.println("remShadingA[1] = 1");
                //System.out.println("because !"+r2[1]+" containsAll "+r1[1]);
            }
            remShading = Math.max(remShadingA[0], remShadingA[1]);

            cmc1 = new CompoundMeasureCalculator(vf1);
            cmc2 = new CompoundMeasureCalculator(vf2);
            r1 = cmc1.calcMeasuresList("Spiders");
            r2 = cmc2.calcMeasuresList("Spiders");

            int[] addSpidersA = new int[] { 0, 0 };
            // addspiders1 is the size of spiders1 of d2 set difference spiders1 of d1
            // addspiders2 is the size of spiders2 of d2 set difference spiders2 of d1

            // take a copy of r2[0] and a copy of r1[0]
            // and remove "common" spiders from r1[0]
            // addSpiders1 is the size of the remainder
            Vector tmp1 = (Vector) r2[0].clone();
            Vector tmp2 = (Vector) r1[0].clone();
            Vector toRemove = new Vector();
            for (int i = 0; i < tmp1.size(); i++) {
                Spider s = (Spider) tmp1.get(i);
                Vector hab = s.getHabitat();
                Spider match = null;
                matchLoop : for (int j = 0; j < tmp2.size(); j++) {
                    Spider other = (Spider) tmp2.get(j);
                    Vector hab2 = other.getHabitat();
                    if (hab.containsAll(hab2) && hab2.containsAll(hab)) {
                        match = other;
                        break matchLoop;
                    }
                }
                if (match != null) {
                    tmp2.remove(match);
                    toRemove.add(s);
                }
            }
            tmp1.removeAll(toRemove);
            addSpidersA[0] = tmp1.size();
            // take a copy of r2[1] and a copy of r1[1]
            // and remove spiders which share the same habitat from r1[1]
            // addSpiders2 is the size of the remainder
            tmp1 = (Vector) r2[1].clone();
            tmp2 = (Vector) r1[1].clone();
            toRemove = new Vector();
            for (int i = 0; i < tmp1.size(); i++) {
                Spider s = (Spider) tmp1.get(i);
                Vector hab = s.getHabitat();
                Spider match = null;
                matchLoop : for (int j = 0; j < tmp2.size(); j++) {
                    Spider other = (Spider) tmp2.get(j);
                    Vector hab2 = other.getHabitat();
                    if (hab.containsAll(hab2) && hab2.containsAll(hab)) {
                        match = other;
                        break matchLoop;
                    }
                }
                if (match != null) {
                    tmp2.remove(match);
                    toRemove.add(s);
                }
            }
            tmp1.removeAll(toRemove);
            addSpidersA[1] = tmp1.size();
            // take the maximum of addSpiders1 and addSpiders2
            // this is called addSpiders
            addSpiders = Math.max(addSpidersA[0], addSpidersA[1]);

            int[] remSpidersA = new int[] { 0, 0 };
            // remspiders1 is the size of spiders1 of d1 set difference spiders1 of d2
            tmp1 = (Vector) r1[0].clone();
            tmp2 = (Vector) r2[0].clone();
            toRemove = new Vector();
            for (int i = 0; i < tmp1.size(); i++) {
                Spider s = (Spider) tmp1.get(i);
                Vector hab = s.getHabitat();
                Spider match = null;
                matchLoop : for (int j = 0; j < tmp2.size(); j++) {
                    Spider other = (Spider) tmp2.get(j);
                    Vector hab2 = other.getHabitat();
                    if (hab.containsAll(hab2) && hab2.containsAll(hab)) {
                        match = other;
                        break matchLoop;
                    }
                }
                if (match != null) {
                    tmp2.remove(match);
                    toRemove.add(s);
                }
            }
            tmp1.removeAll(toRemove);
            remSpidersA[0] = tmp1.size();
            // remspiders2 is the size of spiders2 of d1 set difference spiders2 of d2
            tmp1 = (Vector) r1[1].clone();
            tmp2 = (Vector) r2[1].clone();
            toRemove = new Vector();
            for (int i = 0; i < tmp1.size(); i++) {
                Spider s = (Spider) tmp1.get(i);
                Vector hab = s.getHabitat();
                Spider match = null;
                matchLoop : for (int j = 0; j < tmp2.size(); j++) {
                    Spider other = (Spider) tmp2.get(j);
                    Vector hab2 = other.getHabitat();
                    if (hab.containsAll(hab2) && hab2.containsAll(hab)) {
                        match = other;
                        break matchLoop;
                    }
                }
                if (match != null) {
                    tmp2.remove(match);
                    toRemove.add(s);
                }
            }
            tmp1.removeAll(toRemove);
            remSpidersA[1] = tmp1.size();
            remSpiders = Math.max(remSpidersA[0], remSpidersA[1]);

            int[] andOr1 = cmc1.calcMeasuresNum("AndOrNum");
            int[] andOr2 = cmc2.calcMeasuresNum("AndOrNum");

            double andNum;
            if (andOr1[0] == 0 && andOr2[0] == 0) {
                andNum = 0.0;
            } else if (andOr1[0] == 0) {
                andNum = 1 + Math.log(andOr2[0]) / Math.log(2);
            } else if (andOr2[0] == 0) {
                andNum = 1 + Math.log(andOr1[0]) / Math.log(2);
            } else {
                andNum =
                    Math.abs(Math.log(andOr1[0] + 1) - Math.log(andOr2[0] + 1))
                        / Math.log(2);
            }
            // round it
            andNum = ((int) (andNum * 100)) / 100;
            /*if (andOr1[0] == andOr2[0]) {
                andNum = 0;
            } else {
                andNum = 1;
            }*/

            double orNum;
            if (andOr1[1] == 0 && andOr2[1] == 0) {
                orNum = 0.0;
            } else if (andOr1[1] == 0) {
                orNum = 1 + Math.log(andOr2[1]) / Math.log(2);
            } else if (andOr2[1] == 0) {
                orNum = 1 + Math.log(andOr1[1]) / Math.log(2);
            } else {
                orNum =
                    Math.abs(Math.log(andOr1[1] + 1) - Math.log(andOr2[1] + 1))
                        / Math.log(2);
            }
            // round it
            orNum = ((int) (orNum * 100)) / 100;
            /*if (andOr1[1] == andOr2[1]) {
                orNum = 0;
            } else {
                orNum = 1;
            }*/

            int[] nots1 = cmc1.calcMeasuresNum("NotNum");
            int[] nots2 = cmc2.calcMeasuresNum("NotNum");

            notMeasure =
  //              (Math.abs(nots1[0] - nots2[0])+
  //                      Math.abs(nots1[1] - nots2[1]))
  //                  / 2; ExcludedMiddle not admissible
            (Math
                .max(
                    Math.abs(nots1[0] - nots2[0]),
                    Math.abs(nots1[1] - nots2[1])))
                / 2;
            if (debug) {

                System.out.println("c1 = " + c1);
                System.out.println("c2 = " + c2);
                System.out.println("addZone = " + addZone);
                System.out.println("remZone = " + remZone);
                System.out.println("addShading = " + addShading);
                System.out.println("remShading = " + remShading);
                System.out.println("addSpider = " + addSpiders);
                System.out.println("remSpider = " + remSpiders);
                System.out.println("andNum = " + andNum);
                System.out.println("orNum = " + orNum);
                System.out.println("notNum = " + notMeasure);
            }
            result =            
                Math.max(c1, c2) +addZone + remZone + 
                max(
                    addShading + remShading, 
                    addSpiders / 2,
                    remSpiders / 2,
                    andNum,
                    orNum) +
                notMeasure;
            // watch out!! split spider can introduce 2 new spiders
            // and remove one!
            // watch out!! split spider twice can introduce 4 new spiders
            // and remove two!
            // watch out!! split spider thrice can introduce 6 new spiders
            // and remove three!

            // watch out!! neg eq can change   REMOVE!!!!!!!!!
            // both add shading and rem shading
            // can also change orNum by loads, and andNum

            // watch out!! merge contradiction can eliminate lots of spiders
            // in one fell swoop REMOVE !!!
            return result;
        } catch (ContainsFalseException x) {
            return 0.0;
        }
    }
    public static void main(String args[]) {
        Vector HRules = HeuristicProofWriter.compoundHRules;
        Rule[] rules = new Rule[HRules.size()];
        for (int i = 0; i < HRules.size(); i++) {
            rules[i] = (Rule) HRules.get(i);
        }
        HeuristicCalculatorCompound hc = new HeuristicCalculatorCompound();
        //CostCalculator cc = new DefaultCompoundCostCalculator();
        CostCalculator cc = new DefaultCostCalculator();

        ProofGetterRandom pgr1 = new ProofGetterRandom(2, 3, 2, 1, rules);
        ProofGetterRandom pgr2 = new ProofGetterRandom(1, 1, 2, 6, rules);
        ProofGetterRandom pgr3 = new ProofGetterRandom(0, 3, 2, 2, rules);

        // see how the measures change
        loop : for (int i = 0; i < 5000000; i++) {
            if (i % 100000 == 0) {
                System.out.println("" + i);
            }
            Proof p = null;
            int length = (int) (Math.random() * 6);
            if (i % 3 == 0) {
                p = pgr1.getProof(length);
            } else if (i % 3 == 1) {
                p = pgr2.getProof(length);
            } else {
                p = pgr3.getProof(length);
            }
            if (p.numSteps() == 0) {
                i--;
                continue loop;
            } else {
                HeuristicCalculatorCompound.debug = false;
                double hnum =
                    hc.calcHeuristic(p.getFirstDiagram(), p.getLastDiagram());
                double cost = cc.cost(p);
                System.out.println(
                    "cost is " + cost + " and heuristic is " + hnum);
                if (hnum > cost) {
                    HeuristicCalculatorCompound.debug = true;
                    hnum =
                        hc.calcHeuristic(
                            p.getFirstDiagram(),
                            p.getLastDiagram());
                    System.out.println(
                        "proof " + ProofFileUser.toSaveString(p));
                    System.exit(0);
                }
            }
        }
    }
    private double max(double d1,double d2,double d3,double d4,double d5){
        return 
        Math.max(d1,
        Math.max(d2,
        Math.max(d3,
        Math.max(d4,
        d5))));
    }
}
