package rwd.rules.heuristics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import rwd.rules.NoProofException;
import rwd.rules.Proof;
import rwd.rules.ProofFileUser;
//import rwd.rules.ProofFileUser;
//import rwd.rules.Rule;
import rwd.rules.UnitaryProofGetterRandom;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.rules.autoproof.AutoProofWriterShortened;

/**
 * @author jaf28
 */

public class HeuristicProofAssessor {

    static HeuristicCalculator[] calculators = { new HeuristicCalculatorZero(),
        //new HeuristicCalculatorOptimistic(
        //    new boolean[] { true, false, false, false }),
        //new HeuristicCalculatorOptimistic(
        //    new boolean[] { true, true, false, false }),
        //new HeuristicCalculatorOptimistic(
        //    new boolean[] { true, true, true, false }),
        //new HeuristicCalculatorOptimistic(
        //    new boolean[] { true, true, true, true }),
        new HeuristicCalculatorOptimistic()};

    public static void main(String[] args) {
        //System.out.println("start assessment");
        String dataFilename = args[0];
        int numberOfDataPoints = Integer.parseInt(args[1]);
        int numInitialContours = Integer.parseInt(args[2]);
        int maxNumContours = numInitialContours;
        int numSpiders = Integer.parseInt(args[3]);
        int depthOfProofs = Integer.parseInt(args[4]);
        //System.out.println("depth of proofs is " + depthOfProofs);
        boolean collectAll = (args[5].equals("all"));
        //System.out.println("collecting all :" + collectAll);

        try {
            BufferedWriter bfw =
                new BufferedWriter(new FileWriter(new File(dataFilename)));
            UnitaryProofGetterRandom pgr =
                new UnitaryProofGetterRandom(
                    numInitialContours,
                    maxNumContours,
                    numSpiders);
            dataPointLoop : for (int i = 0; i < numberOfDataPoints; i++) {
                // get some proofs
                Proof p = pgr.getProof(depthOfProofs);
                //System.out.print(".");

                // pass the start/end diagrams to the proof-writer
                Diagram start = p.getFirstDiagram();
                Diagram target = p.getLastDiagram();
                int autonum = 0;
                try {
                    Proof autoProof =
                        AutoProofWriterShortened.writeProof(start, target);
                    autonum = autoProof.numSteps();
                } catch (NoProofException e) {
                    System.out.println("autoproof failed!");
                }

                int optimalLength = -1;
                int[] lengthBuilt = new int[calculators.length];
                int[] numBuilt = new int[calculators.length];
                for (int j = calculators.length - 1; j >= 0; j--) { //
                    Proof pfound = null;
                    try {
                        BufferedWriter bf =
                            new BufferedWriter(
                                new FileWriter(new File("log" + j + ".log")));
                        //String message = ProofFileUser.toSaveString(p);
                        //bf.write("random proof was :");
                        //bf.newLine();
                        //while (message.length() > 0) {
                        //    int n = message.indexOf("\n");
                        //    if (n == -1) {
                        //        bf.write(message);
                        //        message = "";
                        //    } else {
                        //        bf.write(message.substring(0, n));
                        //        bf.newLine();
                        //        message =
                        //            message.substring(n + 1, message.length());
                        //    }
                        //}
                        //bf.write(message);
                        //bf.newLine();

                        Vector outcome =
                            HeuristicProofWriter.seekProof(
                                start,
                                target,
                                depthOfProofs,
                                calculators[j],
                                null,
                                HeuristicProofWriter.unitaryHRules,
                                null,
                                true,
                                false,
                                null,
                                collectAll);
                        //System.out.print(".");
                        if(outcome==null){
                            System.out.println("no proof found!!");
                            DiagramFileUser.saveToFile(start);
                            DiagramFileUser.saveToFile(target);
                            System.out.println(
                                "" + ProofFileUser.toSaveString(p));
                            System.exit(0);
                        }
                        pfound = (Proof) outcome.get(0);
                        if (pfound == null) {
                            System.out.println("null proof found!!");
                            DiagramFileUser.saveToFile(start);
                            DiagramFileUser.saveToFile(target);
                            System.out.println(
                                "" + ProofFileUser.toSaveString(p));
                            System.exit(0);
                        }
                        numBuilt[j] =
                            ((Integer) outcome.get(outcome.size() - 1))
                                .intValue();
                        lengthBuilt[j] = pfound.numSteps();
                        System.out.print(" " + lengthBuilt[j]);
                        System.out.print(" " + numBuilt[j]);
                        if (lengthBuilt[j] > autonum) {
                            System.out.println(
                                "heuristic proof longer than (short)autoproof!");
                        }
                        //System.out.println(
                        //    ""
                        //        + j
                        //        + " proof found is "
                        //        + ProofFileUser.toSaveString(pfound));
                        if (optimalLength == -1) {
                            optimalLength = lengthBuilt[j];
                        } else if (lengthBuilt[j] != optimalLength) {
                            System.out.println("nonoptimal proof found!! : ");
                            System.exit(0);
                        }
                    } catch (NoProofException x) {
                        System.out.println("no proof found!!");
                        DiagramFileUser.saveToFile(start);
                        DiagramFileUser.saveToFile(target);
                        System.exit(0);
                    }
                }
                // check numBuilt decreases
                for (int j = 1; j < calculators.length; j++) {
                    if (numBuilt[j - 1] < numBuilt[j]) {
                        System.out.println(
                            "heuristic failed to improve "
                                + (j - 1)
                                + "->"
                                + j
                                + "!! : numbuilt were "
                                + numBuilt[j
                                - 1]
                                + " and "
                                + numBuilt[j]);
                        System.exit(0);
                    }
                }
                // log
                String toLog = "" + numBuilt[0];
                for (int j = 1; j < calculators.length; j++) {
                    toLog = toLog + "," + numBuilt[j];
                }
                bfw.write(toLog);
                bfw.newLine();
                System.out.println("");
            } // end of datapoint loop
            bfw.flush();
            bfw.close();
        } catch (IOException x) {
            System.out.println("io problem " + x);
            x.printStackTrace();
            System.exit(0);
        }
    }

}
