package rwd.tableaux.semantic;

import java.awt.Component;
import java.util.Enumeration;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.gui.editor.ControlPanel;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class ETableauRenderImpl 
	extends DefaultTreeCellRenderer 
{
	public ETableauRenderImpl(List diags, DefaultMutableTreeNode root, Options opts, ILog log) {
		this.root = root;
		this.diags = diags;
		this.opts = opts;
		this.log = log;
		trueIcon = ControlPanel.makeImageIcon("images/tableaux/open.gif", "true");
		falseIcon = ControlPanel.makeImageIcon("images/tableaux/close.gif", "false");
		undefinedIcon = ControlPanel.makeImageIcon("images/tableaux/undefined.gif", "undefined");
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, 
												  boolean expanded, boolean leaf, int row, boolean hasFocus) {
		super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
		
		// Get the user object and draw it		
		if (value instanceof DefaultMutableTreeNode && value != null) {
			Object userObject = ((DefaultMutableTreeNode)value).getUserObject();
			if (userObject == null) return this;
			if (userObject instanceof ENode) {
				ENode eNode = (ENode)userObject;
				if (opts.isDebug()) {
					System.out.println("Compute icon for '"+eNode+"'"+eNode.getOperationKind());
				}
				ImageIcon icon = computeIcon((DefaultMutableTreeNode)value);
				if (opts.isDebug()) {
					System.out.println(eNode+" "+icon);
				}
				if (icon == trueIcon) {
					setIcon(trueIcon);
					setToolTipText("This node is satisfied.");
				} else if (icon == falseIcon) {
					setIcon(falseIcon);
					setToolTipText("This is not satisfied.");
				} else {
					this.setIcon(undefinedIcon);
					setToolTipText("This node is undefined.");
				}
				return this;
			}
			// Diagrams
			if (userObject.getClass() == UnitaryDiagram.class) {
				UnitaryDiagram ud = (UnitaryDiagram)userObject;			
				Component com = ((ETableauImpl)tree).makeComponent(ud, opts, new LinkedHashSet());
				if (com == null) return this;
				else return com;
			}
			if (userObject instanceof NotCompoundDiagram) {
				NotCompoundDiagram not = (NotCompoundDiagram)userObject;
				Diagram child = (Diagram)not.getChildren().get(0);				
				// Diagrams
				if (child.getClass() == UnitaryDiagram.class) {				
					UnitaryDiagram ud = (UnitaryDiagram)child;			
					Component com = ((ETableauImpl)tree).makeComponent(ud, opts, new LinkedHashSet());
					if (com == null) return this;
					else return com;
				}
				return this;
			}
			// Panel				
			if (userObject instanceof JScrollPane) {
				return (Component)userObject;
			}
		}

		return this;
	}

	/** Compute the state: undefined, satisfied, unsatisfied */
	protected ImageIcon computeIcon(DefaultMutableTreeNode jNode) {
		// Leaf
		Object userObject = ((DefaultMutableTreeNode)jNode).getUserObject();
		if (userObject == null) return undefinedIcon;
		if (userObject instanceof ENode) {
			// ENode
			ENode eNode = (ENode)userObject;
			// Obvious cases
			if (eNode.isObviousTrueCase()) {
				return trueIcon;
			} else if (eNode.isObviousFalseCase()) {
				return falseIcon;
			} else {
				// Leaf in the tableau
				if (eNode.isLeaf()) {
					// Can compute contradictions and icon
					Contradiction con = eNode.computeContradiction();
					if (con.isContradiction())
						return falseIcon;
					else 
						return trueIcon;
				// Use the childern to compute the value
				} else {
					// Remove grahic info
					Enumeration children = jNode.children();
					List jChildren = getCleanJChildren(children); 
					return computeIcon(jChildren, eNode);
				}			
			}
		}
		return undefinedIcon;
	}
	
	protected ImageIcon computeIcon(List jChildren, ENode eNode) {
		// Compute the children's icons
		List icons = new Vector();
		for(int i=0; i<jChildren.size(); i++) {
			DefaultMutableTreeNode jNode = (DefaultMutableTreeNode)jChildren.get(i);
			ImageIcon icon = computeIcon(jNode);
			if (icon != null)
				icons.add(icon);
		}
		// Make and
		OperationKind makeWhat = OperationKind.UNDEFINED; 
		OperationKind kind = eNode.getOperationKind();
		if (kind == OperationKind.EXPAND_AND) {
			// One child
			makeWhat = kind;
		} else if (kind == OperationKind.EXPAND_OR) {
			// More children 
			makeWhat = kind;
		} else if (kind == OperationKind.EXPAND_NOT) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.EQUALISE_DIAGRAMS) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.EQUALISE_ZONES) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.EQUALISE_CONTOURS) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.ADD_ZONE) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.ADD_CONTOURS) {
			// One child
			makeWhat = OperationKind.EXPAND_AND;
		} else if (kind == OperationKind.SPLIT_SPIDERS) {
			// Operation is executed if there is at least a spider to split
			makeWhat = OperationKind.EXPAND_OR;
		} else if (kind == OperationKind.NEGATE_DIAGRAM) {
			// Operation is executed if there is at least a not X: x unitary
			makeWhat = OperationKind.EXPAND_OR;
		}
		// An AND was expanded
		if (makeWhat == OperationKind.EXPAND_AND) {
			boolean hasUndefined = false;
			for(int i=0; i<icons.size(); i++) {
				ImageIcon icon = (ImageIcon)icons.get(i);
				// false
				if (icon == falseIcon) 
					return falseIcon;
				// undefined
				else if (icon == undefinedIcon) {
					hasUndefined = true;
				}
			}
			// true or undefined		
			if (hasUndefined)
				return undefinedIcon;
			else
				return trueIcon;
		// An OR was expanded
		} else if (makeWhat == OperationKind.EXPAND_OR) {
			boolean hasUndefined = false;
			for(int i=0; i<icons.size(); i++) {
				ImageIcon icon = (ImageIcon)icons.get(i);
				// true
				if (icon == trueIcon) 
					return trueIcon;
				// undefined
				else if (icon == undefinedIcon) {
					hasUndefined = true;
				}
			}
			// false or undefined
			if (hasUndefined)
				return undefinedIcon;
			else
				return falseIcon;
		}
		return undefinedIcon;
	}

	protected List getCleanJChildren(Enumeration children) {
		List cleanChildren = new Vector();
		while (children.hasMoreElements()) {
			DefaultMutableTreeNode jn = (DefaultMutableTreeNode)children.nextElement();
			Object en = jn.getUserObject();
			if (en != null && en instanceof ENode)
				cleanChildren.add(jn);
		}
		return cleanChildren;
	}
	protected List getEChildren(Enumeration jChildren) {
		List cleanChildren = new Vector();
		while (jChildren.hasMoreElements()) {
			DefaultMutableTreeNode jn = (DefaultMutableTreeNode)jChildren.nextElement();
			Object en = jn.getUserObject();
			if (en != null && en instanceof ENode)
				cleanChildren.add(en);
		}
		return cleanChildren;
	}
	protected List getEChildren(List jChildren) {
		List cleanChildren = new Vector();
		for(int i=0; i<jChildren.size(); i++) {
			DefaultMutableTreeNode jn = (DefaultMutableTreeNode)jChildren.get(i);
			Object en = jn.getUserObject();
			if (en != null && en instanceof ENode)
				cleanChildren.add(en);
		}
		return cleanChildren;
	}
		
	protected DefaultMutableTreeNode root;
	protected ImageIcon trueIcon;
	protected ImageIcon falseIcon;
	protected ImageIcon undefinedIcon;
	protected List diags;
	protected Options opts;
	protected ILog log;
}
