/*
 * Created on 21-Oct-2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * 
 * made countPoints and showPoints public for debugging RNB 7.6.05  20:21
 * added getLineSegments RNB 9.6.05  08:47
 * 
 * @version 9.6.05   08:47
 */
package rwd.utilities;

//import java.awt.Polygon;
import java.awt.Shape;
//import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
//import java.util.Iterator;

/**
 * @author pjr
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class AreaRWD extends Area implements Serializable{

/**
 * the distance within which we consider points to be "equal"
 * and remove "duplicates"
 */
	public final double DUPLICATEPOINTSLIMIT = 1.5;

    private double epsilon;
    
    
    // constructors
    // ------------
    
    public AreaRWD() {
    	super();
    }

    public AreaRWD(Shape s) {
        super(s);
    }
    
    
    // attributes
    // ----------
    
	/**
	 * @return list of line segments approximating to the boundaries of 
	 *         the area
	 */
    /*
     * test:
     *   area with no points
     *   area with line segments
     *             quadratic segments
     *             cubic segments
     *   area with close-together points in middle of segment sequence
     *   area with close-together points at beginning and end of segment sequence
     *   area with several closed sequences
     */
    public List getLineSegments() {
        List segments = new ArrayList();
        int type = 0;
        float origX = 0;
        float origY = 0;
        float oldX = 0;
        float oldY = 0;
        float[] coords = new float[6];
        PathIterator it = getPathIterator(null);
        while (!it.isDone()) {
            type = it.currentSegment(coords);
                        
            float x1 = coords[0];
            float y1 = coords[1];

            if (type == PathIterator.SEG_MOVETO) {
            	origX = x1;
            	origY = y1;
                oldX = x1;
                oldY = y1;
            } else if (
                type == PathIterator.SEG_LINETO
                    || type == PathIterator.SEG_CUBICTO
                    || type == PathIterator.SEG_QUADTO) {
            	if (!pointsAreClose(oldX,oldY,x1,y1)){
	                segments.add(new Line2D.Float(oldX, oldY, x1, y1));
	                oldX = x1;
	                oldY = y1;
            	}
            } else if (type == PathIterator.SEG_CLOSE) {
            	if (!pointsAreClose(oldX,oldY,origX,origY))
            		segments.add(new Line2D.Float(oldX,oldY,origX,origY));
            }
            it.next();
        }
        return segments;
    }

    
/** For debugging. */
    public int countPoints() {
        PathIterator p = getPathIterator(null);
        int ret = 0;
        while (!p.isDone()) {
            p.next();
            ret++;
        }
        return ret;
    }

    
/** For debugging. */
    public String showPoints() {
        PathIterator p = getPathIterator(null);
        StringBuffer ret = new StringBuffer("[");
        double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
        PathIterator pi = getPathIterator(null);
        while(!pi.isDone()) {
            int segmentType = pi.currentSegment(pathPoint);
            if(segmentType == PathIterator.SEG_MOVETO) {
                ret.append("MOVETO,"+pathPoint[0]+","+pathPoint[1]+",\n  ");
            }
            if(segmentType == PathIterator.SEG_LINETO) {
                ret.append("LINETO,"+pathPoint[0]+","+pathPoint[1]+",\n  ");
            }
            if(segmentType == PathIterator.SEG_QUADTO) {
                ret.append("QUADTO,"+pathPoint[0]+","+pathPoint[1]+","+
                		             pathPoint[2]+","+pathPoint[3]+",\n  ");
            }
            if(segmentType == PathIterator.SEG_CUBICTO) {
                ret.append("CUBICTO,"+pathPoint[0]+","+pathPoint[1]+","+
                		              pathPoint[2]+","+pathPoint[3]+","+
									  pathPoint[4]+","+pathPoint[5]+",\n  ");
            }
            if(segmentType == PathIterator.SEG_CLOSE) {
                ret.append("CLOSE,\n");
            }
            pi.next();
        }
        ret.append("]");
        return ret.toString();
    }


    // operations
    // ----------
    
    public void subtract(Area a){// over-rides buggy method in Area
        super.subtract(a);
        Area area = new Area(this);

        super.subtract(area);
        if (!this.isEmpty()) {
            System.out.println("Area is not empty, this shouldn't happen!");
        }
        area = removeExtraPoints(area);
        this.add(area);
    }

    public void intersect(Area a){// over-rides buggy method in Area
        super.intersect(a);
        Area area = new Area(this);

        super.subtract(area);
        if (!this.isEmpty()) {
            System.out.println("Area is not empty, this shouldn't happen!");
        }
        area = removeExtraPoints(area);
        this.add(area);
    }

    public void exclusiveOr(Area a){// over-rides buggy method in Area
        super.exclusiveOr(a);
        Area area = new Area(this);

        super.subtract(area);

        if (!this.isEmpty()) {
            System.out.println("Area is not empty, this shouldn't happen!");
        }
        area = removeExtraPoints(area);

        this.add(area);
    }


    // private methods
    // ---------------
    
    private AreaRWD removeExtraPoints(Area area) {
        GeneralPath path = new GeneralPath();
        int type = 0;
        float[] coords = new float[6];
        float x0 = 0;
        float y0 = 0;
        PathIterator it = area.getPathIterator(null);
        while (!it.isDone()) {
            type = it.currentSegment(coords);
            
            float x1 = coords[0];
            float y1 = coords[1];
            float x2 = coords[2];
            float y2 = coords[3];
            float x3 = coords[4];
            float y3 = coords[5];
            
            if (type == PathIterator.SEG_CLOSE) {
                path.closePath();
            }
            else if (type == PathIterator.SEG_CUBICTO) {
                if (!pointsAreClose(x0, y0, x3, y3)) {
                    path.curveTo(x1, y1, x2, y2, x3, y3);
                }
                x0 = x3;
                y0 = y3;
            }
            else if (type == PathIterator.SEG_LINETO) {
                if (!pointsAreClose(x0, y0, x1, y1)) {
                    path.lineTo(x1, y1);
                }
                x0 = x1;
                y0 = y1;
            }
            else if (type == PathIterator.SEG_MOVETO) {
                path.moveTo(x1, y1);
            }
            else if (type == PathIterator.SEG_QUADTO) {
                if (!pointsAreClose(x0, y0, x2, y2)) {
                    path.quadTo(x1, y1, x2, y2);
                }
                x0 = x2;
                y0 = y2;
            }
            else if (type == PathIterator.WIND_EVEN_ODD) {
                path.setWindingRule(type);
            }
            else if (type == PathIterator.WIND_NON_ZERO) {
                path.setWindingRule(type);
            }

            it.next();
        }
        
        return new AreaRWD(path);
    }
    
    private boolean pointsAreClose(float x1, float y1, float x2, float y2) {
        float dx = x2 - x1;
        float dy = y2 - y1;
        float distSq = dx * dx + dy * dy;
        return distSq < DUPLICATEPOINTSLIMIT;
    }

    /*private AreaRWD removeExtraPoints(Area area) {
        // use PathIterator to get at the points round the boundaries
        // remember that the boundary could be multiple polygons
        //this.getPathIterator()

        // check points for closeness - if they're within DUPLICATEPOINTSLIMIT of each other
        // in both coordinates then consider one point deleted

        Vector polygons = new Vector();
        Vector xs = new Vector();
        Vector ys = new Vector();
        double[] doubles = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
        int firstx = 0, firsty = 0;
        int oldx = 0, oldy = 0;
        PathIterator p = area.getPathIterator(null);
        boolean isFirst = true;
        if (p.isDone()) {
            System.out.println("pathIterator is done straight away");
            return this;
        }
        while (!p.isDone()) {

            int segmentType = p.currentSegment(doubles);
            if (isFirst) {
                firstx = (int) doubles[0];
                firsty = (int) doubles[1];
                //System.out.println("("+firstx+","+firsty+") adding 1st");
                xs.add(new Integer((int) firstx));
                ys.add(new Integer((int) firsty));
                isFirst = false;
            } else if (
                ((((double) (firstx)) - doubles[0])
                    * (((double) (firstx)) - doubles[0])
                    > DUPLICATEPOINTSLIMIT
                    || (((double) (firsty)) - doubles[1])
                        * (((double) (firsty)) - doubles[1])
                        > DUPLICATEPOINTSLIMIT)
                    && ((((double) (oldx)) - doubles[0])
                        * (((double) (oldx)) - doubles[0])
                        > DUPLICATEPOINTSLIMIT
                        || (((double) (oldy)) - doubles[1])
                            * (((double) (oldy)) - doubles[1])
                            > DUPLICATEPOINTSLIMIT)) {
                xs.add(new Integer((int) (doubles[0])));
                ys.add(new Integer((int) (doubles[1])));
            }

            if (segmentType == PathIterator.SEG_CLOSE) {
                int[] xpts = new int[xs.size()];
                int[] ypts = new int[ys.size()];
                for (int index = 0; index < xs.size(); index++) {
                    xpts[index] = ((Integer) (xs.get(index))).intValue();
                    ypts[index] = ((Integer) (ys.get(index))).intValue();
                }
                if (xpts.length > 2) // line added 26th Feb 03
                    {
                    polygons.add(new Polygon(xpts, ypts, xpts.length));
                }
                xs = new Vector();
                ys = new Vector();
                isFirst = true;
            } else {
                oldx = (int) doubles[0];
                oldy = (int) doubles[1];
            }
            p.next();
        }

        Polygon containingOne = null;
        iLoop : for (int i = 0; i < polygons.size(); i++) {
            Polygon p1 = (Polygon) polygons.get(i);
            for (int j = 0; j < polygons.size(); j++) {
                if (i == j) {
                    continue;
                }
                Polygon p2 = (Polygon) polygons.get(j);
                AreaRWD atmp = new AreaRWD(p1);
                atmp.subtractWithoutCheck(new AreaRWD(p2));
                if (atmp.isEmpty()) {
                    continue iLoop;
                }
            }
            containingOne = p1;
            break iLoop;
        }

        AreaRWD a = new AreaRWD(containingOne);
        for (int i = 0; i < polygons.size(); i++) {
            Polygon p2 = (Polygon) polygons.get(i);
            if (p2 != containingOne) {
                a.subtractWithoutCheck(new Area(p2));
            }
        }

        return (AreaRWD)a;
    }*/
    
    private void subtractWithoutCheck(Area a) {
        super.subtract(a);
    }
    
    /*
    // test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }
    
    private static String stringOf(Line2D l){
    	return "("+ l.getP1()+","+l.getP2()+")";
    }
    
    
    // pre: c = list of lines
    // post: returns string corresponding to contents of collection
    private static String stringOf(List c){
    	String s = "[";
    	Iterator it = c.iterator();
    	while (it.hasNext()){
    		Line2D l = (Line2D) it.next();
    		s += stringOf(l);
    		if (it.hasNext()) s += ",\n  ";
    	}
    	s += "]";    	
    	return s;    	
    }

    public static void main(String args[]) throws Exception {
    	AreaRWD a = new AreaRWD();
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();
    	
    	// make a triangle from three points
    	pr("triangular area");
    	int[] xs = {0,10,-10};
    	int[] ys = {0,20,20};
    	Polygon t = new Polygon(xs,ys, 3);
    	pr("t="+t);
    	a = new AreaRWD(t);
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();
    	
    	// make a chord
    	pr("quarter-circle area");
    	Arc2D ch = new Arc2D.Float(-10, -10, 20, 20, 0, 90, Arc2D.CHORD);
    	a = new AreaRWD(ch);
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();
    	
    	
    	
    	// make a circle
    	pr("whole circle");
    	Arc2D c = new Arc2D.Float(-10, -10, 20, 20, 0, 360, Arc2D.OPEN);
    	pr("c="+c);
    	a = new AreaRWD(c);
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();
    	
    	// make a shape with two boundaries
    	pr("make a a box");
    	Polygon box = new Polygon();
    	box.addPoint(-50,-50);
    	box.addPoint(50,-50);
    	box.addPoint(50,50);
    	box.addPoint(-50,50);
    	
    	a = new AreaRWD(box);
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();
    	
    	pr("make a2 a triangle");
    	Polygon tri = new Polygon();
    	tri.addPoint(0,-10);
    	tri.addPoint(10,10);
    	tri.addPoint(-10,10);
    	AreaRWD a2 = new AreaRWD(tri);
    	pr("a2="+a2);
    	pr("a2.showPoints()="+a2.showPoints());
    	pr("a2.getLineSegments()="+stringOf(a2.getLineSegments()));
    	pg();
    	
    	pr("subtract a2 from a");
    	a.subtract(a2);
    	pr("a="+a);
    	pr("a.showPoints()="+a.showPoints());
    	pr("a.getLineSegments()="+stringOf(a.getLineSegments()));
    	pg();    	    	
    	
    }
    */
    
    

}
