package rwd.views.abstracttoconcrete.converting;

//import rwd.views.abstracttoconcrete.oldaed.*;
import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.views.graph.*;
import java.util.Vector;
import java.util.Iterator;
import java.util.TreeSet;

public class ResultDirector {
	public Result makeResult(UnitaryDiagram ud) { 
		/////////////////////////////////////
		// handle trivial case with one zone
		/////////////////////////////////////
		if (ud.getNumZones() == 1) {
			return new EmptyAtomicResult();
		} // do something better here
		//////////////////////////////////////
		// create dual rwd.views.graph
		//////////////////////////////////////
		
		Graph g = Graph.maxDual(ud);
		//System.out.println("max dual graph is "+g);
		//////////////////////////////////////
		// create dual rwd.views.graph
		//////////////////////////////////////
		//System.out.println("about to test connectivity for "+g);
		if (!g.passesConnectivity()) {
			return new FailsConnectivityResult(g.getConnectivityFix());
		}
		//System.out.println("passed connectivity");
		//////////////////////////////////////
		// handle trivial case with two zones
		// might be two zones like null, a or even bc, bcd.
		//////////////////////////////////////
		if (ud.getNumZones() == 2) {
			//System.out.println("got a case with 2 zones");
			//System.out.println("ud.zones() is "+ud.getCloneOfZoneSet());
			Iterator i = ud.getCloneOfZoneSet().iterator();
			Zone z = (Zone) (i.next());
			if (z.getContainingContours().size() == 0) {
				z = (Zone) (i.next());
			}
			Contour c = (Contour) (z.getContainingContours().iterator().next());
			return new Diamond(new TreeSet(), new TreeSet(),c);
		}
		//////////////////////////////////////
		// get nested components of the diagram
		//////////////////////////////////////
		BlockTree bt = new BlockTree(g);
		//System.out.println("block tree is "+bt);

		// BlockGraph masterbg = new BlockGraph(g);
		// use this to  create a CompositeResult
		// pick out the atomic components one by one and try to build an
		// AtomicResult for each.
		//System.out.println("building a new CompositeResult object");
		CompositeResult result = new CompositeResult(bt);
		//System.out.println("built a new CompositeResult object");
		Vector blocks = bt.top().getBlocks();
		//System.out.println("blocks number "+blocks.size());
		Iterator i = blocks.iterator();
		while (i.hasNext()) { //System.out.println("extract a block now");
			BlockGraph bg = (BlockGraph) (i.next());
			/// try to build an AtomicResult for this bg
			//System.out.println("need to add a view for "+bg);
			Graph gcomp = bg.graph();
			if (gcomp.nodes().size() == 2) {
				Iterator ii = gcomp.nodes().iterator();
				Node n1 = (Node) ii.next();
				Node n2 = (Node) ii.next();
				TreeSet baseCContours;
				TreeSet baseEContours;
				Contour c = null;
				if (n1.label1().containsAll(n2.label1())) {
					baseCContours = new TreeSet(n2.label1());
					baseEContours = new TreeSet(n1.label2());
					TreeSet singleton = (TreeSet) n1.label1().clone();
					singleton.removeAll(n2.label1());
					c =
						Contour.getContour(
							(String) singleton.iterator().next());
				} else {
					baseCContours = new TreeSet(n1.label1());
					baseEContours = new TreeSet(n2.label2());
					TreeSet singleton = (TreeSet) n2.label1().clone();
					singleton.removeAll(n1.label1());
					c = Contour.getContour("" + singleton.iterator().next());
				}
				// here we make a diamond...but...
				Diamond d = new Diamond(baseCContours,baseEContours,c);
				result.add(d.getAEDV(), bg);
				//System.out.println("added a diamond view "+d);
				continue;
			}
			boolean didPlanariseOnce = false;
			boolean couldShrink = true;
			boolean hasPassedWordConditions = false;
			//////////////////////////////////////////////////////////////////
			planariseLoop : while (
				!didPlanariseOnce || !hasPassedWordConditions) {
				//System.out.println("just entered planarise loop");
				boolean didPlanarise = Planariser.planariseAgainNew(gcomp);
				//System.out.println("could planarise? "+didPlanarise);
				if (!didPlanarise
					&& couldShrink) { 
						//System.out.println("about to shrink the graph \n"+gcomp);
					Graph shrunkGraph = gcomp.shrink();
					if (shrunkGraph != null) {
						gcomp = shrunkGraph;
						couldShrink = true;
						//System.out.println("could shrink graph to \n"+gcomp);
						continue planariseLoop;
						// back round with a smaller graph
					} else {
						couldShrink = false;
						//System.out.println("shrinkage returned null");
						break planariseLoop; // exit now
					}
				} else if (!didPlanarise) { //System.out.println("no plane embedding");
					break planariseLoop;
				} // no plane emb. even after shrinking
				////////////////////////////////////////////////////////////////
				didPlanariseOnce = true;
				Vector faceCycles = FaceCycle.faceCycles(gcomp);
				Iterator faceIt = faceCycles.iterator();
				hasPassedWordConditions = true; // be optimistic
				while (faceIt.hasNext()) {
					FaceCycle fc = (FaceCycle) (faceIt.next());
					if (!fc.passesWordConditions()) {
						hasPassedWordConditions = false;
					}
				}
				//if(hasPassedWordConditions)
				//{ //System.out.println("words passed");
				//}
				//else
				//{ //System.out.println("planarised but didn't pass words");
				//}
				// upon return to start of loop,
				// either we have planarised and passed word conditions: exit
				// or we planarised but failed to pass word conditions: try again
			}
			///////////////////////////////////////////////////////////////////
			//System.out.println
			//    ("just left planarise loop: didPlanariseOnce is "+didPlanariseOnce);
			if (!didPlanariseOnce) {
				return new NonPlanarResult();
			} else if (!hasPassedWordConditions) {
				return new FailedWordResult();
			}
			//////////////////////////////////////////////////////////////////
			// g is now embedded in the plane and passes Word Conditions
			// send graph to Dualiser
			//////////////////////////////////////////////////////////////////
			//System.out.println("about to build AEDView object");
			//System.out.println("gcomp is "+gcomp);
			
			try{
                AEDView view =
				Dualiser.makePolygonsNew(
					gcomp,
					new Vector(),
					"no sensible description");
                result.add(view, bg);
            } catch(NullPointerException x){
                System.out.println("couldn't draw");
                System.out.println(""+DiagramFileUser.toSaveString(ud));
                System.exit(0);
            }
			//System.out.println("about to add an "+
			//                   "AEDView to the composite result");
		}
		//System.out.println("result director returning "+result);
		return result;
	}
	
/*	public Result makeResult(OldAbstractEulerDiagram aed) { 
		/////////////////////////////////////
		// handle trivial case with one zone
		/////////////////////////////////////
		if (aed.zones().size() == 1) {
			return new EmptyAtomicResult();
		} // do something better here
		//////////////////////////////////////
		// create dual rwd.views.graph
		//////////////////////////////////////
		Graph g = Graph.maxDual(aed);
		//System.out.println("max dual graph is "+g);
		//////////////////////////////////////
		// create dual rwd.views.graph
		//////////////////////////////////////
		//System.out.println("about to test connectivity for "+g);
		if (!g.passesConnectivity()) {
			return new FailsConnectivityResult(g.getConnectivityFix());
		}
		//System.out.println("passed connectivity");
		//////////////////////////////////////
		// handle trivial case with two zones
		// might be two zones like null, a or even bc, bcd.
		//////////////////////////////////////
		if (aed.zones().size() == 2) {
			//System.out.println("got a case with 2 zones");
			//System.out.println("aed.zones() is "+aed.zones());
			Iterator i = aed.zones().iterator();
			OldZone z = (OldZone) (i.next());
			if (z.contours().size() == 0) {
				z = (OldZone) (i.next());
			}
			OldContour c = (OldContour) (z.contours().iterator().next());
			return new Diamond(new TreeSet(),c);
		}
		//////////////////////////////////////
		// get nested components of the diagram
		//////////////////////////////////////
		BlockTree bt = new BlockTree(g);
		//System.out.println("block tree is "+bt);

		// BlockGraph masterbg = new BlockGraph(g);
		// use this to  create a CompositeResult
		// pick out the atomic components one by one and try to build an
		// AtomicResult for each.
		//System.out.println("building a new CompositeResult object");
		CompositeResult result = new CompositeResult(bt);
		//System.out.println("built a new CompositeResult object");
		Vector blocks = bt.top().getBlocks();
		//System.out.println("blocks number "+blocks.size());
		Iterator i = blocks.iterator();
		while (i.hasNext()) { //System.out.println("extract a block now");
			BlockGraph bg = (BlockGraph) (i.next());
			/// try to build an AtomicResult for this bg
			//System.out.println("need to add a view for "+bg);
			Graph gcomp = bg.graph();
			if (gcomp.nodes().size() == 2) {
				Iterator ii = gcomp.nodes().iterator();
				Node n1 = (Node) ii.next();
				Node n2 = (Node) ii.next();
				TreeSet baseContours;
				OldContour c = null;
				if (n1.label().containsAll(n2.label())) {
					baseContours = new TreeSet(n2.label());
					TreeSet singleton = (TreeSet) n1.label().clone();
					singleton.removeAll(n2.label());
					c =
						OldContour.getContour(
							(String) singleton.iterator().next());
				} else {
					baseContours = new TreeSet(n1.label());
					TreeSet singleton = (TreeSet) n2.label().clone();
					singleton.removeAll(n1.label());
					c = OldContour.getContour("" + singleton.iterator().next());
				}
				// here we make a diamond...but...
				Diamond d = new Diamond(baseContours,c);
				result.add(d.getAEDV(), bg);
				System.out.println("added a diamond view "+d);
				continue;
			}
			boolean didPlanariseOnce = false;
			boolean couldShrink = true;
			boolean hasPassedWordConditions = false;
			//////////////////////////////////////////////////////////////////
			planariseLoop : while (
				!didPlanariseOnce || !hasPassedWordConditions) {
				//System.out.println("just entered planarise loop");
				boolean didPlanarise = Planariser.planariseAgain(gcomp);
				//System.out.println("could planarise? "+didPlanarise);
				if (!didPlanarise
					&& couldShrink) { 
						//System.out.println("about to shrink the graph \n"+gcomp);
					Graph shrunkGraph = gcomp.shrink();
					if (shrunkGraph != null) {
						gcomp = shrunkGraph;
						couldShrink = true;
						//System.out.println("could shrink graph to \n"+gcomp);
						continue planariseLoop;
						// back round with a smaller graph
					} else {
						couldShrink = false;
						//System.out.println("shrinkage returned null");
						break planariseLoop; // exit now
					}
				} else if (!didPlanarise) { //System.out.println("no plane embedding");
					break planariseLoop;
				} // no plane emb. even after shrinking
				////////////////////////////////////////////////////////////////
				didPlanariseOnce = true;
				Vector faceCycles = FaceCycle.faceCycles(gcomp);
				Iterator faceIt = faceCycles.iterator();
				hasPassedWordConditions = true; // be optimistic
				while (faceIt.hasNext()) {
					FaceCycle fc = (FaceCycle) (faceIt.next());
					if (!fc.passesWordConditions()) {
						hasPassedWordConditions = false;
					}
				}
				//if(hasPassedWordConditions)
				//{ //System.out.println("words passed");
				//}
				//else
				//{ //System.out.println("planarised but didn't pass words");
				//}
				// upon return to start of loop,
				// either we have planarised and passed word conditions: exit
				// or we planarised but failed to pass word conditions: try again
			}
			///////////////////////////////////////////////////////////////////
			//System.out.println
			//    ("just left planarise loop: didPlanariseOnce is "+didPlanariseOnce);
			if (!didPlanariseOnce) {
				return new NonPlanarResult();
			} else if (!hasPassedWordConditions) {
				return new FailedWordResult();
			}
			//////////////////////////////////////////////////////////////////
			// g is now embedded in the plane and passes Word Conditions
			// send graph to Dualiser
			//////////////////////////////////////////////////////////////////
			AEDView view =
				Dualiser.makePolygons(
					gcomp,
					new Vector(),
					aed.shortString());
			//System.out.println("about to add an "+
			//                   "AEDView to the composite result");
			result.add(view, bg);
		}
		//System.out.println("result director returning "+result);
		return result;
	}
	*/
}
