package rwd.views.graph;
import java.util.TreeSet;
import java.util.Iterator;
/**
 * Represent edges in a rwd.views.graph.
 * Edges know about their ends (Node objects).
 */
public class Edge implements Comparable {
	public TreeSet nodes;
	static TreeSet edgeLibrary = new TreeSet();
	static TreeSet alternateLibrary = new TreeSet();
	private Comparable label;
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Creates an Edge with given Node ends.
	 */
	private Edge(Node n1, Node n2) {
		nodes = new TreeSet();
		nodes.add(n1);
		nodes.add(n2);
		if (n1.symmDiff(n2).size() == 1) {
			label = (Comparable) (n1.symmDiff(n2).iterator().next());
		}
	}
	///////////////////////////////////////////////////////////////////////////
	public static Edge getEdge(Node n1, Node n2) {
		Iterator i = edgeLibrary.iterator();
		try {
			while (i.hasNext()) {
				Edge e = (Edge) (i.next());
				if (e.nodes.contains(n1) && e.nodes.contains(n2)) {
					return e;
				}
			}
		} catch (ClassCastException x) {
		}
		i = alternateLibrary.iterator();
		while (i.hasNext()) {
			Edge e = (Edge) (i.next());
			if (e.nodes.contains(n1) && e.nodes.contains(n2)) {
				return e;
			}
		}
		Edge e = new Edge(n1, n2);
		//System.out.println("built a new Edge "+e);
		try {
			edgeLibrary.add(e);
		} catch (ClassCastException x) {
			alternateLibrary.add(e);
		}
		return e;
	}
	///////////////////////////////////////////////////////////////////////////
	public String toString() // reqd for OSElement
	{
		Iterator i = nodes.iterator();
		Node n1 = (Node) (i.next());
		Node n2 = (Node) (i.next());
		return "[" + n1 + "," + n2 + "]";
	}
	////////////////////////////////////////////////////////////////////////////
	public TreeSet ends() {
		return nodes;
	}
	////////////////////////////////////////////////////////////////////////////
	public int compareTo(Object o) {
		Edge other = null;
		try {
			other = (Edge) o;
		} catch (ClassCastException x) { /**/
			System.out.println("class cast problem with edges");
		}
		Iterator thisIt = nodes.iterator();
		Iterator otherIt = other.ends().iterator();
		int n = ((Comparable) (thisIt.next())).compareTo(otherIt.next());
		if (n != 0) {
			return n;
		} else {
			return ((Comparable) (thisIt.next())).compareTo(otherIt.next());
		}
	}
	////////////////////////////////////////////////////////////////////
	protected static void startAgain() {
		edgeLibrary = new TreeSet();
		alternateLibrary = new TreeSet();
	}
	////////////////////////////////////////////////////////////////////
	public Edge substitute(
		Node oldNode,
		Node newNode) { //System.out.println("edge "+this.hashCode());
		Iterator i = nodes.iterator();
		Node n1 = (Node) (i.next());
		//System.out.println("has node 1 is "+n1);
		Node n2 = (Node) (i.next());
		//System.out.println("and node 2 is "+n2);
		if (n1.label1().equals(oldNode.label1())) {
			//System.out.println(""+n1+"="+oldNode+" making a new edge between "+newNode+" and "+n2);
			return new Edge(newNode, n2);
		} else { //System.out.println(""+n1+"!="+oldNode+" making a new edge between "+n1+" and "+newNode);
			return new Edge(n1, newNode);
		}
	}
	/////////////////////////////////////////////////////////////////////
	public void setLabel(Comparable c) {
		label = c;
	}
	/////////////////////////////////////////////////////////////////////
	public Comparable label() {
		return label;
	}

/*	public static void main(String args[]) {
		TreeSet label;
		Node n, n2;
		Edge e, e2;
		label = new TreeSet();
		label.add("a");
		n = Node.getNode(label);
		label = new TreeSet();
		label.add("a");
		label.add("b");
		n2 = Node.getNode(label);
		e = Edge.getEdge(n, n2); // got an edge from a to ab
		label = new TreeSet();
		label.add("a");
		n = Node.getNode(label);
		label = new TreeSet();
		label.add("a");
		label.add("b");
		n2 = Node.getNode(label);
		e2 = Edge.getEdge(n, n2); // got an edge from a to ab
		System.out.println(
			"edge hashCodes are " + e.hashCode() + " " + e2.hashCode());
		label = new TreeSet();
		label.add("a");
		n = Node.getNode(label);
		label = new TreeSet();
		label.add("a");
		label.add("b");
		n2 = Node.getNode(label);
		e2 = Edge.getEdge(n2, n); // got an edge from ab to a
		System.out.println(
			"edge hashCodes are " + e.hashCode() + " " + e2.hashCode());

	}*/
}
