package rwd.views.graph;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Iterator;
/**
 * A FaceCycle is a cyclic sequence of directed edges.
**/
public class FaceCycle { ///////////////////////////////////////////////////////////////////////////
	public static Vector faceCycles(Graph g) {
		Vector result = null; // will contain FaceCycle objects
		result = new Vector();
		TreeSet context = DirectedEdge.getAllDirectedEdges(g);
		while (context.size() > 0) {
			DirectedEdge de = (DirectedEdge) (context.iterator().next());
			DirectedEdge firstde = de;
			//context.remove(de);
			//System.out.println("start at "+de);
			Vector thisCycle = new Vector();
			thisCycle.add(de);
			boolean cycleIsComplete = false;
			while (!cycleIsComplete) { //System.out.println("about to turn left from "+de);
				//System.out.println("in context "+context);
				DirectedEdge nextEdge = de.turnLeft(context);
				//System.out.println("and got edge "+nextEdge);
				context.remove(nextEdge);
				de = nextEdge;
				if (nextEdge == firstde) {
					cycleIsComplete = true;
				} else {
					thisCycle.add(nextEdge);
				}
			} // completed cycle? loop if not
			//System.out.println("Vector of edges is "+thisCycle);
			FaceCycle faceCyc = new FaceCycle(thisCycle);
			result.add(faceCyc);
			//System.out.println("faceCycles() is returning "+result);
		} // added all cycles?
		return result;
	}
	/////////////////ATTRIBUTE/////////////////////////////////////////////////
	public Vector directedEdges;
	/////////////////CONSTRUCTOR//////////////////////////////////////////////
	private FaceCycle(Vector directedEdges) {
		this.directedEdges = directedEdges;
	}
	////////////////////////////////////////////////////////////////////////////
	private TreeSet labels() {
		TreeSet result = new TreeSet();
		Iterator i = directedEdges.iterator();
		while (i.hasNext()) {
			DirectedEdge de = (DirectedEdge) (i.next());
			Comparable c = de.label();
			result.add(c);
		}
		return result;
	}
	////////////////////////////////////////////////////////////////////////////
	public boolean passesWordConditions() {
		TreeSet labels = labels();
		//System.out.println("got labels:"+labels);
		Vector loops = getLoops();
		Iterator loopsIt = loops.iterator();
		while (loopsIt.hasNext()) {
			Vector loop = (Vector) (loopsIt.next());
			boolean foundXYX = true;
			while (foundXYX) {
				foundXYX = false;
				Iterator labelIt = labels.iterator();
				//System.out.println("loop is "+loop);
				while (labelIt.hasNext()) {
					Comparable label = (Comparable) (labelIt.next());
					//System.out.println("label is "+label);
					DirectedEdge firstEdge = null;
					int first = 0;
					Iterator loopIt = loop.iterator();
					boolean foundFirst = false;
					while (foundFirst == false && loopIt.hasNext()) {
						first++;
						DirectedEdge edge = (DirectedEdge) (loopIt.next());
						Comparable loopElt = edge.label();
						if (loopElt == label) {
							foundFirst = true;
							firstEdge = edge;
						}
					}
					int second = first;
					DirectedEdge secondEdge = null;
					boolean foundSecond = false;
					while (foundSecond == false && loopIt.hasNext()) {
						second++;
						DirectedEdge edge = (DirectedEdge) (loopIt.next());
						Comparable loopElt = edge.label();
						if (loopElt == label) {
							foundSecond = true;
							secondEdge = edge;
						}
					}
					//System.out.println("positions are "+first+" and "+second);
					if (second - first == 2
						|| first + second == loop.size()) {
						//System.out.println("found removable "+label);
						//System.out.println("removing "+firstEdge+","+secondEdge);
						loop.remove(firstEdge);
						loop.remove(secondEdge);
						//System.out.println("leaving "+loop);
						foundXYX = true;
					} // end of seeing if label-pair can be removed
				} // label loop
				//System.out.println("end of label loop");
			}
			// now we cannot find XYX
			if (loop.size() != 2) {
				return false;
			}
		}
		return true;
	}

	////////////////////////////////////////////////////////////////////////////
	private Vector getLoops() {
		Vector loops = new Vector();
		Vector dEdgesCopy = (Vector) (directedEdges.clone());
		while (dEdgesCopy.size() > 0) {
			Vector loop = new Vector();
			DirectedEdge firstDe =
				(DirectedEdge) (dEdgesCopy.iterator().next());
			loop.add(firstDe);
			dEdgesCopy.remove(firstDe);
			DirectedEdge lastAddedDe = firstDe;
			while (lastAddedDe.to() != firstDe.from()) {
				Iterator i = dEdgesCopy.iterator();
				getEdge : while (i.hasNext()) {
					DirectedEdge addedDe = (DirectedEdge) (i.next());
					if (addedDe.from() == lastAddedDe.to()) {
						lastAddedDe = addedDe;
						loop.add(lastAddedDe);
						dEdgesCopy.remove(lastAddedDe);
						break getEdge;
					} // end of check
				} // end of edge search
			} // finished loop
			loops.add(loop);
		}
		return loops;
	}

	////////////////////////////////////////////////////////////////////////////
	public String toString() {
		return "fc[" + directedEdges + "]";
	}

	/**
	* This only really makes sense for planar cycles - some refactoring
	* needed.  It returns false in uncertain nonplanar circumstances.
	**/
	public boolean isAntiClockWise() {
		double angleSum = 0;
		Iterator it = directedEdges.iterator();
		DirectedEdge firstEdge = (DirectedEdge) (it.next());
		while (it.hasNext()) {
			DirectedEdge nextEdge = (DirectedEdge) (it.next());
			angleSum = angleSum + firstEdge.smallAngleTo(nextEdge);
			firstEdge = nextEdge;
		}
		DirectedEdge lastEdge = firstEdge;
		firstEdge = (DirectedEdge) (directedEdges.iterator().next());
		angleSum = angleSum + lastEdge.smallAngleTo(firstEdge);
		if (angleSum == Math.PI * (directedEdges.size() - 2)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	* A FaceCycle contains a Node "inside" if the Node is placed in the
	* topological interior of the FaceCycle polygon.
	**/
	public boolean containsStrictlyInside(Node n) {
		double angleSum = 0;
		Iterator i = directedEdges.iterator();
		while (i.hasNext()) {
			DirectedEdge de = (DirectedEdge) (i.next());
			double newAngle = de.angleTo(new DirectedEdge(de.to(), n));
			angleSum = angleSum + newAngle;
			//System.out.println("from "+de+" to "+n+" is "+newAngle);
		}
		//System.out.println("angle sum is "+angleSum);
		if (angleSum > Math.PI * (directedEdges.size() - 2)) {
			return true;
		} else {
			return false;
		}
	}
	/**
	 * A FaceCycle contains a Node "inside" if the Node is placed in the
	 * topological interior of the FaceCycle polygon, or on the boundary
	 * Edges or Nodes.
	**/
	public boolean containsInside(Node n) {
		if (containsOnBoundary(n)) {
			return true;
		}
		if (!isAntiClockWise()) {
			return !reverseFaceCycle().containsStrictlyInside(n);
		} else {
			return containsStrictlyInside(n);
		}
	}
	/**
	 * A FaceCycle contains a Node "inside" if the Node is placed in the
	 * topological interior of the FaceCycle polygon, or on the boundary
	 * Edges or Nodes.
	**/ //////////////////???????????????????????????????????????????????????
	public boolean containsInsideAll(TreeSet ns) {
		Iterator i = ns.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			//System.out.println("does this cycle contain "+n+"?");
			if (!containsInside(n)) { //System.out.println("no");
				return false;
			}
			//System.out.println("yes");
		}
		return true;
	}
	////////////////////////////////////////////////////////////////////////
	/**
	 * A FaceCycle contains a Node "on the boundary" if the Node is placed
	 * on the boundary Edges or Nodes.
	**/
	public boolean containsOnBoundary(Node n) {
		Iterator i = directedEdges.iterator();
		while (i.hasNext()) {
			DirectedEdge de = (DirectedEdge) (i.next());
			if (de.crosses(n)) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Reverses the order of the DirectedEdges and their orientation.
	 **/
	public FaceCycle reverseFaceCycle() {
		Vector reversed = new Vector();
		Iterator i = directedEdges.iterator();
		while (i.hasNext()) {
			DirectedEdge de = (DirectedEdge) (i.next());
			reversed.add(0, new DirectedEdge(de.to(), de.from()));
		}
		return new FaceCycle(reversed);
	}
}
