package rwd.views.smoother;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.Event;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public abstract class AbstractHillClimber 
	extends Manipulator 
{
	public AbstractHillClimber(
		List checkers,
		ConcreteUnitaryDiagram cud,
		Vector metrics,
		DiagramViewerInterface viewer,
		boolean moveContours,
		boolean moveFeet,
		boolean isAnimated) 
	{
		super(checkers, cud, metrics, viewer, moveContours, moveFeet, isAnimated);		
	}
	
	// Notify observers
	protected void notifyObservers(DiagramItem item, boolean isSmoothed, boolean animated) {
		cud.refreshZones(isSmoothed);
		cud.setChanged();
		if (animated) {
			cud.notifyObservers(Event.CHANGE_ITEM);
		} else {
			cud.notifyObservers(Event.CHANGE_ITEM_NO_ANIMATION);
		}
		item.setChanged();
		item.notifyObservers();		
	}
	
	// Common methods
	/**
	 * Sets the points in polygon to be the points in the Vector.
	 */
	public static Vector getPolygonPoints(Polygon p) {
		Vector ret = new Vector();
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;
		for (int posNumber = 0;	posNumber < p.npoints;	posNumber++) { 
			// do something to polys
			int[] point = new int[2];
			point[0] = xs[posNumber];
			point[1] = ys[posNumber];

			ret.add(point);
		}
		return ret;
	}
	
	/**
	 * Sets the points in the Vector to be the points of the
	 * polygon. This method does no checking that the array list
	 * has the correct number of points for the shape.
	 */
	public static void setPolygonPoints(Vector list, Polygon p) {
		int count = 0;
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;

		Iterator li = list.iterator();
		while (li.hasNext()) {
			int[] point = (int[]) li.next();
			xs[count] = point[0];
			ys[count] = point[1];
			count++;
		}
		// Delete cached data in Polygon
		p.invalidate();		
	}

	/**
	 * Changes all points in the polygon by the given x and y values
	 */
	public static void movePolygonPoints(Polygon p, int dx, int dy) {
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;
		for (int posNumber = 0;	posNumber < p.npoints;	posNumber++) { 
			// Move points
			xs[posNumber] = xs[posNumber] + dx;
			ys[posNumber] = ys[posNumber] + dy;
		}
		// Delete cached data in Polygon
		p.invalidate();
	}

	/**
	 * Changes all points in the arraylist by the given x and y values
	 */
	public static void moveListPoints(Vector list, int dx, int dy) {
		Iterator li = list.iterator();
		while (li.hasNext()) {
			int[] point = (int[]) li.next();
			point[0] = point[0] + dx;
			point[1] = point[1] + dy;
		}
	}
}
