package rwd.views.smoother;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;

import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;

import rwd.gui.editor.*;
import rwd.gui.editor.viewers.*;

import rwd.metrics.*;

import rwd.views.abstracttoconcrete.forcemodel.SpiderPlacer;

public class ComparisonParameterWindow
	extends AbstractParameterWindow
//	extends MetricSmootherParameterWindow
	implements ActionListener 
{
	public ComparisonParameterWindow(
		EditorFrame frame,
		DiagramViewerComponent dvcModify) {
		super(frame, null, null, "Dynamic Drawing Parameters", 0, true);
	}

	public ComparisonParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram moveCUD,
		ConcreteUnitaryDiagram stillCUD,
		DiagramViewerComponent viewer,
		String title,
		int offset) 
	{
		super(
			frame,
			null,
			null,
			"Dynamic Drawing Parameters " + title,
			0,
			true);

		this.frame = frame;
		this.moveCUD = moveCUD;
		this.stillcud = stillCUD;
		this.viewer = viewer;
		this.viewer.draw(moveCUD);

		setupMetrics();
		setupManipulators();
		setupPanels();

		coolBox.setSelected(true);
		loadDefaultParameters();

		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		double maxX = d.getWidth() - this.getWidth();
		this.setLocation((int) maxX, 0);
		setVisible(true);

		footMapping = new SpiderFeetMapper().getFootMapping(stillCUD, moveCUD);
		//System.out.println("about to compare legs of spiders");
		//System.out.println("mapping is " + footMapping);
		// trash the legs in moveCUD of spiders which have partner spiders in stillCUD
		// then use legs in stillCUD to rebuild legs in moveCUD

		//iterate through spiders in moveCUD and consider trashing the edges
		Iterator spiderIt = moveCUD.getConcreteSpiders().iterator();
		spiderLoop : while (spiderIt.hasNext()) {
			ConcreteSpider sMove = (ConcreteSpider) spiderIt.next();
			// take a foot and see whether it's been mapped by the mapping
			Iterator feetIt = sMove.getFeet().iterator();
			while (feetIt.hasNext()) {
				ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feetIt.next();
				//System.out.println("foot " + foot + " is in moveCUD");
				// foot is in moveCUD
				ConcreteSpiderFoot partner =
					(ConcreteSpiderFoot) footMapping.get(foot);
				// partner is in stillCUD
				if (partner == null) {
					continue spiderLoop;
				} else {
					//System.out.println(
					//    "its partner " + partner + " is in stillCUD");
					// destroy all legs of sMove
					// and build a new set of legs
					LinkedHashSet newLegSet = new LinkedHashSet();
					// find out the spider in stillCUD which contains the foot called partner
					Iterator spiderItStill =
						stillCUD.getConcreteSpiders().iterator();
					stillSpiderLoop : while (spiderItStill.hasNext()) {
						ConcreteSpider stillSpider =
							(ConcreteSpider) spiderItStill.next();
						Set feetOfThisOne = stillSpider.getFeet();
						if (feetOfThisOne.contains(partner)) {
							// we have the right spider to use to work out the new legs
							// extract its legs one by one
							LinkedHashSet newLegs = new LinkedHashSet();
							Iterator rightLegsIt =
								stillSpider.getLegs().iterator();
							while (rightLegsIt.hasNext()) {
								ConcreteSpiderLeg rightLeg =
									(ConcreteSpiderLeg) rightLegsIt.next();
								// rightLeg is in stillCUD
								ConcreteSpiderFoot oldSource =
									rightLeg.getSource();
								ConcreteSpiderFoot oldTarget =
									rightLeg.getTarget();
								/*System.out.println(
								    "still leg source " + oldSource);
								System.out.println(
								    "still leg target " + oldTarget);*/

								ConcreteSpiderFoot newSource =
									(ConcreteSpiderFoot) footMapping.get(
										oldSource);
								ConcreteSpiderFoot newTarget =
									(ConcreteSpiderFoot) footMapping.get(
										oldTarget);
								/*System.out.println(
								    "moving leg source " + newSource);
								System.out.println(
								    "moving leg target " + newTarget);*/
								// newSource and newTarget are in moveCUD 
								newLegs.add(
									new ConcreteSpiderLeg(
										rightLeg.getLabel(),
										newSource,
										newTarget,
										rightLeg.getType()));
								// this should be a leg in moveCUD
							}
							/*                            System.out.println("trashing " + sMove.getLegs());
							                            Iterator itmp = sMove.getLegs().iterator();
							                            while (itmp.hasNext()) {
							                                ConcreteSpiderLeg csl =
							                                    (ConcreteSpiderLeg) itmp.next();
							                                System.out.println(
							                                    "source : "
							                                        + csl.getSource().hashCode()
							                                        + " and target "
							                                        + csl.getTarget().hashCode());
							                            }*/
							//							viewer.trashSpiderLegs(sMove.getLegs());
							viewer
								.getConcreteDiagram()
								.getFeetAndLegsSet()
								.removeLegs(
								sMove.getLegs());
							sMove.setLegs(newLegs);
							//							viewer.addSpiderLegs(sMove.getLegs());
							viewer
								.getConcreteDiagram()
								.getFeetAndLegsSet()
								.addLegs(
								sMove.getLegs());
							/*System.out.println(
							    "substituting " + sMove.getLegs());
							itmp = sMove.getLegs().iterator();
							while (itmp.hasNext()) {
							    ConcreteSpiderLeg csl =
							        (ConcreteSpiderLeg) itmp.next();
							    System.out.println(
							        "source : "
							            + csl.getSource().hashCode()
							            + " and target "
							            + csl.getTarget().hashCode());
							}*/
							continue spiderLoop;
						} else {
							continue stillSpiderLoop;
						}
					}
					// should never get here - would mean that
					// the "partner" foot is not in any spider in stillCUD
					throw new IllegalArgumentException();
				}
			}
		}
		spiderPlacer =
			new SpiderPlacer(
				(DiagramViewerInterface) viewer,
				footMapping,
				stillCUD);
		spiderPlacer.setAnimated(true);
	}

	protected void loadDefaultParameters() {
		// TBD for comparison
		//        loadSettings(new File(COMPARISON_PROPERTY_FILE_NAME));

	}

	protected void setupMetrics() {
		metrics = new Vector();
		metricBooleans = new Vector();
		metricValues = new Vector();

		// register the Metrics
//		addMetric(new ContourPositionComparisonMetric(moveCUD, 1E4, stillcud), false);
//		addMetric(new ContourPointsDifferenceComparisonMetric(moveCUD, 2E4, stillcud), false);
//		addMetric(new ContourRoundnessMetric(moveCUD, 2E3), true);
//		addMetric(new ContourEdgeLengthMetric(moveCUD, 8E3), true);
//		addMetric(new ContourAreaMetric(moveCUD, 1E2), false);
//		addMetric(new ZoneAreaMetric(moveCUD, 1E0), true);
//		addMetric(new ContourClosenessMetricPts(moveCUD, 20.0), true);
//		addMetric(new ContourClosenessMetricEdgePt(moveCUD, 2000.0), true);
//		addMetric(new DiagramAreaMetric(moveCUD, 0.5E-2), false);

		addMetric(new ContourPositionComparison(moveCUD, 1E4, stillcud), false);
		addMetric(new ContourPointsDifferenceComparison(moveCUD, 2E4, stillcud), false);
		addMetric(new ContourRoundnessAngle(moveCUD, 1E2), true);
		addMetric(new ContourRoundnessSegmentLength(moveCUD, 1E4), true);
		addMetric(new ContourArea(moveCUD, 1E2), false);
		addMetric(new ZoneArea(moveCUD, 2E-1), true);
		addMetric(new ContourClosenessPointPoint(moveCUD, 2E-4), true);
		addMetric(new ContourClosenessSegmentPoint(moveCUD, 4E-6), true);
		addMetric(new DiagramArea(moveCUD, 0.5E-2), false);
		numRows = 9;
		
		addMetric(new NodeNodeSeparation(moveCUD, 1.0), true);
		addMetric(new NodeContourPointSeparation(moveCUD, 1.0), true);
		addMetric(new NodeContourSegmentSeparation(moveCUD, 1.0), false);
		addMetric(new NodeEdgeSeparation(moveCUD, 1.0), true);
		addMetric(new EdgeLength(moveCUD, 1.0), true);
		addMetric(new AngularResolution(moveCUD, 1.0), true);
		addMetric(new EdgeEdgeCrossing(moveCUD, 1.0), true);
		addMetric(new EdgeContourCrossing(moveCUD, 1.0), true);

		//		metrics.add(new ZoneConvexnessMetric(1.0E14));
		//		metricBooleans.add(new Boolean(false));
	}

	protected void setupManipulators() {
		manipulators = new Vector();

		// register the manipulators
		java.util.List checkers = new Vector();
		checkers.add(new ZonePresenceChecker(cud));
		checkers.add(new SpiderChecker(cud));
		manipulators.add(
			new ComparisonFastHillClimber(
				checkers,
				moveCUD,
				metrics,
				viewer,
				MOVE_CONTOURS,
				MOVE_FEET,
				SHOW_STEPS));
		manipulators.add(
			new ComparisonRandomHillClimber(
				new Vector(checkers),
				moveCUD,
				metrics,
				viewer,
				MOVE_CONTOURS,
				MOVE_FEET,
				SHOW_STEPS));
	}

	protected MetricSmoother generateSmoother(
		ContourPolygonSet polys,
		DiagramViewerInterface viewer,
		Manipulator manipulator,
		JLabel progressLabel,
		int iterations) {
		return new ComparisonMetricSmoother(
			moveCUD,
			stillcud,
			viewer,
			manipulator,
			progressLabel,
			iterations);
	}

	protected void setupButtons(JPanel panel) {
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;
		c.anchor = GridBagConstraints.CENTER;

		scaleDownButton = new JButton("Scale Down");
		getRootPane().setDefaultButton(startButton);
		scaleDownButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				scaleDownButton(event);
			}
		});

		c.gridx = 0;
		c.gridy = 0;
		gridbag.setConstraints(scaleDownButton, c);
		panel.add(scaleDownButton);

		scaleUpButton = new JButton("Scale Up");
		scaleUpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				scaleUpButton(event);
			}
		});

		c.gridx = 1;
		c.gridy = 0;
		gridbag.setConstraints(scaleUpButton, c);
		panel.add(scaleUpButton);

		startButton = new JButton("Start");
		getRootPane().setDefaultButton(startButton);
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				startButton(event);
			}
		});

		c.gridx = 2;
		c.gridy = 0;
		gridbag.setConstraints(startButton, c);
		panel.add(startButton);

		stopButton = new JButton("Stop");
		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				stopButton(event);
			}
		});

		c.gridx = 3;
		c.gridy = 0;
		gridbag.setConstraints(stopButton, c);
		panel.add(stopButton);

		iterationsField.requestFocus();

		qualityButton = new JButton("Get Quality");
		qualityButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				qualityButton(event);
			}
		});

		c.gridx = 0;
		c.gridy = 1;
		gridbag.setConstraints(qualityButton, c);
		panel.add(qualityButton);

		bezierButton = new JButton("Bezier");
		bezierButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				bezierButton(event);
			}
		});

		c.gridx = 1;
		c.gridy = 1;
		gridbag.setConstraints(bezierButton, c);
		panel.add(bezierButton);

		loadButton = new JButton("Load Settings");
		loadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				loadButton(event);
			}
		});

		c.gridx = 2;
		c.gridy = 1;
		gridbag.setConstraints(loadButton, c);
		panel.add(loadButton);

		saveButton = new JButton("Save Settings");
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				saveButton(event);
			}
		});

		c.gridx = 3;
		c.gridy = 1;
		gridbag.setConstraints(saveButton, c);
		panel.add(saveButton);

		JButton spiderButton = new JButton("Draw Spiders");
		spiderButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				drawSpiders(event);
			}
		});

		c.gridx = 0;
		c.gridy = 2;
		gridbag.setConstraints(spiderButton, c);
		panel.add(spiderButton);

		JButton animateButton = new JButton("Animate");
		animateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if (animateViewer == null) {
					animateViewer = frame.addViewer("", new ConcreteDiagram());
				}
				animate(frame, animateViewer, moveCUD, stillcud, 1500, 2500);
			}
		});

		c.gridx = 1;
		c.gridy = 2;
		c.gridwidth = 1;
		gridbag.setConstraints(animateButton, c);
		panel.add(animateButton);

	}

	public void startButton(ActionEvent event) {
		if (running) {
			return;
		}
	
		assignValues();
	
		worker = new SwingWorker() {
			public Object construct() {
				running = true;
				statusLabel.setText("RUNNING  ");
				iterationsLabel.setText(iterationsField.getText());
				Metric.NORMALIZE = normalizeBox.isSelected();
	
				Date start = new Date();
				manipulator.viewer.draw(moveCUD);
				int iterations =
					(int) Double.parseDouble(iterationsField.getText());
	
				smoother =
					generateSmoother(
						moveCUD,
						stillcud,
						viewer,
						manipulator,
						progressLabel,
						iterations);
				smoother.run();
	
				Date stop = new Date();
				outputDataArea.append(
					"\nDone in " + (stop.getTime() - start.getTime()) / 1000.0);
				running = false;
				statusLabel.setText(
					"Completed in "
						+ Double.toString(
							((int) (stop.getTime() - start.getTime())
								/ 1000.0))
						+ "s. ");
				return manipulator;
			}
		};
	
		worker.start();
	}

	/**
	 * This is needed because it is overloaded for the Comparison Smoothing as
	 * a different smoother is used.
	 */
	protected ComparisonMetricSmoother generateSmoother(
		ConcreteUnitaryDiagram moveCUD,
		ConcreteUnitaryDiagram stillcud,
		DiagramViewerInterface viewer,
		Manipulator manipulator,
		JLabel progressLabel,
		int iterations) {
		return new ComparisonMetricSmoother(
			moveCUD,
			stillcud,
			viewer,
			manipulator,
			progressLabel,
			iterations);
	}

	protected void drawSpiders(ActionEvent e) {
		if (stillcud == null) {
			return;
		}
		Vector zoneAreas =
			new Vector(
				((DiagramViewerInterface) viewer)
					.getConcreteDiagram()
					.getVisibleConcreteZones());
		HashMap mapping =
			ConcreteUnitaryConverter.mapUnitaryDiagramCalled(
				moveCUD,
				moveCUD.getDecorationBox().getLabel());

		java.util.List spiderFeet =
			((DiagramViewerInterface) viewer)
				.getConcreteDiagram()
				.getFeetAndLegsSet()
				.getFeet();
		ConcreteSpiderFeetAndLegsSet css =
			new ConcreteSpiderFeetAndLegsSet(mapping);
		css.placeNodes(zoneAreas, spiderFeet, true, footMapping);
		spiderPlacer.reinitializeSpiderFeet();
		spiderPlacer.embed();

		viewer.redrawImage();
	}

	/**
	 * the viewer window should be an empty window, or a previous animation window.
	 * fadeSpeed and moveSpeed are milliseconds. The movement step size adjusts to the
	 * speed of the computer, so the times will always be reasonably accurate.
	 */
	protected static void animate(
		EditorFrame frame,
		DiagramViewerComponent viewer,
		ConcreteUnitaryDiagram moveCud,
		ConcreteUnitaryDiagram stillCud,
		int fadeSpeed,
		int moveSpeed) {
		// generate a smoother with zero iterations and run it to set up the mappings
		ComparisonMetricSmoother comparisonSmoother =
			new ComparisonMetricSmoother(moveCud, stillCud, viewer, 0);
		comparisonSmoother.run();

		ConcreteUnitaryDiagram startcud =
			(ConcreteUnitaryDiagram) moveCud.getClone();
		ConcreteUnitaryDiagram endcud =
			(ConcreteUnitaryDiagram) comparisonSmoother
				.getNewStillCud()
				.getClone();

		Set startSpiders = startcud.getConcreteSpiders();
		Set endSpiders = endcud.getConcreteSpiders();

		//System.out.println("startcud "+startcud);
		//System.out.println("endcud "+endcud);

		//System.out.println("startSpiders "+startSpiders);
		//System.out.println("endSpiders "+endSpiders);

		// generate the mappings between spider feet
		HashMap footMapping =
			new SpiderFeetMapper().getFootMapping(startcud, endcud);

		System.out.println("MAPPING: " + footMapping);

		// find the shared contours
		Set startContours = startcud.getConcreteContours();
		Set endContours = endcud.getConcreteContours();

		Vector startSharedContourList = new Vector();
		Vector endSharedContourList = new Vector();
		Iterator i = startContours.iterator();
		while (i.hasNext()) {
			ConcreteContour startcp = (ConcreteContour) i.next();
			Iterator j = endContours.iterator();
			while (j.hasNext()) {
				ConcreteContour endcp = (ConcreteContour) j.next();
				if (startcp.getContour().equals(endcp.getContour())) {
					startSharedContourList.add(startcp);
					endSharedContourList.add(endcp);
				}
			}
		}

		// find the contours in the start, but not the end
		Vector removedContourList = new Vector();
		i = startContours.iterator();
		while (i.hasNext()) {
			boolean inEndList = false;
			ConcreteContour startcp = (ConcreteContour) i.next();
			Iterator j = endContours.iterator();
			while (j.hasNext()) {
				ConcreteContour endcp = (ConcreteContour) j.next();
				if (startcp.getContour().equals(endcp.getContour())) {
					inEndList = true;
				}
			}
			if (!inEndList) {
				removedContourList.add(startcp);
			}
		}

		// find the contours in the end, but not the start
		Vector addedContourList = new Vector();
		Iterator j = endContours.iterator();
		while (j.hasNext()) {
			boolean inStartList = false;
			ConcreteContour endcp = (ConcreteContour) j.next();
			i = startContours.iterator();
			while (i.hasNext()) {
				ConcreteContour startcp = (ConcreteContour) i.next();
				if (startcp.getContour().equals(endcp.getContour())) {
					inStartList = true;
				}
			}
			if (!inStartList) {
				addedContourList.add(endcp);
			}
		}

		Vector removedSpiderList = new Vector();
		Vector addedSpiderList = new Vector();
		Vector startSharedSpiderList = new Vector();
		Vector endSharedSpiderList = new Vector();

		Iterator startIt = startSpiders.iterator();
		while (startIt.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) startIt.next();
			Iterator feetIt = spider.getFeet().iterator();
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feetIt.next();
			if (footMapping.containsKey(foot)) {
				startSharedSpiderList.add(spider);
			} else {
				removedSpiderList.add(spider);
			}
		}

		Iterator endIt = endSpiders.iterator();
		while (endIt.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) endIt.next();
			Iterator feetIt = spider.getFeet().iterator();
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feetIt.next();
			if (footMapping.containsValue(foot)) {
				endSharedSpiderList.add(spider);
			} else {
				addedSpiderList.add(spider);
			}
		}

		//System.out.println("removedSpiderList "+removedSpiderList);
		//System.out.println("startSharedSpiderList "+startSharedSpiderList);
		//System.out.println("endSharedSpiderList "+endSharedSpiderList);
		//System.out.println("addedSpiderList "+addedSpiderList);

		// This colours in the mapping in both original diagrams, shared items are blue
		//colorMapping(removedSpiderList,startSharedSpiderList);
		//colorMapping(addedSpiderList,endSharedSpiderList);

		viewer.setConcreteDiagram(startcud);

		// TDB remove the box

		// fade out the removed polygons and spiders
		if (removedContourList.size() > 0 || removedSpiderList.size() > 0) {
			int stepTime = fadeSpeed / 255;
			int step = 10;
			long startTime = System.currentTimeMillis();
			int ic = 0;
			while (ic < 255) {
				boolean wait = false;
				while (System.currentTimeMillis()
					< startTime + stepTime * step) {
					wait = true;
				}
				if (!wait) {
					step++;
				}

				Color fadeColor = new Color(ic, ic, ic);

				startTime = System.currentTimeMillis();
				Iterator removedContorIt1 = removedContourList.iterator();
				while (removedContorIt1.hasNext()) {
					ConcreteContour cp =
						(ConcreteContour) removedContorIt1.next();
					cp.setForeColor(fadeColor);
					cp.setTextColor(fadeColor);
				}

				Iterator removedSpiderIt1 = removedSpiderList.iterator();
				while (removedSpiderIt1.hasNext()) {
					ConcreteSpider spider =
						(ConcreteSpider) removedSpiderIt1.next();
					colorSpider(spider, fadeColor);
				}

				// Repaint the diagram
				viewer.drawAgain();

				ic += step;
			}
		}

		// remove the contours and spiders not in the new diagram
		Iterator removedContourIt2 = removedContourList.iterator();
		while (removedContourIt2.hasNext()) {
			ConcreteContour cp = (ConcreteContour) removedContourIt2.next();
			startContours.remove(cp);
			startcud.removeConcreteContour(cp);
		}
		Iterator removedSpiderIt2 = removedSpiderList.iterator();
		while (removedSpiderIt2.hasNext()) {
			ConcreteSpider cs = (ConcreteSpider) removedSpiderIt2.next();
			startSpiders.remove(cs);
			removeSpider(startcud, cs);
		}
		viewer.drawAgain();

		// generate step size for each contour
		int numberOfSteps = 1000;
		Vector contourStepList = new Vector();
		for (int ic = 0; ic <= startSharedContourList.size() - 1; ic++) {
			ConcreteContour startcp =
				(ConcreteContour) startSharedContourList.get(ic);
			ConcreteContour endcp =
				(ConcreteContour) endSharedContourList.get(ic);
			contourStepList.add(new ContourStep(startcp, endcp, numberOfSteps));
		}

		// generate step size for each spider foot
		Vector footStepList = new Vector();
		Iterator sharedSpiderIt = startSharedSpiderList.iterator();
		while (sharedSpiderIt.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) sharedSpiderIt.next();

			Vector feet = new Vector(spider.getFeet());
			for (int iFeet = 0; iFeet < feet.size(); iFeet++) {
				ConcreteSpiderFoot startFoot =
					(ConcreteSpiderFoot) feet.get(iFeet);
				if (footMapping.containsValue(startFoot)) {
					ConcreteSpiderFoot endFoot =
						(ConcreteSpiderFoot) footMapping.get(startFoot);
					footStepList.add(
						new FootStep(startFoot, endFoot, numberOfSteps));
				} else {
					System.out.println(
						"In Animate: Foot mapping "
							+ startFoot
							+ " does not have a mapping");
				}
			}
		}

		// do the animation
		if (startSharedContourList.size() > 0
			|| startSharedSpiderList.size() > 0) {
			int stepTime = moveSpeed / numberOfSteps;
			int step = 20;
			long startTime = System.currentTimeMillis();
			int ic = 0;
			while (ic <= numberOfSteps) {
				boolean wait = false;
				while (System.currentTimeMillis()
					< startTime + stepTime * step) {
					wait = true;
				}
				if (!wait) {
					step++;
				}
				startTime = System.currentTimeMillis();

				Iterator contourStepIt1 = contourStepList.iterator();
				while (contourStepIt1.hasNext()) {
					ContourStep cs = (ContourStep) contourStepIt1.next();
					cs.doStep(step);
					cs.startcp.autoPlaceLabel();
				}

				Iterator footStepIt1 = footStepList.iterator();
				while (footStepIt1.hasNext()) {
					FootStep footStep = (FootStep) footStepIt1.next();
					footStep.doStep(footStep.startFoot, step);
				}

				// Repaint the diagram
				viewer.drawAgain();

				ic += step;
			}
		}

		// ensure the points are exactly correct
		Iterator stepIt2 = contourStepList.iterator();
		while (stepIt2.hasNext()) {
			ContourStep cs = (ContourStep) stepIt2.next();
			cs.finalizePoints();
			cs.startcp.autoPlaceLabel();
		}

		// Repaint the diagram 
		viewer.drawAgain();

		// add the contours and spiders in the new diagram
		Iterator addedContourIt1 = addedContourList.iterator();
		while (addedContourIt1.hasNext()) {
			ConcreteContour cp = (ConcreteContour) addedContourIt1.next();
			viewer.getConcreteDiagram().addConcreteContour(cp);
			startContours.add(cp);
		}

		Iterator addedSpiderIt1 = addedSpiderList.iterator();
		while (addedSpiderIt1.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) addedSpiderIt1.next();
			//add the feet
			Iterator feetIt = spider.getFeet().iterator();
			while (feetIt.hasNext()) {
				ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feetIt.next();
				viewer.getConcreteDiagram().addConcreteFoot(foot);
			}
			//add the legs
			Iterator legsIt = spider.getLegs().iterator();
			while (legsIt.hasNext()) {
				ConcreteSpiderLeg leg = (ConcreteSpiderLeg) legsIt.next();
				viewer.getConcreteDiagram().addConcreteLeg(leg);
			}
			startSpiders.add(spider);
		}

		// fade in the new polygons
		if (addedContourList.size() > 0 || addedSpiderList.size() > 0) {
			int stepTime = fadeSpeed / 255;
			int step = 10;
			long startTime = System.currentTimeMillis();
			int ic = 255;
			while (ic > 0) {
				boolean wait = false;
				while (System.currentTimeMillis()
					< startTime + stepTime * step) {
					wait = true;
				}
				if (!wait) {
					step++;
				}

				Color fadeColor = new Color(ic, ic, ic);

				startTime = System.currentTimeMillis();
				Iterator addedContourIt2 = addedContourList.iterator();
				while (addedContourIt2.hasNext()) {
					ConcreteContour cp =
						(ConcreteContour) addedContourIt2.next();
					cp.setForeColor(fadeColor);
					cp.setTextColor(fadeColor);
				}

				Iterator addedSpiderIt2 = addedSpiderList.iterator();
				while (addedSpiderIt2.hasNext()) {
					ConcreteSpider spider =
						(ConcreteSpider) addedSpiderIt2.next();
					colorSpider(spider, fadeColor);
				}

				// Repaint the diagram
				viewer.drawAgain();

				ic -= step;
			}

			Color blackColor = new Color(0, 0, 0);

			Iterator addedContourIt3 = addedContourList.iterator();
			while (addedContourIt3.hasNext()) {
				ConcreteContour cp = (ConcreteContour) addedContourIt3.next();
				cp.setForeColor(blackColor);
				cp.setTextColor(blackColor);
			}

			Iterator addedSpiderIt3 = addedSpiderList.iterator();
			while (addedSpiderIt3.hasNext()) {
				ConcreteSpider spider = (ConcreteSpider) addedSpiderIt3.next();
				colorSpider(spider, blackColor);
			}

			// Repaint the diagram
			viewer.drawAgain();
		}

	}

	private static void removeSpider(
		ConcreteUnitaryDiagram cud,
		ConcreteSpider spider) {

		Vector legs = new Vector(spider.getLegs());
		Vector feet = new Vector(spider.getFeet());

		for (int i = 0; i < legs.size(); i++) {
			cud.removeConcreteLeg((ConcreteSpiderLeg) legs.get(i));
		}

		for (int i = 0; i < feet.size(); i++) {
			cud.removeConcreteFoot((ConcreteSpiderFoot) feet.get(i));
		}

	}

	private static void addSpider(
		ConcreteUnitaryDiagram cud,
		ConcreteSpider spider) {

		Vector legs = new Vector(spider.getLegs());
		Vector feet = new Vector(spider.getFeet());

		for (int i = 0; i < feet.size(); i++) {
			cud.addConcreteFoot((ConcreteSpiderFoot) feet.get(i));
		}

		for (int i = 0; i < legs.size(); i++) {
			cud.addConcreteLeg((ConcreteSpiderLeg) legs.get(i));
		}
	}

	public static void colorSpider(ConcreteSpider spider, Color color) {
		Vector legs = new Vector(spider.getLegs());
		for (int i = 0; i < legs.size(); i++) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg) legs.get(i);
			leg.setForeColor(color);
		}

		Vector feet = new Vector(spider.getFeet());
		for (int i = 0; i < feet.size(); i++) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feet.get(i);
			foot.setBackColor(color);
		}
	}

	/**
	 * For debugging
	 */
	public static void colorMapping(Vector spiders1, Vector spiders2) {
		Iterator test1It = spiders1.iterator();
		while (test1It.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) test1It.next();
			Color color = Color.red;
			colorSpider(spider, color);
		}

		Iterator test2It = spiders2.iterator();
		while (test2It.hasNext()) {
			ConcreteSpider spider = (ConcreteSpider) test2It.next();
			Color color = Color.blue;
			colorSpider(spider, color);
		}
	}

	// Local properties
	protected DiagramViewerComponent animateViewer = null;
	protected Set stillcps;
	protected ConcreteUnitaryDiagram stillcud = null;
	protected ConcreteUnitaryDiagram moveCUD = null;
	protected HashMap footMapping = null;
	protected SpiderPlacer spiderPlacer = null;

	protected String COMPARISON_PROPERTY_FILE_NAME = "comparison.parametersettings";
}
