package rwd.views.smoother;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.swing.JLabel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.metrics.AreaPolygonAdapter;
import rwd.metrics.DiagramArea;

public class RandomHillClimber
	extends AbstractHillClimber 
{
    public RandomHillClimber(
        List checkers,
        ConcreteUnitaryDiagram cud,
        Vector metrics,
        DiagramViewerInterface viewer,
		boolean moveContours,
		boolean moveFeet,
        boolean isAnimated) {
        super(checkers, cud, metrics, viewer, moveContours, moveFeet, isAnimated);
    }

    protected void makeChange(int numberOfSteps, JLabel changeLabel) {
        setStopFlag(false);
        Set contourPolys = cud.getConcreteContours();
		Quality quality = currentQuality();
        Random r = new Random();
        double move = maxMoveContourDistance;

        double diagramArea = DiagramArea.getDiagramArea(cud);

        for (int j = 1; j <= numberOfSteps; j++) {
            if (stopFlag) {
                viewer.draw(cud);
                return;
            }
            if (changeLabel != null) {
                changeLabel.setText(Integer.toString(j));
            }
            if (outputData) {
                /**/
                System.out.println("ITERATION in " + getClass() + ": " + j);
            }
            
            if (moveContours)
            	moveContours(cud.getConcreteContours(), diagramArea, move, quality, r);

            if (cooling) {
                move = move - (maxMoveContourDistance / numberOfSteps);
            }
            
            if (animated) {
            	viewer.redrawImage();
            }
        }
    }
    
    protected void moveContours(Set contours, double diagramArea, double move, Quality quality, Random r) {
		Iterator k = contours.iterator();
		while(k.hasNext()){
			ConcreteContour cp = (ConcreteContour) k.next();
			Polygon p = cp.getPolygon();

			double pArea = AreaPolygonAdapter.getAreaOf(p);
			// this allows the scaling of movement according to the size of contours
			double sizeFactor = Math.sqrt(pArea / diagramArea);
			double adjustedMove = move * sizeFactor;

			int[] xs = p.xpoints;
			int[] ys = p.ypoints;
			for (int posNumber = 0; posNumber < p.npoints; posNumber++) {
				// do something to polys
				int oldX = xs[posNumber];
				int oldY = ys[posNumber];

				double moveX = adjustedMove - (r.nextDouble() * move * 2);
				double moveY = adjustedMove - (r.nextDouble() * move * 2);

				xs[posNumber] += moveX;
				ys[posNumber] += moveY;
                    
				notifyObservers(cp, true, false);
                    
				Quality newQuality = currentQuality();

				if (structureStillOK() && newQuality.compare(quality) <= 0) {
					quality = newQuality;
				} else {
					xs[posNumber] = oldX;
					ys[posNumber] = oldY;
				}

				if (animated) {
					viewer.draw(cud);
				}
			}

			if (contourMove) {
				Vector oldPoints = getPolygonPoints(p);
				double moveX = adjustedMove - (r.nextDouble() * move * 2);
				double moveY = adjustedMove - (r.nextDouble() * move * 2);

				movePolygonPoints(p, (int) moveX, (int) moveY);

				notifyObservers(cp, true, false);

				Quality newQuality = currentQuality();
				if (structureStillOK() && newQuality.compare(quality) < 0) {
					quality = newQuality;
				} else {
					setPolygonPoints(oldPoints, p);
				}

				if (animated) {
					viewer.draw(cud);
				}
			}
			cp.autoPlaceLabel();
		}
    }
}
